Imports System
Imports System.ComponentModel
Imports System.Workflow.ComponentModel
Imports ESB.Core.XMLSchemaClasses

Namespace Xslt
    Public Class XsltActivity
        Inherits EsbWorkflowActivityBase

#Region "Activity Property Exposure Registration Statements - One for each of the Workflow Activity Properties to be exposed to the Designer"
        Public Shared ResolveNameProperty As DependencyProperty = DependencyProperty.Register("ResolveName", GetType(System.String), GetType(XsltActivity))
        Public Shared ContextProperty As DependencyProperty = DependencyProperty.Register("Context", GetType(System.String), GetType(XsltActivity))
        Public Shared XsltPathProperty As DependencyProperty = DependencyProperty.Register("XsltPath", GetType(System.String), GetType(XsltActivity))
        Public Shared XMLStringToTransformProperty As DependencyProperty = DependencyProperty.Register("XMLStringToTransform", GetType(System.String), GetType(XsltActivity))
        Public Shared TransformResultProperty As DependencyProperty = DependencyProperty.Register("TransformResult", GetType(System.String), GetType(XsltActivity))
        Public Shared EnvelopeProperty As DependencyProperty = DependencyProperty.Register("Envelope", GetType(IEnvelope), GetType(XsltActivity))
        Public Shared DocumentIndexProperty As DependencyProperty = DependencyProperty.Register("DocumentIndex", GetType(Integer), GetType(XsltActivity))
        Public Shared DocumentNameProperty As DependencyProperty = DependencyProperty.Register("DocumentName", GetType(System.String), GetType(XsltActivity))

#End Region
#Region "Activity Designer category Constants"
        ' Define constant values for the Property categories.  
        Private Const ESBServiceParametersCategory As String = "ESB Service in/out Parameters"
        Private Const ESBRuntimeCategory As String = "ESB Runtime"
        Private Const ESBActivityInvocationParametersCategory As String = "ESB Activity Invocation Parameters"
#End Region
#Region "Workflow Activity Properties"
        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
       <BrowsableAttribute(True)> _
       <DescriptionAttribute("The ResolveName that forms part of the key used to lookup the Mapping Pipeline which specifies the XSLT to use.")> _
       <CategoryAttribute(ESBActivityInvocationParametersCategory)> _
       Public Property ResolveName() As String
            Get
                Return CType(MyBase.GetValue(XsltActivity.ResolveNameProperty), String)
            End Get
            Set(ByVal value As String)
                MyBase.SetValue(XsltActivity.ResolveNameProperty, value)
            End Set
        End Property

        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
       <BrowsableAttribute(True)> _
       <DescriptionAttribute("The Context that forms part of the key used to lookup the Mapping Pipeline which specifies the XSLT to use.")> _
       <CategoryAttribute(ESBActivityInvocationParametersCategory)> _
        Public Property Context() As String

            Get
                Return CType(MyBase.GetValue(XsltActivity.ContextProperty), String)
            End Get
            Set(ByVal value As String)
                MyBase.SetValue(XsltActivity.ContextProperty, value)
            End Set
        End Property

        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
        <BrowsableAttribute(True)> _
        <DescriptionAttribute("A relative (to the ESB installation directory) or absolute path which specifies which XSLT file to use.")> _
        <CategoryAttribute(ESBServiceParametersCategory)> _
        Public Property XsltPath() As System.String
            Get
                Return CType(MyBase.GetValue(XsltActivity.XsltPathProperty), System.String)
            End Get
            Set(ByVal value As System.String)
                MyBase.SetValue(XsltActivity.XsltPathProperty, value)
            End Set
        End Property

        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
        <BrowsableAttribute(True)> _
        <DescriptionAttribute("The Request XML String to transform.")> _
        <CategoryAttribute(ESBServiceParametersCategory)> _
        Public Property XMLStringToTransform() As System.String
            Get
                Return CType(MyBase.GetValue(XsltActivity.XMLStringToTransformProperty), System.String)
            End Get
            Set(ByVal value As System.String)
                MyBase.SetValue(XsltActivity.XMLStringToTransformProperty, value)
            End Set

        End Property

        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
        <BrowsableAttribute(True)> _
        <DescriptionAttribute("The result of the XSLT Transform operation.")> _
        <CategoryAttribute(ESBRuntimeCategory)> _
        Public Property TransformResult() As String
            Get
                Return CType(MyBase.GetValue(XsltActivity.TransformResultProperty), String)
            End Get
            Set(ByVal value As String)
                MyBase.SetValue(XsltActivity.TransformResultProperty, value)
            End Set
        End Property

        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
<BrowsableAttribute(True)> _
<DescriptionAttribute("The ResolveName that forms part of the key used to lookup the Mapping Pipeline which specifies the XSLT to use.")> _
<CategoryAttribute(ESBActivityInvocationParametersCategory)> _
Public Property Envelope() As IEnvelope
            Get
                Return CType(MyBase.GetValue(XsltActivity.EnvelopeProperty), IEnvelope)
            End Get
            Set(ByVal value As IEnvelope)
                MyBase.SetValue(XsltActivity.EnvelopeProperty, value)
            End Set
        End Property
        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
<BrowsableAttribute(True)> _
<DescriptionAttribute("The ResolveName that forms part of the key used to lookup the Mapping Pipeline which specifies the XSLT to use.")> _
<CategoryAttribute(ESBActivityInvocationParametersCategory)> _
Public Property DocumentIndex() As Integer
            Get
                Return CType(MyBase.GetValue(XsltActivity.DocumentIndexProperty), Integer)
            End Get
            Set(ByVal value As Integer)
                MyBase.SetValue(XsltActivity.DocumentIndexProperty, value)
            End Set
        End Property

        <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
<BrowsableAttribute(True)> _
<DescriptionAttribute("The ResolveName that forms part of the key used to lookup the Mapping Pipeline which specifies the XSLT to use.")> _
<CategoryAttribute(ESBActivityInvocationParametersCategory)> _
Public Property DocumentName() As String
            Get
                Return CType(MyBase.GetValue(XsltActivity.DocumentNameProperty), String)
            End Get
            Set(ByVal value As String)
                MyBase.SetValue(XsltActivity.DocumentNameProperty, value)
            End Set
        End Property


#End Region
#Region "Workflow Execution Method"
        Protected Overrides Function Execute(ByVal executionContext As System.Workflow.ComponentModel.ActivityExecutionContext) As System.Workflow.ComponentModel.ActivityExecutionStatus
            StartMethod()
            'if an XSLT path is specified, then use it

            If Envelope Is Nothing Then
                If XsltPath <> "" Then
                    GetContext.Transform(ESB.Core.Util.Utils.ExpandRelativePath(XsltPath), XMLStringToTransform, TransformResult)
                Else
                    'otherwise look for matching ResolveName and Context entries - both of which can be empty...
                    GetContext.Transform(ResolveName, Context, XMLStringToTransform, TransformResult)
                End If
            Else
                If XsltPath <> "" Then
                    GetContext.Transform(ESB.Core.Util.Utils.ExpandRelativePath(XsltPath), Envelope.GetDocument(DocumentName, DocumentIndex).GetAsString, TransformResult)
                Else
                    'otherwise look for matching ResolveName and Context entries - both of which can be empty...
                    GetContext.Transform(ResolveName, Context, Envelope.GetDocument(DocumentName, DocumentIndex).GetAsString, TransformResult)
                End If
            End If
            EndMethod()
            Return MyBase.Execute(executionContext)
        End Function
#End Region

    End Class
End Namespace
