﻿Imports System.Activities
Imports ESB.Core.Interfaces.Envelope
Imports ESB.Core.XMLSchemaClasses
Imports System.ComponentModel


Public NotInheritable Class InvokeESBComponentCodeActivity
    Inherits ESBWorkflowActivityBase

    <RequiredArgument(), Description("Class name of Activity to be invoked")> _
    Property ActivityClassName() As InArgument(Of String)

    <RequiredArgument(), Description("Assembly name of Activity to be invoked")> _
    Property ActivityAssemblyName() As InArgument(Of String)

    <RequiredArgument(), Description("Assembly name of Activity to be invoked")> _
    Property EnvRs() As InOutArgument(Of IEnvelope)

    ''Also, do a service invocation...
    'Instance name/URL/alias
    'Transport type
    'Transport profile (WCF only)

    Public Overrides Function ProcessMsgSync(ByVal oEnv As Core.Interfaces.Envelope.IEnvelope) As Core.Interfaces.Envelope.IEnvelope
        MyBase.ProcessMsgSync(oEnv)

        m_oEnvRq = WfContext.GetValue(EnvRq)

        StartMethod()
        InvokeObject(WfContext.GetValue(ActivityClassName), _
                     ESB.Core.Util.Utils.ExpandRelativePath(WfContext.GetValue(ActivityAssemblyName)), _
                     m_oEnvRq, m_oEnvRs)

        EndMethod()

        m_oEnvRs = WfContext.GetValue(EnvRs)
        Return m_oEnvRs

    End Function

#Region "Helper Methods"
    Protected Overloads Sub InvokeObject(ByRef sTransactionComponent As String, ByRef sComponentServer As String, ByRef oEnvRq As IEnvelope, ByRef oEnvRs As IEnvelope)
        Dim sInterfaceName As String
        Dim oTimerCLR As ESB.Core.Util.Timer
        Dim sExecutionTime As String
        Dim lExecutionTime As Double

        'The Interface that takes the Keystroke Generic Envelope object as parameters.
        'This is the preferred interface for local calls as it is object based
        'and the XML Message does not need to be parsed...
        Dim oESBIf As ESB.Core.Interfaces.ProcessMsg.IProcessMsg


        oTimerCLR = New ESB.Core.Util.Timer
        oTimerCLR.StartTimer()


        'This should be a native CLR based object...
        oESBIf = CreateObjectCLR(sTransactionComponent, sComponentServer)
        If ShouldLog(LogDefs.eLogLevel.eLOG_ACTIVITY_HANDLER_INVOKED) Then : LogMsg(LogDefs.eLogLevel.eLOG_ACTIVITY_HANDLER_INVOKED, sComponentServer & "." & sTransactionComponent) : End If
        If oESBIf Is Nothing Then
            'Error
            Dim sError As String
            sError = "Attempted to create object:" & sTransactionComponent & " in Assembly:" & sComponentServer & " Failed. "
            LogError(eLogLevel.eLOG_ERROR, sError)
            Throw New Exception(sError)
        Else
            'Everything OK- Object created successfully

            If ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then : LogMsg(LogDefs.eLogLevel.eLOG_TRACE, "Successfully created CLR based component ==>" & sTransactionComponent & "<== on Server==>" & sComponentServer & "<==" & "Resolve name sought was: .") : End If
            If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(LogDefs.eLogLevel.eLOG_DATAXFER, "InvokeObjectCLR sending:sXMLrq=" & oEnvRq.XML) : End If
            oEnvRs = InvokeObjectCLR(oESBIf, oEnvRq)
            If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(LogDefs.eLogLevel.eLOG_DATAXFER, "InvokeObjectCLR returned:sXMLrs=" & oEnvRs.XML) : End If
            oTimerCLR.StopTimer()
            sInterfaceName = "CLR"
            sExecutionTime = oTimerCLR.Duration
            lExecutionTime = oTimerCLR.ElapsedTime
        End If

        LogTiming(eLogLevel.eLOG_TIMING, "Pipeline Executer - Execution time on interface:" & sInterfaceName & "=" & sExecutionTime)


    End Sub
    Private Function CreateObjectCLR(ByRef sTransactionComponent As String, ByRef sComponentServer As String) As ESB.Core.Interfaces.ProcessMsg.IProcessMsg
        StartMethod()
        'try & create a .NET object first...
        Dim oESBIf As ESB.Core.Interfaces.ProcessMsg.IProcessMsg
        Try
            oESBIf = DirectCast(ESBClassFactory.CreateObjectByStringName(sTransactionComponent, sComponentServer), ESB.Core.Interfaces.ProcessMsg.IProcessMsg)
            Return oESBIf
        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(eLogLevel.eLOG_ERROR, "error creating object from :" & "sTransactionComponent=" & sTransactionComponent & "sComponentServer(Assembly file path) = " & sComponentServer & ". Error=" & ex.ToString)
            Return Nothing
        Finally
            EndMethod()
        End Try
    End Function
    Private Function InvokeObjectCLR(ByRef oESBIf As ESB.Core.Interfaces.ProcessMsg.IProcessMsg, ByRef oEnvRq As IEnvelope) As IEnvelope
        'try & create a .NET object first...
        StartMethod()
        Dim oEnvRs As IEnvelope = Nothing
        Try
            oEnvRs = oESBIf.ProcessMsgSync(oEnvRq)
            Return oEnvRs
        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(eLogLevel.eLOG_ERROR, ex.ToString)

            'better return something...???
            If oEnvRs Is Nothing Then
                oEnvRs = oEnvRq.CreateResponse()


                'TODO: (MINA) Add this functionality to the ErrorXML object or something...
                Dim oErrXML As New ErrorXML.ErrorXML
                Dim oErrEntry As New ErrorXML.ErrorEntry
                oErrEntry.Description = ex.ToString
                oErrEntry.Source = ex.Source
                oErrXML.ErrorEntry.Add(oErrEntry)
                oEnvRs.AddDocument(New InlineDocument(oErrXML))
            End If
            Return oEnvRs
        Finally
            EndMethod()
        End Try
    End Function
#End Region



End Class

