'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
'Base class for ESB components

Imports ESB.Core.Services.Logger
Imports ESB.Core.XMLSchemaClasses.ErrorXML
Imports ESB.Core.XMLSchemaClasses.ESBEnvelope
Imports ESB.Core.Services
Imports ESB.Core.Interfaces.ServerContext
Imports Microsoft.ApplicationBlocks.ConfigurationManagement
Imports System
Imports System.ComponentModel
Imports System.Activities
Imports ESB.Core.Interfaces.Envelope
Imports ESB.Core.XMLSchemaClasses

Public Class ESBWorkflowActivityWithBookmarkBase
    Inherits NativeActivity(Of IEnvelope)  'Define the activity output argument as being of type IEnvelope
    '    Inherits NativeActivity(Of IEnvelope)  'Define the activity output argument as being of type IEnvelope
    Implements ESB.Core.Interfaces.ProcessMsg.IProcessMsg

    'Define an activity input argument of type IEnvelope
    <RequiredArgument(), Description("ESB request envelope object interface")> _
    Property EnvRq() As InOutArgument(Of IEnvelope)

    Dim m_oServerContext As ESB.Core.Interfaces.ServerContext.IServerContext

    Protected m_oCfgData As CustomConfigurationData

    Protected m_oEnvRq As IEnvelope
    Protected m_oEnvRs As IEnvelope

    Protected m_oErr As ErrorXML
    Protected m_oErrEntry As ErrorEntry
    Protected m_oWarningEntry As WarningEntry

    Private m_oContext As ServerContext

    Property WfContext As System.Activities.NativeActivityContext


    Public Overridable Sub DoWork()

        m_oEnvRs.SetCustomParam("WF4 Custom Param", "Ola!") 'gets stripped at ESB perimeter

        m_oEnvRs.AddDocument(New InlineDocument("<Ola/>"))
        m_oEnvRs.AddDocument(New InlineDocument("<AS:Ola xmlns:AS='urn:AnSt' />"))

    End Sub

    Protected Overrides Sub Execute(ByVal context As System.Activities.NativeActivityContext)
        'Obtain the runtime value of the Envelope input argument
        WfContext = context

        Dim oEnv As IEnvelope
        oEnv = context.GetValue(Me.EnvRq)

        'Do this in derived activity class
        'Dim name As String = oEnv.GetCustomParam(ESB.Core.Util.C_sCustomParamWorkflowBookmarkName)

        'If (name = String.Empty) Then
        '    'Don't create a Bookmark
        'Else
        '    WfContext.CreateBookmark(name, New BookmarkCallback(AddressOf OnMyEventComplete))
        'End If

        m_oEnvRs = ProcessMsgSync(oEnv)

    End Sub

    '// NativeActivity derived activities that do asynchronous operations by calling 
    '// one of the CreateBookmark overloads defined on System.Activities.NativeActivityContext 
    '// must override the CanInduceIdle property and return true.
    Protected Overrides ReadOnly Property CanInduceIdle As Boolean
        Get
            Return True
        End Get
    End Property

    Overridable Sub OnMyEventComplete(ByVal context As NativeActivityContext, ByVal bookmark As Bookmark, ByVal state As Object)
        Dim oEnvRq As IEnvelope
        Dim oEnvRs As IEnvelope
        oEnvRq = context.GetValue(Me.EnvRq)
        oEnvRs = oEnvRq.CreateResponse
        oEnvRs.AddDocument(New InlineDocument("<AS:Ola xmlns:AS='urn:AnSt'>First base!</AS:Ola>"))


        Me.Result.Set(context, state)
    End Sub


    'Look at keeping a local pointer to the thing...
    ''' <summary>
    ''' TODO: Need to change this to somehow get the context (or thread ID) from the parent - ESBEnvelope custom param maybe...???
    ''' and save it somewhere...
    ''' Then, this method needs to use that threadID to get the context...so that all Workflow threads (even the ESBWorkflow Adaptor initiating thread) use the same 
    ''' context as the original request handling code. We then get a complete request trace as required.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Function GetContext() As ESB.Core.Interfaces.ServerContext.IServerContext
        If m_oServerContext Is Nothing Then
            m_oServerContext = DirectCast(DirectCast(ESB.Core.Services.ContextManager.getContextManager(True), ESB.Core.Services.ContextManager).GetExistingContext, ESB.Core.Interfaces.ServerContext.IServerContext)
        End If
        Return m_oServerContext
    End Function

    Public Sub New()
        m_oErr = New ErrorXML
        m_oErrEntry = New ErrorEntry
        m_oWarningEntry = New WarningEntry
    End Sub

    'Feel free to change this base class as you require
    Public Overridable Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope Implements Core.Interfaces.ProcessMsg.IProcessMsg.ProcessMsgSync

        StartMethod()
        Try
            'Load up any Adaptor (usually for the VS.NET project wide) Specific Configuration data
            'Note that you need to define and setup the configuration before enabling this
            'Unlike Custom Params set in the Pipeline Config file which apply to individual ESB.NET services 
            '(.NET classes + ESB.NET Service Pipeline Configuration combination)
            'this is static inter-service configuration that applies to the whole adaptor 
            '(i.e. available & static amongst all classes in this VS.NET Project)

            'Todo: comment this out - no config data setup by default...
            'GetConfigData()

            'Keep a class wide reference to the request message
            'so that the derived class' methods also have it available
            m_oEnvRq = oEnv

            'Create a response envelope using the request envelope as a starting point. This preserves routing information etc.
            'but strips out the original documents, attachments & relevant manifest entries.
            m_oEnvRs = m_oEnvRq.CreateResponse()


            Return m_oEnvRq

        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString())
            ESB.Core.XMLSchemaClasses.ExtensionMethods.AddError(m_oEnvRs, "", ex.Source, ex.ToString)
            Return m_oEnvRs
        Finally
            EndMethod()
        End Try

    End Function

    'Not usually used for Asynchronous processing 
    'ProcessMsgSync is typically also invoked for Asynchronous processing 
    'Ignore this method
    Public Overridable Sub ProcessMsgAsync(ByVal oEnv As IEnvelope) Implements Core.Interfaces.ProcessMsg.IProcessMsg.ProcessMsgAsync
        StartMethod()

        LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.MessageID)
        LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.XML)

        EndMethod()
    End Sub
    Protected Overridable Sub GetConfigData()
        Dim m_sSourceSection As String = "ESB.Adaptors.Application.Services.Workflow"
        m_oCfgData = CType(ConfigurationManager.Read(m_sSourceSection), CustomConfigurationData)
        If m_oCfgData Is Nothing Then
            Throw New ApplicationException("no config on provider.")
        End If
    End Sub

End Class

