Option Explicit On
Option Strict On
Option Infer On

'Imports System.Workflow.Runtime.Hosting
'Imports System.Workflow.Runtime

Imports System.Activities
Imports System.Activities.Statements
Imports System.Diagnostics
Imports System.Linq
Imports System.Collections.Generic
Imports System.Threading

Imports System.Activities.DurableInstancing

''' <summary>
''' Determines whether the Workflow adaptor caches workflows and what type of caching is done.
''' </summary>
''' <remarks>Note that any workflow caching means workflow needs to clean up its state when finished/initialise its state when starting.</remarks>
Public Enum eWorkflowCacheMode
    eNone = 0
    eThread = 1
    eInstance = 2
End Enum

''' 
'''  <summary>
''' 
''' </summary>
''' <remarks></remarks>
Public Class FlowchartWorkflowAdaptor
    Inherits ServiceBase
    Shared m_sLogDBConnectionString As String
    'Const connectionString As String = "Server=localhost\esb;Initial Catalog=EsbWfPersistence;Integrated Security=SSPI"

#Region "ESB.NET Documents"
    Dim m_oDocRq As Object 'Todo: replace 'Object' with your schema request class eg.Dim m_oDocRq As MemberEnquiryRQ.MemberDetailsEnquiryRQ
    Dim m_oDocRs As Object 'Todo: replace 'Object' with your schema response class eg. Dim m_oDocRs As MemberEnqRSUpdRQ.MemberEnqRSUpdRQ 
#End Region
#Region "Workflow"
#Region "Constants"
    Const STR_WorkflowAssembly As String = "WorkflowAssembly"
    Const STR_WorkflowClass As String = "WorkflowClass"
    Const STR_WorkflowCacheMode As String = "WorkflowCacheMode"
#End Region
#Region "Event Handles"
    'Dim m_evtWaitHandle As New AutoResetEvent(False)

    Dim m_eSyncEvent As New AutoResetEvent(False)
    Dim m_eIdleEvent As New AutoResetEvent(False)
    Dim m_eUnloadedEvent As New AutoResetEvent(False)

#End Region
#Region "Custom Params"
    Protected m_sWorkflowAssembly As String
    Protected m_sWorkflowClass As String
    Protected m_eWorkflowCacheMode As eWorkflowCacheMode

    Dim m_guidWorkflowInstanceID As Guid

    Protected m_bResumeExistingWorkflow As Boolean = False
    Protected m_sBookmarkName As String
    Protected m_bRequiresPersistence As Boolean = False
#End Region
#End Region

    Shared Sub New()

        m_sLogDBConnectionString = System.Configuration.ConfigurationManager.AppSettings("MsSqlWfPersistenceConnectionString")

    End Sub

    ''' <summary>
    ''' Just create a default constructor, cos class creation seems to have started failing
    ''' after adding the shared sub new method above...!!!???
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()

    End Sub
    '!!!Using Project Template that has older version of assemblies!!!
    ' Need to Fix asap & ship!!!
    'Service Entry Point 
    Public Overrides Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope
        StartMethod()

        Try

            MyBase.ProcessMsgSync(oEnv)
            '(This statement gets a document out of the request envelope. This is your service input.)
            'todo: replace this with your schema request class
            'm_oDocRq = oEnv.GetDocumentAsObject(GetType(MemberEnquiryRQ.MemberDetailsEnquiryRQ)  

            'todo: replace this with your schema response class
            'm_oDocRs = New MemberEnqRSUpdRQ.MemberEnqRSUpdRQ	  

            'Uncomment to get any custom parameters specified in the Pipeline config file for this request
            'sParam1 = m_oEnvrq.GetCustomParam(C_sCustomParamCSCInteropCore, "Param1Name")
            'sParam2 = m_oEnvrq.GetCustomParam(C_sCustomParamCSCInteropCore, "Param2Name")

            'This is where you manipulate your document, create a response etc.
            DoWork()

            'Example method to do XSLT
            'DoXslt()

            'Add the response document to the response envelope
            '(This is how your response is sent back)
            ' m_oEnvRs.AddDocument(New InlineDocument(m_oDocRs))

            Return m_oEnvRs

        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString())
            Throw

        Finally
            EndMethod()
        End Try
    End Function

    Protected Sub DoWork()
        Try

            GetWorkflowCustomParams()

            'Load the requested assembly
            Dim oAsm As System.Reflection.Assembly = System.Reflection.Assembly.LoadFrom(m_sWorkflowAssembly)

            'Get a reference to the System.Type for the OrderWorkflows.Workflow1
            Dim oWorkflowType As System.Type = oAsm.GetType(m_sWorkflowClass)
            Dim oWfActivity As System.Activities.Activity

            If m_eWorkflowCacheMode = eWorkflowCacheMode.eNone Then
                oWfActivity = DirectCast(ESB.Core.Services.ESBClassFactory.CreateObjectByStringName(m_sWorkflowClass, m_sWorkflowAssembly), System.Activities.Activity)
            ElseIf m_eWorkflowCacheMode = eWorkflowCacheMode.eThread Then
                'try get from cache
                Dim oCtx As Server.ServerContext = DirectCast(GetContext(), Server.ServerContext)
                oWfActivity = DirectCast(oCtx.cacheManager.GetFromCache("FlowchartAdaptor", m_sWorkflowAssembly, m_sWorkflowClass), System.Activities.Activity)

                'if not there, create and set to cache
                If oWfActivity Is Nothing Then
                    oWfActivity = DirectCast(ESB.Core.Services.ESBClassFactory.CreateObjectByStringName(m_sWorkflowClass, m_sWorkflowAssembly), System.Activities.Activity)
                    If oWfActivity Is Nothing Then
                        'error
                        Throw New Exception("could not create workflow:" & m_sWorkflowClass & "in assembly:" & m_sWorkflowAssembly)
                    Else
                        oCtx.cacheManager.SetToCache("FlowchartAdaptor", m_sWorkflowAssembly, m_sWorkflowClass, DirectCast(oWfActivity, Object))
                    End If
                End If
            End If




            Dim wfApp As WorkflowApplication
            If m_bResumeExistingWorkflow Then
                wfApp = New WorkflowApplication(oWfActivity)
            Else
                Dim oWfParameters As Dictionary(Of String, Object)
                oWfParameters = New Dictionary(Of String, Object)()
                oWfParameters.Add("ESBRequestMessageEnvelope", m_oEnvRq)
                wfApp = New WorkflowApplication(oWfActivity, oWfParameters)
            End If

            If m_bRequiresPersistence Then
                Dim store As New SqlWorkflowInstanceStore(m_sLogDBConnectionString)
                wfApp.InstanceStore = store
                wfApp.PersistableIdle = AddressOf wfAppPersistableIdle
            Else
                wfApp.Idle = AddressOf wfAppIdle
            End If

            m_guidWorkflowInstanceID = wfApp.Id
            m_oEnvRs.ConversationID = m_guidWorkflowInstanceID.ToString

            wfApp.Unloaded = AddressOf wfAppUnloaded
            wfApp.Completed = AddressOf wfAppCompleted
            wfApp.Aborted = AddressOf wfAppAborted
            wfApp.OnUnhandledException = AddressOf wfAppUnhandledException

            If m_bResumeExistingWorkflow Then
                wfApp.Load(m_guidWorkflowInstanceID)
                wfApp.ResumeBookmark(m_sBookmarkName, m_oEnvRq)
            End If

            'm_guidWorkflowInstanceID = wfApp.Id
            'm_oEnvRs.ConversationID = m_guidWorkflowInstanceID.ToString

            wfApp.Run()

            'm_guidWorkflowInstanceID = wfApp.Id
            'm_oEnvRs.ConversationID = m_guidWorkflowInstanceID.ToString
            ' Loop until the workflow completes.
            Dim waitHandles() As WaitHandle = {m_eSyncEvent, m_eIdleEvent}

            Dim nWaitSatisfierIndex As Integer = 0

            'Do 
            nWaitSatisfierIndex = WaitHandle.WaitAny(waitHandles)
            'If nWaitSatisfierIndex = 1 Then
            '    wfApp.Unload()
            'End If
            'Loop While (nWaitSatisfierIndex <> 0)

            wfApp.Unload()


        Catch eTerm As WorkflowTerminatedException
            ExceptionManager.Publish(eTerm)

            'TODO: m_oEnvRs.adderror here
            LogError(LogDefs.eLogLevel.eLOG_ERROR, eTerm.ToString())
            Throw

        Catch exception As Exception
            ExceptionManager.Publish(exception)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, exception.ToString())
            'TODO: m_oEnvRs.adderror here
            ExtensionMethods.AddError(m_oEnvRs, , , exception.ToString)
            Throw

        Finally
            'Dim waitHandle2() As WaitHandle = {m_eUnloadedEvent}
            ''wait for 15 seconds on un-catered for exception flows...
            ''TODO: log the timeout to error log...
            'WaitHandle.WaitAny(waitHandle2, 15000) ' <> 0)
        End Try

    End Sub


#Region "Event handlers"

    Sub wfAppAborted(ByVal e As WorkflowApplicationAbortedEventArgs)
        Console.WriteLine(e.Reason)
        m_eSyncEvent.Set()
    End Sub

    Function wfAppUnhandledException(ByVal e As WorkflowApplicationUnhandledExceptionEventArgs) As UnhandledExceptionAction
        Console.WriteLine(e.UnhandledException.ToString())
        m_eSyncEvent.Set()
        Return UnhandledExceptionAction.Terminate
    End Function

    Sub wfAppCompleted(ByVal e As WorkflowApplicationCompletedEventArgs)
        Try
            'The Response Envelope is stored in the "ESBResponseMessageEnvelope" Output parameter
            Dim oResponseMessageEnvelope As IEnvelope = DirectCast(e.Outputs("ESBResponseMessageEnvelope"), IEnvelope)
            Trace.WriteLine("Response from Sequential Workflow was: " + oResponseMessageEnvelope.ToXML())

            m_oEnvRs = oResponseMessageEnvelope
            m_oEnvRs.ConversationID = m_guidWorkflowInstanceID.ToString
        Catch ex As Exception
            Throw
        Finally
            m_eSyncEvent.Set()
        End Try
    End Sub

    Sub wfAppIdle(ByVal e As WorkflowApplicationIdleEventArgs)
        m_eIdleEvent.Set()
    End Sub


    Function wfAppPersistableIdle(ByVal e As WorkflowApplicationIdleEventArgs) As PersistableIdleAction
        m_eIdleEvent.Set()
        Return PersistableIdleAction.Persist
    End Function

    Sub wfAppUnloaded(ByVal e As WorkflowApplicationEventArgs)
        m_eSyncEvent.Set()
        m_eUnloadedEvent.Set()
    End Sub


#End Region





    ''' <summary>
    ''' Gets the CustomParam values from the request Envelope.
    ''' These have been copied there from the Pipeline Config file Custom Params section for this service
    ''' The parameters refer to the assembly and workflow classes that will be loaded and that these operations will be
    ''' performed upon.
    ''' </summary>
    ''' <remarks></remarks>
    Protected Sub GetWorkflowCustomParams()
        StartMethod()
        m_sWorkflowAssembly = ESB.Core.Util.Utils.ExpandRelativePath(m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, STR_WorkflowAssembly))
        m_sWorkflowClass = m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, STR_WorkflowClass)
        m_eWorkflowCacheMode = DirectCast([Enum].Parse(GetType(eWorkflowCacheMode), m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, STR_WorkflowCacheMode), True), eWorkflowCacheMode)

        'Passed in via request level custom params...
        m_sBookmarkName = m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamWorkflowBookmarkName)

        ''if a Bookmark is set, then we're dealing with a persistence related workflow...potentially...???hmmm...not strictly...
        '...a workflow that requires persistence to be enabled for it...

        'If Not String.IsNullOrEmpty(m_sBookmarkName) Then : m_bRequiresPersistence = Boolean.Parse(m_sBookmarkName) : End If

        'See if we can derive this from the actual workflow attributes at runtime
        Dim sRequiresPersistence As String = m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, ESBConstants.C_sCustomParamWorkflowRequiresPersistence)
        If Not String.IsNullOrEmpty(sRequiresPersistence) Then : m_bRequiresPersistence = Boolean.Parse(sRequiresPersistence) : End If

        'TODO - make this a constant...
        Dim sResumeExistingWorkflow As String = m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, ESBConstants.C_sCustomParamWorkflowResumeExistingWorkflow)
        If Not String.IsNullOrWhiteSpace(sResumeExistingWorkflow) Then : m_bResumeExistingWorkflow = Boolean.Parse(sResumeExistingWorkflow) : End If


        If m_bResumeExistingWorkflow = True Then
            If Not String.IsNullOrWhiteSpace(m_oEnvRq.ConversationID) Then
                If False = Guid.TryParse(m_oEnvRq.ConversationID, m_guidWorkflowInstanceID) Then
                    'add error to document - could not resume workflow instance as specified ConversationID is not a valid GUID
                    Throw New Exception("Error: Could not resume workflow instance as specified ConversationID: " & m_oEnvRq.ConversationID & ":is not a valid GUID")
                End If
            End If
        End If



        EndMethod()
    End Sub

    'This method shows you how to invoke an XSLT Transformation based upon config parameters
    'for a particular service configuration
    Protected Sub DoXslt()

        ' Get the custom parameters which indicate the xslt Resolvename and Context
        'These parameters are used to lookup the XSLT Pipeline Config file
        'which you need to configure to point to your XSLT file.
        Dim sXsltResolveName1 As String = m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "XSLTResolveName1")
        Dim sXsltContext1 As String = m_oEnvRq.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "XSLTContext1")

        ' Get the xml we will pass to xslt processing...
        'Get document from envelope by index or by type (or both)
        'sXsltInputXml = m_oEnvrq.GetDocument(GetType(<InsertDocumentTypeHere>)).GetAsString()
        'Get first document from Envelope & transform
        Dim sXsltInputXml As String = m_oEnvRq.GetDocument(0).GetAsString()

        LogMsg(LogDefs.eLogLevel.eLOG_INFO, "XSLT Input XML = " & sXsltInputXml)

        ' Output from xslt processing goes here... 

        'Get the XSLT Manager to do the transform for us to extract the address information...
        LogMsg(LogDefs.eLogLevel.eLOG_INFO, "Calling XSLT processing, passing ResolveName = " _
                                            & sXsltResolveName1 _
                                            & ", Context = " _
                                            & sXsltContext1)
        Dim sXsltOutputXml As String = ""

        GetContext.Transform(sXsltResolveName1, sXsltContext1, sXsltInputXml, sXsltOutputXml)

        LogMsg(LogDefs.eLogLevel.eLOG_INFO, "XSLT Input XML = " & sXsltInputXml & vbNewLine & " XSLT Output XML= " & sXsltOutputXml)

    End Sub
End Class

'Configure this class name 
'i.e. "ESB.Adaptors.Application.Service1.ServicedComponent"
'in the ESB.NET Service Pipeline if this service is to be hosted by COM+ rather than directly by the .NET Runtime
<EventTrackingEnabled(True), Transaction(TransactionOption.Supported), System.Runtime.InteropServices.ProgId("ESB.Adaptors.Application.Workflow.SequentialWorkflowAdaptor.ServicedComponent")> _
Public Class FlowchartWorkflowAdaptorServicedComponent
    Inherits ESBServicedComponent

    Public Overrides Function ProcessMsgSync(ByVal oEnvRq As IEnvelope) As IEnvelope
        Dim o As New FlowchartWorkflowAdaptor
        Return o.ProcessMsgSync(oEnvRq)
    End Function
End Class