Imports System
Imports System.ComponentModel
Imports System.Workflow.ComponentModel
Imports ESB.Core.XMLSchemaClasses

Public Class EsbActivity
    Inherits System.Workflow.ComponentModel.Activity

#Region "Activity Property Exposure Registration Statements - One for each of the Workflow Activity Properties to be exposed to the Designer"
    Public Shared ActivityClassNameProperty As DependencyProperty = DependencyProperty.Register("ActivityClassName", GetType(System.String), GetType(EsbActivity))
    Public Shared ActivityAssemblyNameProperty As DependencyProperty = DependencyProperty.Register("ActivityAssemblyName", GetType(System.String), GetType(EsbActivity))
    Public Shared ESBRequestMessageEnvelopeActivityProperty As DependencyProperty = DependencyProperty.Register("ESBRequestMessageEnvelopeActivity", GetType(IEnvelope), GetType(EsbActivity))
    Public Shared ESBResponseMessageEnvelopeActivityProperty As DependencyProperty = DependencyProperty.Register("ESBResponseMessageEnvelopeActivity", GetType(IEnvelope), GetType(EsbActivity))
    Public Shared ESBContextThreadActivityProperty As DependencyProperty = DependencyProperty.Register("ESBContextThreadActivity", GetType(System.String), GetType(EsbActivity))
#End Region
#Region "Activity Designer category Constants"
    ' Define constant values for the Property categories.  
    Private Const ESBServiceParametersCategory As String = "ESB Service in/out Parameters"
    Private Const ESBRuntimeCategory As String = "ESB Runtime"
    Private Const ESBActivityInvocationParametersCategory As String = "ESB Activity Invocation Parameters"
#End Region
#Region "Workflow Activity Properties"
    <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
   <BrowsableAttribute(True)> _
   <DescriptionAttribute("The ActivityClassName property is used to specify the class to be invoked.")> _
   <CategoryAttribute(ESBActivityInvocationParametersCategory)> _
   Public Property ActivityClassName() As String

        Get
            Return CType(MyBase.GetValue(EsbActivity.ActivityClassNameProperty), String)
        End Get
        Set(ByVal value As String)
            MyBase.SetValue(EsbActivity.ActivityClassNameProperty, value)
        End Set
    End Property

    <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
   <BrowsableAttribute(True)> _
   <DescriptionAttribute("The ActivityAssemblyName property is used to specify the assembly the class is to be loaded from.")> _
   <CategoryAttribute(ESBActivityInvocationParametersCategory)> _
    Public Property ActivityAssemblyName() As String

        Get
            Return CType(MyBase.GetValue(EsbActivity.ActivityAssemblyNameProperty), String)
        End Get
        Set(ByVal value As String)
            MyBase.SetValue(EsbActivity.ActivityAssemblyNameProperty, value)
        End Set
    End Property

    <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
    <BrowsableAttribute(True)> _
    <DescriptionAttribute("The Request Message Envelope. Bind this to the workflow ESBRequestMessageEnvelopeActivity base class parameter.")> _
    <CategoryAttribute(ESBServiceParametersCategory)> _
    Public Property ESBRequestMessageEnvelopeActivity() As IEnvelope
        Get
            Return CType(MyBase.GetValue(EsbActivity.ESBRequestMessageEnvelopeActivityProperty), IEnvelope)
        End Get
        Set(ByVal value As IEnvelope)
            MyBase.SetValue(EsbActivity.ESBRequestMessageEnvelopeActivityProperty, value)
        End Set
    End Property

    <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
    <BrowsableAttribute(True)> _
    <DescriptionAttribute("The Request Message Envelope.  Bind this to the workflow ESBResponseMessageEnvelopeActivity base class parameter.")> _
    <CategoryAttribute(ESBServiceParametersCategory)> _
    Public Property ESBResponseMessageEnvelopeActivity() As IEnvelope
        Get
            Return CType(MyBase.GetValue(EsbActivity.ESBResponseMessageEnvelopeActivityProperty), IEnvelope)
        End Get
        Set(ByVal value As IEnvelope)
            MyBase.SetValue(EsbActivity.ESBResponseMessageEnvelopeActivityProperty, value)
        End Set

    End Property

    <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
    <BrowsableAttribute(True)> _
    <DescriptionAttribute("The Managed threadID that the ESB Runtime uses to match up the ESB Context for this request. Bind this property to the workflow ESBContextThread base class parameter")> _
    <CategoryAttribute(ESBRuntimeCategory)> _
    Public Property ESBContextThreadActivity() As String

        Get
            Return CType(MyBase.GetValue(EsbActivity.ESBContextThreadActivityProperty), String)
        End Get
        Set(ByVal value As String)
            MyBase.SetValue(EsbActivity.ESBContextThreadActivityProperty, value)
        End Set
    End Property
#End Region
#Region "Workflow Execution Method"
    Protected Overrides Function Execute(ByVal executionContext As System.Workflow.ComponentModel.ActivityExecutionContext) As System.Workflow.ComponentModel.ActivityExecutionStatus
        StartMethod()
        InvokeObject(ActivityClassName, ESB.Core.Util.Utils.ExpandRelativePath(ActivityAssemblyName), ESBRequestMessageEnvelopeActivity, ESBResponseMessageEnvelopeActivity)
        EndMethod()
        Return MyBase.Execute(executionContext)
    End Function
#End Region
#Region "Helper Methods"
    Protected Overloads Sub InvokeObject(ByRef sTransactionComponent As String, ByRef sComponentServer As String, ByRef oEnvRq As Envelope, ByRef oEnvRs As Envelope)
        Dim sInterfaceName As String
        Dim oTimerCLR As ESB.Core.Util.Timer
        Dim sExecutionTime As String
        Dim lExecutionTime As Double

        'The Interface that takes the Keystroke Generic Envelope object as parameters.
        'This is the preferred interface for local calls as it is object based
        'and the XML Message does not need to be parsed...
        Dim oESBIf As ESB.Core.Interfaces.ProcessMsg.IProcessMsg


        oTimerCLR = New ESB.Core.Util.Timer
        oTimerCLR.StartTimer()


        'This should be a native CLR based object...
        oESBIf = CreateObjectCLR(sTransactionComponent, sComponentServer)
        If ShouldLog(LogDefs.eLogLevel.eLOG_ACTIVITY_HANDLER_INVOKED) Then : LogMsg(LogDefs.eLogLevel.eLOG_ACTIVITY_HANDLER_INVOKED, sComponentServer & "." & sTransactionComponent) : End If
        If oESBIf Is Nothing Then
            'Error
            Dim sError As String
            sError = "Attempted to create object:" & sTransactionComponent & " in Assembly:" & sComponentServer & " Failed. "
            LogError(eLogLevel.eLOG_ERROR, sError)
            Throw New Exception(sError)
        Else
            'Everything OK- Object created successfully

            If ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then : LogMsg(LogDefs.eLogLevel.eLOG_TRACE, "Successfully created CLR based component ==>" & sTransactionComponent & "<== on Server==>" & sComponentServer & "<==" & "Resolve name sought was: .") : End If
            If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(LogDefs.eLogLevel.eLOG_DATAXFER, "InvokeObjectCLR sending:sXMLrq=" & oEnvRq.XML) : End If
            oEnvRs = InvokeObjectCLR(oESBIf, oEnvRq)
            If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(LogDefs.eLogLevel.eLOG_DATAXFER, "InvokeObjectCLR returned:sXMLrs=" & oEnvRs.XML) : End If
            oTimerCLR.StopTimer()
            sInterfaceName = "CLR"
            sExecutionTime = oTimerCLR.Duration
            lExecutionTime = oTimerCLR.ElapsedTime
        End If

        LogTiming(eLogLevel.eLOG_TIMING, "Pipeline Executer - Execution time on interface:" & sInterfaceName & "=" & sExecutionTime)


    End Sub
    Private Function CreateObjectCLR(ByRef sTransactionComponent As String, ByRef sComponentServer As String) As ESB.Core.Interfaces.ProcessMsg.IProcessMsg
        StartMethod()
        'try & create a .NET object first...
        Dim oESBIf As ESB.Core.Interfaces.ProcessMsg.IProcessMsg
        Try
            oESBIf = ESBClassFactory.CreateObjectByStringName(sTransactionComponent, sComponentServer)
            Return oESBIf
        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(eLogLevel.eLOG_ERROR, "error creating object from :" & "sTransactionComponent=" & sTransactionComponent & "sComponentServer(Assembly file path) = " & sComponentServer & ". Error=" & ex.ToString)
            Return Nothing
        Finally
            EndMethod()
        End Try
    End Function
    Private Function InvokeObjectCLR(ByRef oESBIf As ESB.Core.Interfaces.ProcessMsg.IProcessMsg, ByRef oEnvRq As Envelope) As Envelope
        'try & create a .NET object first...
        StartMethod()
        Dim oEnvRs As IEnvelope = Nothing
        Try
            oEnvRs = oESBIf.ProcessMsgSync(oEnvRq)
            Return oEnvRs
        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(eLogLevel.eLOG_ERROR, ex.ToString)

            'better return something...???
            If oEnvRs Is Nothing Then
                oEnvRs = oEnvRq.CreateResponse()


                'TODO: (MINA) Add this functionality to the ErrorXML object or something...
                Dim oErrXML As New ErrorXML.ErrorXML
                Dim oErrEntry As New ErrorXML.ErrorEntry
                oErrEntry.Description = ex.ToString
                oErrEntry.Source = ex.Source
                oErrXML.ErrorEntry.Add(oErrEntry)
                oEnvRs.AddDocument(New InlineDocument(oErrXML))
            End If
            Return oEnvRs
        Finally
            EndMethod()
        End Try
    End Function
#End Region

End Class
