'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
'Base class for ESB components

Imports ESB.Core.Services.Logger
Imports ESB.Core.XMLSchemaClasses.ESBEnvelope
Imports ESB.Core.Services
Imports ESB.Core.Interfaces.ServerContext
Imports Microsoft.ApplicationBlocks.ConfigurationManagement
Imports System
Imports System.ComponentModel
Imports System.Workflow.ComponentModel

Public Class EsbWorkflowActivityBase
    'Inherits System.Workflow.ComponentModel.CompositeActivity
    Inherits System.Workflow.ComponentModel.Activity
    Implements ESB.Core.Interfaces.ProcessMsg.IProcessMsg


    Dim m_oServerContext As ESB.Core.Interfaces.ServerContext.IServerContext

    Protected m_oCfgData As CustomConfigurationData

    Protected m_oEnvRq As IEnvelope
    Protected m_oEnvRs As IEnvelope

    Protected m_oErr As ErrorXML
    Protected m_oErrEntry As ErrorEntry
    Protected m_oWarningEntry As WarningEntry

    Private m_oContext As ServerContext

    ''' <summary>
    ''' TODO: Need to change this to somehow get the context (or thread ID) from the parent - ESBEnvelope custom param maybe...???
    ''' and save it somewhere...
    ''' Then, this method needs to use that threadID to get the context...so that all Workflow threads (even the ESBWorkflow Adaptor initiating thread) use the same 
    ''' context as the original request handling code. We then get a complete request trace as required.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overridable Function GetContext() As ESB.Core.Interfaces.ServerContext.IServerContext
        If m_oServerContext Is Nothing Then
            m_oServerContext = DirectCast(DirectCast(ESB.Core.Services.ContextManager.getContextManager(True), ESB.Core.Services.ContextManager).GetExistingContext, ESB.Core.Interfaces.ServerContext.IServerContext)
        End If
        Return m_oServerContext
    End Function

    Public Sub New()
        m_oErr = New ErrorXML
        m_oErrEntry = New ErrorEntry
        m_oWarningEntry = New WarningEntry
    End Sub

    'Feel free to change this base class as you require
    Public Overridable Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope Implements Core.Interfaces.ProcessMsg.IProcessMsg.ProcessMsgSync

        StartMethod()
        Try
            'Load up any Adaptor (usually for the VS.NET project wide) Specific Configuration data
            'Note that you need to define and setup the configuration before enabling this
            'Unlike Custom Params set in the Pipeline Config file which apply to individual ESB.NET services 
            '(.NET classes + ESB.NET Service Pipeline Configuration combination)
            'this is static inter-service configuration that applies to the whole adaptor 
            '(i.e. available & static amongst all classes in this VS.NET Project)

            'Todo: comment this out - no config data setup by default...
            'GetConfigData()

            'Keep a class wide reference to the request message
            'so that the derived class' methods also have it available
            ' m_oEnvRq = oEnv

            'Create a response envelope using the request envelope as a starting point. This preserves routing information etc.
            'but strips out the original documents, attachments & relevant manifest entries.
            m_oEnvRs = m_oEnvRq.CreateResponse()

            Return m_oEnvRq

        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString())

            'return a more structured error to the PipelineManager
            Dim oErrEntry As New ErrorEntry
            oErrEntry.Description = ex.ToString
            oErrEntry.Source = ex.Source
            m_oErr.ErrorEntry.Add(oErrEntry)
            m_oEnvRs.AddDocument(New InlineDocument(m_oErr))

            'Decide whether to continue processing the pipeline in case of error..
            'Default behaviour set by "ContinueProcessingPipelineOnAppAdaptorProcessingError" flag in web.Config for particular transport adaptor

            'if the above flag is set to true, and you're writing an Authorization Service for example, then 
            'you can uncomment the line below to ensure the pipeline processing stops after this class is invoked.
            'GetContext.continueProcessingPipeline = False

            Return m_oEnvRs

        Finally
            EndMethod()
        End Try

    End Function

    'Not usually used for Asynchronous processing 
    'ProcessMsgSync is typically also invoked for Asynchronous processing 
    'Ignore this method
    Public Overridable Sub ProcessMsgAsync(ByVal oEnv As IEnvelope) Implements Core.Interfaces.ProcessMsg.IProcessMsg.ProcessMsgAsync
        StartMethod()

        LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.MessageID)
        LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.XML)

        EndMethod()
    End Sub

    Protected Overridable Sub GetConfigData()
        Dim m_sSourceSection As String = "ESB.Adaptors.Application.Services.Workflow"
        m_oCfgData = CType(ConfigurationManager.Read(m_sSourceSection), CustomConfigurationData)
        If m_oCfgData Is Nothing Then
            Throw New ApplicationException("no config on provider.")
        End If
    End Sub
#Region "Workflow Adaptor Interface Parameters"
    '<DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
    '<BrowsableAttribute(True)> _
    '<DescriptionAttribute("The Request Message Envelope. Bind this to the workflow ESBRequestMessageEnvelope base class parameter.")> _
    '<CategoryAttribute(ESBServiceParametersCategory)> _
    Public Property ESBRequestMessageEnvelope() As IEnvelope
        Get
            Return m_oEnvRq
        End Get
        Set(ByVal value As IEnvelope)
            m_oEnvRq = value
        End Set
    End Property
    '    <DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)> _
    '<BrowsableAttribute(True)> _
    '<DescriptionAttribute("The Request Message Envelope. Bind this to the workflow ESBRequestMessageEnvelope base class parameter.")> _
    '<CategoryAttribute(ESBServiceParametersCategory)> _
    Public Property ESBResponseMessageEnvelope() As IEnvelope
        Get
            Return m_oEnvRs
        End Get
        Set(ByVal value As IEnvelope)
            m_oEnvRs = value
        End Set
    End Property
    'Private m_oESBRequestMessageEnvelope As IEnvelope
    'Public Property ESBRequestMessageEnvelope() As IEnvelope
    '    Get
    '        Return m_oESBRequestMessageEnvelope
    '    End Get
    '    Set(ByVal value As IEnvelope)
    '        m_oESBRequestMessageEnvelope = value
    '    End Set
    'End Property






    Private m_oESBResponseMessageEnvelope As IEnvelope
    'Public Property ESBResponseMessageEnvelope() As IEnvelope
    '    Get
    '        Return m_oESBResponseMessageEnvelope
    '    End Get
    '    Set(ByVal value As IEnvelope)
    '        m_oESBResponseMessageEnvelope = value
    '    End Set
    'End Property

    Private m_oESBContextThread As String
    Public Property ESBContextThread() As String
        Get
            Return m_oESBContextThread
        End Get
        Set(ByVal value As String)
            m_oESBContextThread = value
            'Me.UserData("ESBContextKey") = ESBContextThread()

        End Set
    End Property
#End Region



End Class

