<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:import href="FigtreeCommonVariables.xsl" />
	<xsl:output method="xml" />
	<xsl:template match="acord:OLifE">
		<xsl:element name="Parties">
			<xsl:apply-templates />
		</xsl:element>
	</xsl:template>
	<xsl:template match="acord:Holding" />
	<xsl:template match="acord:Party">
		<xsl:variable name="vPartyId" select="@id" />
		<xsl:variable name="vAddressId">
			<xsl:choose>
				<xsl:when test="$vPartyId = 'Witness1'">
					<xsl:value-of select="'claim.address_number_02'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'Witness2'">
					<xsl:value-of select="'claim.address_number_09'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'Witness3'">
					<xsl:value-of select="'claim.address_number_06'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'ContactPerson1'">
					<xsl:value-of select="'reh_creditor_number'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'ContactPerson2'">
					<xsl:value-of select="'claim.address_number_07'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'ContactPerson3'">
					<xsl:value-of select="'claim.address_number_03'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'Doctor1'">
					<xsl:value-of select="'claim.address_number_10'" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="' '" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="vAddressType">
			<xsl:choose>
				<xsl:when test="$vPartyId = 'Witness1' or $vPartyId = 'Witness2' or $vPartyId = 'Witness3'">
					<xsl:value-of select="'WIT'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'ContactPerson1' or $vPartyId = 'ContactPerson2' or $vPartyId = 'ContactPerson3'">
					<xsl:value-of select="'CON'" />
				</xsl:when>
				<xsl:when test="$vPartyId = 'Doctor1'">
					<xsl:value-of select="'TDR'" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="' '" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:if test="@id='Witness1' or @id='Witness2' or @id='Witness3' or @id='ContactPerson1'
					  or @id='ContactPerson2' or @id='ContactPerson3' or @id='Doctor1'">
			<xsl:element name="Address_Entry">
				<xsl:attribute name="ID">
					<xsl:value-of select="$vAddressId" />
				</xsl:attribute>
				<xsl:element name="Figtree_Systems">
					<xsl:element name="head">
						<xsl:element name="version">
							<xsl:value-of select="$fcvVersion" />
						</xsl:element>
						<xsl:element name="function">
							<xsl:attribute name="type">
								<xsl:value-of select="'request'" />
							</xsl:attribute>
							<xsl:attribute name="number">
								<xsl:value-of select="'34'" />
							</xsl:attribute>
						</xsl:element>
						<xsl:element name="module">
							<xsl:value-of select="$fcvModule-HK" />
						</xsl:element>
						<xsl:element name="client">
							<xsl:value-of select="$fcvClient-Disability" />
						</xsl:element>
						<xsl:element name="cob" />
						<xsl:element name="date">
							<xsl:attribute name="format">
								<xsl:value-of select="'dd/mm/yyyy'" />
							</xsl:attribute>
							<xsl:value-of select="$cvTodayDateddMMyyyy" />
						</xsl:element>
						<xsl:element name="time">
							<xsl:value-of select="$cvCurrentTime12AMPM"/>
						</xsl:element>
						<xsl:element name="time_requested">
							<xsl:value-of select="$cvCurrentTime12AMPM"/>
						</xsl:element>
						<xsl:element name="requested_by">
							<xsl:value-of select="$fcvRequestBy-CSCInterop" />
						</xsl:element>
					</xsl:element>
					<xsl:element name="security">
						<xsl:element name="encryption">
							<xsl:attribute name="type">
								<xsl:value-of select="'1'" />
							</xsl:attribute>
						</xsl:element>
						<xsl:element name="username">
							<xsl:value-of select="$fcvUsername" />
						</xsl:element>
						<xsl:element name="password">
							<xsl:value-of select="$fcvPassword" />
						</xsl:element>
					</xsl:element>
					<xsl:element name="request">
						<xsl:element name="mode">
							<xsl:attribute name="type">
								<!-- Use for add only, need to change code if update is included -->
								<xsl:value-of select="'add'" />
							</xsl:attribute>
						</xsl:element>
						<xsl:element name="address">
							<xsl:attribute name="address_type">
								<xsl:value-of select="$vAddressType" />
							</xsl:attribute>
							<xsl:attribute name="address_number">
								<xsl:value-of select="'automatic'" />
							</xsl:attribute>
							<!--
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'company_name'" />
								<xsl:with-param name="attr2Value" select="acord:FullName" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							-->
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'search'" />
								<xsl:with-param name="attr2Value" select="acord:Person/acord:LastName" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'surname'" />
								<xsl:with-param name="attr2Value" select="acord:Person/acord:LastName" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'given_name'" />
								<xsl:with-param name="attr2Value" select="concat(acord:Person/acord:FirstName,' ', acord:Person/acord:MiddleName)" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'title'" />
								<xsl:with-param name="attr2Value" select="acord:Person/acord:Title" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'address_1'" />
								<xsl:with-param name="attr2Value" select="acord:Address/acord:Line1" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'address_2'" />
								<xsl:with-param name="attr2Value" select="acord:Address/acord:Line2" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'city_1'" />
								<xsl:with-param name="attr2Value" select="acord:Address/acord:City" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'city_2'" />
								<xsl:with-param name="attr2Value" select="acord:Address/acord:AddressState" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'post'" />
								<xsl:with-param name="attr2Value" select="acord:Address/acord:Zip" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'country_code'" />
								<xsl:with-param name="attr2Value" select="' '" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'email'" />
								<xsl:with-param name="attr2Value" select="' '" />
								<xsl:with-param name="attr3Value" select="'address'" />
							</xsl:call-template>
							<xsl:apply-templates select="acord:Phone" />
						</xsl:element>
						<xsl:element name="confirm">
							<xsl:element name="format">
								<xsl:attribute name="structure">
									<xsl:value-of select="''" />
								</xsl:attribute>
							</xsl:element>
						</xsl:element>
					</xsl:element>
				</xsl:element>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	<xsl:template match="acord:Phone">
		<xsl:choose>
			<xsl:when test="acord:PhoneTypeCode/@tc = '1' or acord:PhoneTypeCode/@tc = '2'">
				<xsl:call-template name="AddFieldRow">
					<xsl:with-param name="attr1Value" select="'phone'" />
					<xsl:with-param name="attr2Value" select="acord:DialNumber" />
					<xsl:with-param name="attr3Value" select="'address'" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="acord:PhoneTypeCode/@tc = '12'">
				<xsl:call-template name="AddFieldRow">
					<xsl:with-param name="attr1Value" select="'fax'" />
					<xsl:with-param name="attr2Value" select="acord:DialNumber" />
					<xsl:with-param name="attr3Value" select="'address'" />
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise />
		</xsl:choose>
	</xsl:template>
	<xsl:template name="AddFieldRow">
		<xsl:param name="attr1Name" select="'name'" />
		<xsl:param name="attr1Value" select="' '" />
		<xsl:param name="attr2Name" select="'value'" />
		<xsl:param name="attr2Value" select="' '" />
		<xsl:param name="attr3Name" select="'table'" />
		<xsl:param name="attr3Value" select="' '" />
		<xsl:element name="field">
			<xsl:if test="$attr1Name != 'IGNORE'">
				<xsl:attribute name="{$attr1Name}">
					<xsl:value-of select="$attr1Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr2Name != 'IGNORE'">
				<xsl:attribute name="{$attr2Name}">
					<xsl:value-of select="$attr2Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr3Name != 'IGNORE'">
				<xsl:attribute name="{$attr3Name}">
					<xsl:value-of select="$attr3Value" />
				</xsl:attribute>
			</xsl:if>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
