<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:import href="FigtreeCommonVariables.xsl" />
	<!-- Output type -->
	<xsl:output method="xml" />
	<!-- OLifE -->
	<xsl:template match="/acord:OLifE">
		<xsl:if test="acord:Party/@id = 'Claimant'"></xsl:if>
		<xsl:element name="Figtree_Systems">
			<xsl:element name="head">
				<xsl:element name="version"><xsl:value-of select="$fcvVersion" /></xsl:element>
				<xsl:element name="function">
					<xsl:attribute name="type">
						<xsl:value-of select="'request'" />
					</xsl:attribute>
					<xsl:attribute name="number">
						<xsl:value-of select="'853'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="module"><xsl:value-of select="$fcvModule-WC" /></xsl:element>
				<xsl:element name="client"><xsl:value-of select="$fcvClient-Disability" /></xsl:element>
				<xsl:element name="cob" />
				<xsl:element name="date">
					<xsl:attribute name="format">
						<xsl:value-of select="'dd/mm/yyyy'" />
					</xsl:attribute>
					<xsl:value-of select="$cvTodayDateddMMyyyy" />
				</xsl:element>
				<xsl:element name="time">
					<xsl:value-of select="$cvCurrentTime12AMPM" />
				</xsl:element>
				<xsl:element name="time_requested">
					<xsl:value-of select="$cvCurrentTime12AMPM" />
				</xsl:element>
				<xsl:element name="requested_by"><xsl:value-of select="$fcvRequestBy-CSCInterop" /></xsl:element>
			</xsl:element>
			<xsl:element name="security">
				<xsl:element name="encryption">
					<xsl:attribute name="type">
						<xsl:value-of select="'1'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="username"><xsl:value-of select="$fcvUsername" /></xsl:element>
				<xsl:element name="password"><xsl:value-of select="$fcvPassword" /></xsl:element>
			</xsl:element>
			<xsl:element name="request">
				<xsl:element name="mod">
					<xsl:attribute name="web_id">
						<xsl:value-of select="'WEB1'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="system_init" />
				<xsl:element name="screen">
					<xsl:attribute name="mode">
						<xsl:value-of select="'add'" />
					</xsl:attribute>
					<xsl:attribute name="add">
						<xsl:value-of select="'Add a Claim'" />
					</xsl:attribute>
					<xsl:attribute name="params">
						<xsl:value-of select="'From_menu;table=claim;folders=header,Accident,Codes 1,Codes 2,Other,Tab 2,Finance;options=auto-numbering-only,status-readonly,status-I,dont_default_related_info'" />
					</xsl:attribute>
					<xsl:attribute name="key_history">
						<xsl:value-of select="'system.records_per_page=10;policy=DISABILITY'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="parameters">
					<!-- Default values -->
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="'Automatic'" />
						<xsl:with-param name="attr1Value" select="'claim.incident_number'" />
					</xsl:call-template>
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="'INC'" />
						<xsl:with-param name="attr1Value" select="'claim.permanent_or_casual_code'" />
					</xsl:call-template>
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="'IP'" />
						<xsl:with-param name="attr1Value" select="'claim.spare_03_code'" />
					</xsl:call-template>
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="'I'" />
						<xsl:with-param name="attr1Value" select="'claim.status'" />
					</xsl:call-template>
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="'3'" />
						<xsl:with-param name="attr1Value" select="'claim.special_1_code'" />
					</xsl:call-template>
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="'ORD'" />
						<xsl:with-param name="attr1Value" select="'claim.prevent_recommendn_code'" />
					</xsl:call-template>
					<xsl:apply-templates />
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	<!-- Holding -->
	<xsl:template match="acord:Holding">
		<xsl:if test="acord:Policy/acord:PolNumber">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="acord:Policy/acord:PolNumber" />
				<xsl:with-param name="attr1Value" select="'claim.policy'" />
			</xsl:call-template>
		</xsl:if>
		<xsl:if test="acord:Policy/acord:PolNumber">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="acord:Policy/acord:PolNumber" />
				<xsl:with-param name="attr1Value" select="'claim.insurer_number'" />
			</xsl:call-template>
		</xsl:if>
		<xsl:if test="acord:Policy/acord:Claim/acord:LossReportDate !=''">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="date:format-date(acord:Policy/acord:Claim/acord:LossReportDate,$cvDateFormatddMMyyyy)" />
				<xsl:with-param name="attr1Value" select="'claim.registration_date'" />
			</xsl:call-template>
		</xsl:if>
		<xsl:if test="acord:Policy/acord:Claim/acord:ClaimDateOfLoss !=''">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="date:format-date(acord:Policy/acord:Claim/acord:ClaimDateOfLoss,$cvDateFormatddMMyyyy)" />
				<xsl:with-param name="attr1Value" select="'claim.accident_date'" />
			</xsl:call-template>
		</xsl:if>
		<xsl:if test="acord:Policy/acord:Claim/acord:ClaimKey">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="acord:Policy/acord:Claim/acord:ClaimKey" />
				<xsl:with-param name="attr1Value" select="'claim.spare_char_10'" />
			</xsl:call-template>
		</xsl:if>
		<xsl:if test="acord:Policy/acord:EffDate !=''">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="date:format-date(acord:Policy/acord:EffDate,$cvDateFormatddMMyyyy)" />
				<xsl:with-param name="attr1Value" select="'claim.spare_date_1'" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	<!-- Party -->
	<xsl:template match="acord:Party">
		<xsl:if test="@id = 'Claimant'">
			<xsl:if test="acord:PartySysKey">
				<xsl:call-template name="AddParameter">
					<xsl:with-param name="elementValue" select="acord:PartySysKey" />
					<xsl:with-param name="attr1Value" select="'claim.employee_number'" />
				</xsl:call-template>
			</xsl:if>
			<xsl:for-each select="acord:KeyedValue">
				<xsl:choose>
					<xsl:when test="@id = 'EmpSit'">
						<xsl:call-template name="AddParameter">
							<xsl:with-param name="elementValue" select="acord:KeyValue" />
							<xsl:with-param name="attr1Value" select="'claim.editor_04'" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise />
				</xsl:choose>
			</xsl:for-each>
			<xsl:apply-templates select="acord:Attachment" />
			<xsl:apply-templates select="acord:Employment" />
			<xsl:apply-templates select="acord:Risk/acord:MedicalCondition" />
		</xsl:if>
	</xsl:template>
	<xsl:template match="acord:Attachment">
		<xsl:if test="acord:AttachmentSource">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="concat(acord:AttachmentSource,'.htm')" />
				<xsl:with-param name="attr1Value" select="'claim.image_id'" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	<xsl:template match="acord:Employment">
		<xsl:choose>
			<xsl:when test="@id = 'CurrentOccupation'">
				<xsl:if test="acord:EmployerName">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="acord:EmployerName" />
						<xsl:with-param name="attr1Value" select="'claim.org_code'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:HireDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:HireDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_22'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:RetirementDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:RetirementDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.ceased_work_date'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:SalaryMode/@tc">
					<xsl:choose>
						<xsl:when test="acord:SalaryMode/@tc = '6'">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'W'" />
								<xsl:with-param name="attr1Value" select="'claim.spare_12_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:SalaryMode/@tc = '7'">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'F'" />
								<xsl:with-param name="attr1Value" select="'claim.spare_12_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:SalaryMode/@tc = '4'">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'M'" />
								<xsl:with-param name="attr1Value" select="'claim.spare_12_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:SalaryMode/@tc = '1'">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'Y'" />
								<xsl:with-param name="attr1Value" select="'claim.spare_12_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise />
					</xsl:choose>
				</xsl:if>
				<xsl:if test="acord:AnnualSalary">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="acord:AnnualSalary" />
						<xsl:with-param name="attr1Value" select="'claim.spare_decimal_4'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:HoursWorkedPerWeek">
					<xsl:choose>
						<xsl:when test="acord:HoursWorkedPerWeek &gt; 34">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'1'" />
								<xsl:with-param name="attr1Value" select="'claim.emp_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'2'" />
								<xsl:with-param name="attr1Value" select="'claim.emp_code'" />
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="acord:TerminationDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:TerminationDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_24'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:Occupation">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="acord:Occupation" />
						<xsl:with-param name="attr1Value" select="'claim.spare_char_20'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:ReturnDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:ReturnDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_20'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:EmploymentStatusTC">
					<xsl:choose>
						<xsl:when test="acord:EmploymentStatusTC = '1'">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'N'" />
								<xsl:with-param name="attr1Value" select="'claim.spare_18_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:EmploymentStatusTC = '2'">
							<xsl:call-template name="AddParameter">
								<xsl:with-param name="elementValue" select="'Y'" />
								<xsl:with-param name="attr1Value" select="'claim.spare_18_code'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise />
					</xsl:choose>
				</xsl:if>
			</xsl:when>
			<xsl:when test="@id = 'PreviousOccupation1'">
				<xsl:if test="acord:HireDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:HireDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_25'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:TerminationDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:TerminationDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_26'" />
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:when test="@id = 'PreviousOccupation2'">
				<xsl:if test="acord:HireDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:HireDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_27'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:TerminationDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:TerminationDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_28'" />
					</xsl:call-template>
				</xsl:if>
			</xsl:when>
			<xsl:otherwise />
		</xsl:choose>
	</xsl:template>
	<xsl:template match="acord:Risk/acord:MedicalCondition">
		<xsl:if test="acord:ExamineeDesc">
			<xsl:call-template name="AddParameter">
				<xsl:with-param name="elementValue" select="acord:ExamineeDesc" />
				<xsl:with-param name="attr1Value" select="'claim.editor_01'" />
			</xsl:call-template>
		</xsl:if>
		<xsl:for-each select="acord:KeyedValue">
			<xsl:choose>
				<xsl:when test="@id = 'Location'">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="acord:KeyValue" />
						<xsl:with-param name="attr1Value" select="'claim.editor_06'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'ResultInj'">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="acord:KeyValue" />
						<xsl:with-param name="attr1Value" select="'claim.editor_02'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'AccidentDescription'">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="acord:KeyValue" />
						<xsl:with-param name="attr1Value" select="'claim.accident_description'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise />
			</xsl:choose>
		</xsl:for-each>
		<xsl:for-each select="acord:MedicalTreatment">
			<xsl:if test="@TreatmentPhysicianPartyID = 'Doctor1'">
				<xsl:if test="acord:TreatmentStartDate !=''">
					<xsl:call-template name="AddParameter">
						<xsl:with-param name="elementValue" select="date:format-date(acord:TreatmentStartDate,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr1Value" select="'claim.spare_date_29'" />
					</xsl:call-template>
				</xsl:if>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<xsl:template name="AddParameter">
		<xsl:param name="elementValue" select="' '" />
		<xsl:param name="attr1Value" select="' '" />
		<xsl:param name="attr2Name" select="' '" />
		<xsl:param name="attr2Value" select="' '" />
		<xsl:element name="parameter">
			<xsl:attribute name="field">
				<xsl:value-of select="$attr1Value" />
			</xsl:attribute>
			<xsl:if test="$attr2Name != ' '">
				<xsl:attribute name="{$attr2Name}">
					<xsl:value-of select="$attr2Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:value-of select="$elementValue" />
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>