<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="acord date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:import href="FigtreeCommonVariables.xsl" />
	<xsl:output method="xml" />
	<xsl:template match="/acord:OLifE">
		<Figtree_Systems>
			<head>
				<version>
					<xsl:value-of select="$fcvVersion" />
				</version>
				<function type="request" number="027" />
				<module>
					<xsl:value-of select="$fcvModule-WC" />
				</module>
				<client>
					<xsl:value-of select="$fcvClient-Disability" />
				</client>
				<cob />
				<date format="dd/mm/yyyy">
					<xsl:value-of select="$cvTodayDateddMMyyyy" />
				</date>
				<time>
					<xsl:value-of select="$cvCurrentTime12AMPM"></xsl:value-of>
				</time>
				<time_requested>
					<xsl:value-of select="$cvCurrentTime12AMPM"></xsl:value-of>
				</time_requested>
				<requested_by>
					<xsl:value-of select="$fcvRequestBy-CSCInterop" />
				</requested_by>
			</head>
			<security>
				<encryption type="1" />
				<username>
					<xsl:value-of select="$fcvUsername" />
				</username>
				<password>
					<xsl:value-of select="$fcvPassword" />
				</password>
			</security>
			<request>
				<mode type="add" />
				<note>
					<xsl:attribute name="incident_number">
						<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:HOClaimReferenceID">
							<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:HOClaimReferenceID" />
						</xsl:if>
					</xsl:attribute>
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'username'" />
						<xsl:with-param name="attr2Value" select="'CSCInterop'" />
						<xsl:with-param name="attr3Name" select="'IGNORE'" />
					</xsl:call-template>
					<xsl:apply-templates select="acord:Party" />
				</note>
				<confirm>
					<field name="policy" />
				</confirm>
			</request>
		</Figtree_Systems>
	</xsl:template>
	<xsl:template match="acord:Party">
		<xsl:if test="@id = 'Claimant'">
			<xsl:if test="acord:Attachment">
				<xsl:if test="acord:Attachment/acord:AttachmentSource">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'image_id'" />
						<xsl:with-param name="attr2Value" select="concat(acord:Attachment/acord:AttachmentSource, '.htm')" />
						<xsl:with-param name="attr3Name" select="'IGNORE'" />
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="acord:Attachment/acord:Description">
					<xsl:choose>
						<xsl:when test="acord:Attachment/acord:Description = 'S_MEDRPT'">
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'expense_code'" />
								<xsl:with-param name="attr2Value" select="'DRI'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'description'" />
								<xsl:with-param name="attr2Value" select="'Medical Details Report'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:Attachment/acord:Description = 'S_INVRPT'">
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'expense_code'" />
								<xsl:with-param name="attr2Value" select="'I2'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'description'" />
								<xsl:with-param name="attr2Value" select="'Investigator Report'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:Attachment/acord:Description = 'S_IMERPT'">
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'expense_code'" />
								<xsl:with-param name="attr2Value" select="'M1'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'description'" />
								<xsl:with-param name="attr2Value" select="'Independent Medical Examiner Report'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="acord:Attachment/acord:Description = 'S_FINRPT'">
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'expense_code'" />
								<xsl:with-param name="attr2Value" select="'I3'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
							<xsl:call-template name="AddFieldRow">
								<xsl:with-param name="attr1Value" select="'description'" />
								<xsl:with-param name="attr2Value" select="'Financial Details Report'" />
								<xsl:with-param name="attr3Name" select="'IGNORE'" />
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise />
					</xsl:choose>
				</xsl:if>
			</xsl:if>
		</xsl:if>
	</xsl:template>
	<xsl:template name="AddFieldRow">
		<xsl:param name="attr1Name" select="'name'" />
		<xsl:param name="attr1Value" select="' '" />
		<xsl:param name="attr2Name" select="'value'" />
		<xsl:param name="attr2Value" select="' '" />
		<xsl:param name="attr3Name" select="'table'" />
		<xsl:param name="attr3Value" select="' '" />
		<xsl:element name="field">
			<xsl:if test="$attr1Name != 'IGNORE'">
				<xsl:attribute name="{$attr1Name}">
					<xsl:value-of select="$attr1Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr2Name != 'IGNORE'">
				<xsl:attribute name="{$attr2Name}">
					<xsl:value-of select="$attr2Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr3Name != 'IGNORE'">
				<xsl:attribute name="{$attr3Name}">
					<xsl:value-of select="$attr3Value" />
				</xsl:attribute>
			</xsl:if>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
