<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:import href="FigtreeCommonVariables.xsl" />
	<xsl:output method="xml" />
	<xsl:template match="acord:OLifE">
		<xsl:element name="Figtree_Systems">
			<xsl:element name="head">
				<xsl:element name="function">
					<xsl:attribute name="type">
						<xsl:value-of select="'request'" />
					</xsl:attribute>
					<xsl:attribute name="number">
						<xsl:value-of select="'21'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="version">
					<xsl:value-of select="$fcvVersion" />
				</xsl:element>
				<xsl:element name="module">
					<xsl:value-of select="$fcvModule-WC" />
				</xsl:element>
				<xsl:element name="client">
					<xsl:value-of select="$fcvClient-Disability" />
				</xsl:element>
				<xsl:element name="cob" />
				<xsl:element name="date">
					<xsl:attribute name="format">
						<xsl:value-of select="'dd/mm/yyyy'" />
					</xsl:attribute>
					<xsl:value-of select="$cvTodayDateddMMyyyy" />
				</xsl:element>
				<xsl:element name="time">
					<xsl:value-of select="$cvCurrentTime12AMPM"></xsl:value-of>
				</xsl:element>
				<xsl:element name="time_requested">
					<xsl:value-of select="$cvCurrentTime12AMPM"></xsl:value-of>
				</xsl:element>
				<xsl:element name="requested_by">
					<xsl:value-of select="$fcvRequestBy-CSCInterop" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="security">
				<xsl:element name="encryption">
					<xsl:attribute name="type">
						<xsl:value-of select="'1'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="username">
					<xsl:value-of select="$fcvUsername" />
				</xsl:element>
				<xsl:element name="password">
					<xsl:value-of select="$fcvPassword" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="request">
				<xsl:element name="mode">
					<xsl:attribute name="type">
						<xsl:value-of select="'Update'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="claim">
					<xsl:attribute name="incident_number">
						<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:HOClaimReferenceID" />
					</xsl:attribute>
					<xsl:apply-templates select="acord:Holding/acord:Policy/acord:Claim" />
				</xsl:element>
				<xsl:element name="confirm">
					<xsl:element name="format">
						<xsl:attribute name="structure">
							<xsl:value-of select="'flat'" />
						</xsl:attribute>
					</xsl:element>
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	<!-- Claim -->
	<xsl:template match="acord:Holding/acord:Policy/acord:Claim">
		<xsl:for-each select="acord:CSCClaimExtension/acord:KeyedValue">
			<xsl:choose>
				<xsl:when test="@id = 'MedicalReport'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_22_code'" />
						<xsl:with-param name="attr2Value" select="acord:KeyName" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'MedicalReportDate'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_date_34'" />
						<xsl:with-param name="attr2Value" select="date:formatdate(acord:KeyValue,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'InvestigatorReport'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_26_code'" />
						<xsl:with-param name="attr2Value" select="acord:KeyName" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'InvestigatorReportDate'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_date_36'" />
						<xsl:with-param name="attr2Value" select="date:formatdate(acord:KeyValue,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'FinancialReport'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_32_code'" />
						<xsl:with-param name="attr2Value" select="acord:KeyName" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="@id = 'FinancialReportDate'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_date_35'" />
						<xsl:with-param name="attr2Value" select="date:formatdate(acord:KeyValue,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise />
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<!-- AddFieldRow -->
	<xsl:template name="AddFieldRow">
		<xsl:param name="attr1Name" select="'name'" />
		<xsl:param name="attr1Value" select="' '" />
		<xsl:param name="attr2Name" select="'value'" />
		<xsl:param name="attr2Value" select="' '" />
		<xsl:param name="attr3Name" select="'table'" />
		<xsl:param name="attr3Value" select="' '" />
		<xsl:element name="field">
			<xsl:if test="$attr1Name != 'IGNORE'">
				<xsl:attribute name="{$attr1Name}">
					<xsl:value-of select="$attr1Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr2Name != 'IGNORE'">
				<xsl:attribute name="{$attr2Name}">
					<xsl:value-of select="$attr2Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr3Name != 'IGNORE'">
				<xsl:attribute name="{$attr3Name}">
					<xsl:value-of select="$attr3Value" />
				</xsl:attribute>
			</xsl:if>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
