<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="date acord">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:import href="FigtreeCommonVariables.xsl" />
	<xsl:output method="xml" />
	<xsl:template match="/acord:OLifE">
		<Figtree_Systems>
			<head>
				<version>
					<xsl:value-of select="$fcvVersion" />
				</version>
				<function type="request" number="021" />
				<module>
					<xsl:value-of select="$fcvModule-WC" />
				</module>
				<client>
					<xsl:value-of select="$fcvClient-Disability" />
				</client>
				<cob />
				<date format="dd/mm/yyyy">
					<xsl:value-of select="$cvTodayDateddMMyyyy" />
				</date>
				<time>
					<xsl:value-of select="$cvCurrentTime12AMPM" />
				</time>
				<time_requested>
					<xsl:value-of select="$cvCurrentTime12AMPM" />
				</time_requested>
				<requested_by>
					<xsl:value-of select="$fcvRequestBy-CSCInterop" />
				</requested_by>
			</head>
			<security>
				<encryption type="1" />
				<username>
					<xsl:value-of select="$fcvUsername" />
				</username>
				<password>
					<xsl:value-of select="$fcvPassword" />
				</password>
			</security>
			<request>
				<mode type="update" />
				<claim>
					<xsl:attribute name="incident_number">
						<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:HOClaimReferenceID" />
					</xsl:attribute>
					<xsl:apply-templates select="acord:Holding/acord:Policy/acord:Claim" />
					<!-- confirm -->
					<confirm>
						<format structure="flat" />
					</confirm>
				</claim>
			</request>
		</Figtree_Systems>
	</xsl:template>
	<!-- Claim -->
	<xsl:template match="acord:Holding/acord:Policy/acord:Claim">
		<xsl:for-each select="acord:CSCClaimExtension/acord:KeyedValue">
			<xsl:choose>
				<xsl:when test="translate(@id,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'ESTIMATERETURNTOWORK'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'estimated_resume_date'" />
						<xsl:with-param name="attr2Value" select="date:formatdate(acord:KeyValue,$cvDateFormatddMMyyyy)" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise />
			</xsl:choose>
		</xsl:for-each>
		<xsl:for-each select="acord:CSCClaimExtension/acord:Estimate">
			<xsl:choose>
				<xsl:when test="translate(@id,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'ESTCOSTCLAIM'">
					<xsl:call-template name="AddFieldRow">
						<xsl:with-param name="attr1Value" select="'spare_decimal_16'" />
						<xsl:with-param name="attr2Value" select="acord:EstimateAmt" />
						<xsl:with-param name="attr3Value" select="'claim'" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise />
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<!-- AddFieldRow -->
	<xsl:template name="AddFieldRow">
		<xsl:param name="attr1Name" select="'name'" />
		<xsl:param name="attr1Value" select="' '" />
		<xsl:param name="attr2Name" select="'value'" />
		<xsl:param name="attr2Value" select="' '" />
		<xsl:param name="attr3Name" select="'table'" />
		<xsl:param name="attr3Value" select="' '" />
		<xsl:element name="field">
			<xsl:if test="$attr1Name != 'IGNORE'">
				<xsl:attribute name="{$attr1Name}">
					<xsl:value-of select="$attr1Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr2Name != 'IGNORE'">
				<xsl:attribute name="{$attr2Name}">
					<xsl:value-of select="$attr2Value" />
				</xsl:attribute>
			</xsl:if>
			<xsl:if test="$attr3Name != 'IGNORE'">
				<xsl:attribute name="{$attr3Name}">
					<xsl:value-of select="$attr3Value" />
				</xsl:attribute>
			</xsl:if>
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>
