<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:import href="FigtreeCommonVariables.xsl" />
	<!-- Output type -->
	<xsl:output method="xml" indent="yes" />
	<xsl:template match="acord:OLifE">
		<!-- Figtree System -->
		<xsl:element name="Figtree_Systems">
			<!-- head -->
			<xsl:element name="head">
				<xsl:element name="version">
					<xsl:value-of select="$fcvVersion" />
				</xsl:element>
				<xsl:element name="function">
					<xsl:attribute name="type">
						<xsl:value-of select="'request'" />
					</xsl:attribute>
					<xsl:attribute name="number">
						<xsl:value-of select="'021'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="module">
					<xsl:value-of select="$fcvModule-WC" />
				</xsl:element>
				<xsl:element name="client">
					<xsl:value-of select="$fcvClient-Disability" />
				</xsl:element>
				<xsl:element name="cob" />
				<xsl:element name="date">
					<xsl:attribute name="format">
						<xsl:value-of select="'dd/mm/yyyy'" />
					</xsl:attribute>
					<xsl:value-of select="$cvTodayDateddMMyyyy" />
				</xsl:element>
				<xsl:element name="time">
					<xsl:value-of select="$cvCurrentTime12AMPM" />
				</xsl:element>
				<xsl:element name="time_requested">
					<xsl:value-of select="$cvCurrentTime12AMPM" />
				</xsl:element>
				<xsl:element name="requested_by">
					<xsl:value-of select="$fcvRequestBy-CSCInterop" />
				</xsl:element>
			</xsl:element> <!-- head -->
			<!-- security -->
			<xsl:element name="security">
				<xsl:element name="encryption">
					<xsl:attribute name="type">
						<xsl:value-of select="'1'" />
					</xsl:attribute>
				</xsl:element>
				<xsl:element name="username">
					<xsl:value-of select="$fcvUsername" />
				</xsl:element>
				<xsl:element name="password">
					<xsl:value-of select="$fcvPassword" />
				</xsl:element>
			</xsl:element> <!-- security -->
			<!-- Request -->
			<xsl:element name="request">
				<!-- mode -->
				<xsl:element name="mode">
					<xsl:attribute name="type">
						<xsl:value-of select="'update'" />
					</xsl:attribute>
				</xsl:element> <!-- mode -->
				<!-- claim -->
				<xsl:element name="claim">
					<xsl:attribute name="incident_number">
						<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:HOClaimReferenceID" />
					</xsl:attribute>
					<!-- IME Date of Examination-->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:ExaminationDate !=''">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_date_37'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="date:formatdate(acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:ExaminationDate,$cvDateFormatddMMyyyy)" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME Injury description -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:TypeOfInjuries">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_char_26'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:TypeOfInjuries" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME Result of Injury-->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:ResultOfInjuries">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_char_42'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:ResultOfInjuries" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME estimated return to work date -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:EstimatedReturnToWork !=''">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_date_40'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="date:formatdate(acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:EstimatedReturnToWork,$cvDateFormatddMMyyyy)" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME type of duties - codes -->
					<!-- 1 - Current Full Duties -->
					<!-- 2 - Current job partial duties -->
					<!-- 3 - Alternate job full Duties -->
					<!-- 4 - Alternate job partial Duties -->
					<!-- 5 - Cannot resume employment -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:TypeOfDuties/@tc !=''">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_35_code'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:TypeOfDuties/@tc" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME type of duties free form description -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:TypeOfDutiesComments">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'editor_03'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:TypeOfDutiesComments" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME recommendation - code 1-accept 2-deny 3-more investigation -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:Recommendation/@tc !=''">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_33_code'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:Recommendation/@tc" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME recommendation comments -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:RecommendationComments">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'editor_05'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:RecommendationComments" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME follow up date -->
					<xsl:if test="acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:FollowUpDate !=''">
						<xsl:element name="field">
							<xsl:attribute name="table">
								<xsl:value-of select="'claim'" />
							</xsl:attribute>
							<xsl:attribute name="name">
								<xsl:value-of select="'spare_date_39'" />
							</xsl:attribute>
							<xsl:attribute name="value">
								<xsl:value-of select="date:formatdate(acord:Holding/acord:Policy/acord:Claim/acord:CSCClaimExtension/acord:MedicalExamination/acord:FollowUpDate,$cvDateFormatddMMyyyy)" />
							</xsl:attribute>
						</xsl:element>
					</xsl:if>
					<!-- IME details received -->
					<xsl:element name="field">
						<xsl:attribute name="table">
							<xsl:value-of select="'claim'" />
						</xsl:attribute>
						<xsl:attribute name="name">
							<xsl:value-of select="'spare_29_code'" />
						</xsl:attribute>
						<xsl:attribute name="value">
							<xsl:value-of select="'2'" />
						</xsl:attribute>
					</xsl:element>
				</xsl:element> <!-- claim -->
				<!-- confirm -->
				<xsl:element name="confirm">
					<!-- format -->
					<xsl:element name="format">
						<xsl:attribute name="structure">
							<xsl:value-of select="'flat'" />
						</xsl:attribute>
					</xsl:element> <!-- format -->
				</xsl:element> <!-- confirm -->
			</xsl:element> <!-- request -->
		</xsl:element> <!-- Figtree System -->
	</xsl:template>
</xsl:stylesheet>
