<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="acord date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:template match="/acord:OLifE">
		<ClaimDetails xmlns="">
			<xsl:apply-templates select="acord:Holding/acord:Policy" />
			<xsl:apply-templates select="acord:Party" />
		</ClaimDetails>
	</xsl:template>
	<xsl:template match="acord:Holding/acord:Policy">
		<xsl:if test="acord:DisabilityHealth/acord:IncomeAmtCov">
			<IncomeAmtCov>
				<xsl:value-of select="acord:DisabilityHealth/acord:IncomeAmtCov" />
			</IncomeAmtCov>
		</xsl:if>
		<xsl:if test="acord:DisabilityHealth/acord:BenefitMode">
			<PaymentFreq>
				<xsl:value-of select="acord:DisabilityHealth/acord:BenefitMode/@tc" />
			</PaymentFreq>
		</xsl:if>
		<xsl:if test="acord:PolNumber">
			<PolNumber>
				<xsl:value-of select="acord:PolNumber" />
			</PolNumber>
		</xsl:if>
		<xsl:if test="acord:Claim/acord:HOClaimReferenceID">
			<HOClaimReferenceID>
				<xsl:value-of select="acord:Claim/acord:HOClaimReferenceID" />
			</HOClaimReferenceID>
		</xsl:if>
		<xsl:if test="acord:Claim/acord:ClaimDateOfLoss !=''">
			<ClaimDateOfLoss>
				<xsl:value-of select="date:formatdate(acord:Claim/acord:ClaimDateOfLoss,$cvDateFormatyyyy-MM-dd)" />
			</ClaimDateOfLoss>
		</xsl:if>
		<xsl:if test="acord:Claim/acord:LossReportDate !=''">
			<LossReportDate>
				<xsl:value-of select="date:formatdate(acord:Claim/acord:LossReportDate,$cvDateFormatyyyy-MM-dd)" />
			</LossReportDate>
		</xsl:if>
		<xsl:for-each select="acord:KeyedValue">
			<xsl:if test="@id ='NumPriorClaims'">
				<NumPriorClaims>
					<xsl:value-of select="acord:KeyValue" />
				</NumPriorClaims>
			</xsl:if>
		</xsl:for-each>
		<xsl:if test="acord:Claim/acord:CSCClaimExtension/acord:CSCPolicyExtension/acord:QualifyingPeriod">
			<QualifyingPeriod>
				<xsl:value-of select="acord:Claim/acord:CSCClaimExtension/acord:CSCPolicyExtension/acord:QualifyingPeriod" />
			</QualifyingPeriod>
		</xsl:if>
		<xsl:if test="acord:Claim/acord:CSCClaimExtension/acord:CSCPolicyExtension/acord:QualifyingPeriodUnits">
			<QualifyingPeriodUnits>
				<xsl:value-of select="acord:Claim/acord:CSCClaimExtension/acord:CSCPolicyExtension/acord:QualifyingPeriodUnits" />
			</QualifyingPeriodUnits>
		</xsl:if>
		<xsl:for-each select="acord:Claim/acord:CSCClaimExtension/acord:Estimate">
			<xsl:choose>
				<xsl:when test="translate(@id,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'ESTCOSTCLAIM'">
					<EstCostClaim>
						<xsl:value-of select="acord:EstimateAmt" />
					</EstCostClaim>
				</xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
		</xsl:for-each>
		<xsl:for-each select="acord:Claim/acord:CSCClaimExtension/acord:KeyedValue">
			<xsl:choose>
				<xsl:when test="translate(@id,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'ESTIMATEDRETURNTOWORK'">
					<EstRTWDate>
						<xsl:value-of select="acord:KeyValue" />
					</EstRTWDate>
				</xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<xsl:template match="acord:Party">
		<xsl:if test="@id ='Claimant'">
			<xsl:for-each select="acord:Employment">
				<xsl:if test="@id ='CurrentOccupation'">
					<xsl:if test="acord:ReturnDate !=''">
						<ReturnDate>
							<xsl:value-of select="date:formatdate(acord:ReturnDate,$cvDateFormatyyyy-MM-dd)" />
						</ReturnDate>
					</xsl:if>
					<xsl:if test="acord:RetirementDate !=''">
						<CeasedWorkDate>
							<xsl:value-of select="date:formatdate(acord:RetirementDate,$cvDateFormatyyyy-MM-dd)" />
						</CeasedWorkDate>
					</xsl:if>
					<xsl:if test="acord:AnnualSalary">
						<AnnualAmt>
							<xsl:value-of select="acord:AnnualSalary" />
						</AnnualAmt>
					</xsl:if>
				</xsl:if>
			</xsl:for-each>
			<xsl:if test="acord:Person/acord:LastName">
				<CustLastName>
					<xsl:value-of select="acord:Person/acord:LastName" />
				</CustLastName>
			</xsl:if>
			<xsl:if test="acord:Person/acord:FirstName">
				<CustFirstName>
					<xsl:value-of select="acord:Person/acord:FirstName" />
				</CustFirstName>
			</xsl:if>
			<xsl:for-each select="acord:Risk/acord:MedicalCondition/acord:KeyedValue">
				<xsl:if test="@id ='Cause'">
					<Cause>
						<xsl:value-of select="acord:KeyName" />
					</Cause>
				</xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
