<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:acord="http://ACORD.org/Standards/Life/2"
	xmlns:date="http://exslt.org/dates-and-times" exclude-result-prefixes="acord date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:template match="/acord:OLifE">
		<ClaimDetails xmlns="">
			<xsl:apply-templates select="acord:Holding/acord:Policy" />
			<xsl:apply-templates select="acord:Party" />
		</ClaimDetails>
	</xsl:template>
	<xsl:template match="acord:Holding/acord:Policy">
		<!--Policy -->
		<xsl:if test="acord:PolNumber">
			<PolNumber>
				<xsl:value-of select="acord:PolNumber" />
			</PolNumber>
		</xsl:if>
		<xsl:if test="acord:PolType">
			<xsl:choose>
				<xsl:when test="translate(acord:PolType,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'INC'">
					<PolType>
						<xsl:value-of select="'Income Protection'" />
					</PolType>
				</xsl:when>
				<xsl:otherwise>
					<PolType>
						<xsl:value-of select="acord:PolType" />
					</PolType>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
		<!--Claim -->
		<xsl:if test="acord:Claim/acord:HOClaimReferenceID">
			<HOClaimReferenceID>
				<xsl:value-of select="acord:Claim/acord:HOClaimReferenceID" />
			</HOClaimReferenceID>
		</xsl:if>
		<xsl:if test="acord:Claim/acord:ClaimDateOfLoss !=''">
			<ClaimDateOfLoss>
				<xsl:value-of select="date:formatdate(acord:Claim/acord:ClaimDateOfLoss,$cvDateFormatyyyy-MM-dd)" />
			</ClaimDateOfLoss>
		</xsl:if>
		<xsl:if test="acord:Claim/acord:ClaimKey">
			<SWCaseID>
				<xsl:value-of select="acord:ClaimKey" />
			</SWCaseID>
		</xsl:if>
		<xsl:for-each select="acord:Claim/acord:CSCClaimExtension/acord:KeyedValue">
			<xsl:if test="@id ='EstimateReturnToWork'">
				<xsl:if test="acord:KeyValue != ''">
					<EstimateReturnToWork>
						<xsl:value-of select="date:formatdate(acord:KeyValue,$cvDateFormatyyyy-MM-dd)" />
					</EstimateReturnToWork>
				</xsl:if>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	<!--Party -->
	<xsl:template match="acord:Party">
		<xsl:if test="@id ='Claimant'">
			<xsl:if test="acord:PartySysKey">
				<Employee_No>
					<xsl:value-of select="acord:PartySysKey" />
				</Employee_No>
			</xsl:if>
			<xsl:if test="acord:EMailAddress">
				<Email>
					<xsl:value-of select="acord:EMailAddress" />
				</Email>
			</xsl:if>
			<!--Person -->
			<xsl:if test="acord:Person/acord:FirstName">
				<FirstName>
					<xsl:value-of select="acord:Person/acord:FirstName" />
				</FirstName>
			</xsl:if>
			<xsl:if test="acord:Person/acord:MiddleName">
				<MiddleName>
					<xsl:value-of select="acord:Person/acord:MiddleName" />
				</MiddleName>
			</xsl:if>
			<xsl:if test="acord:Person/acord:LastName">
				<Surname>
					<xsl:value-of select="acord:Person/acord:LastName" />
				</Surname>
			</xsl:if>
			<xsl:if test="acord:Person/acord:Title">
				<Title>
					<xsl:value-of select="acord:Person/acord:Title" />
				</Title>
			</xsl:if>
			<xsl:if test="acord:Person/acord:BirthDate != ''">
				<BirthDate>
					<xsl:value-of select="date:formatdate(acord:Person/acord:BirthDate,$cvDateFormatyyyy-MM-dd)" />
				</BirthDate>
			</xsl:if>
			<xsl:if test="acord:Person/acord:Gender">
				<xsl:choose>
					<xsl:when test="acord:Person/acord:Gender = '1'">
						<Sex>
							<xsl:value-of select="'M'" />
						</Sex>
					</xsl:when>
					<xsl:when test="acord:Person/acord:Gender = '2'">
						<Sex>
							<xsl:value-of select="'F'" />
						</Sex>
					</xsl:when>
					<xsl:otherwise>
						<Sex>
							<xsl:value-of select="'Unknown'" />
						</Sex>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
			<!-- Address -->
			<xsl:if test="acord:Address/acord:Line1">
				<AddressLine1>
					<xsl:value-of select="acord:Address/acord:Line1" />
				</AddressLine1>
			</xsl:if>
			<xsl:if test="acord:Address/acord:Line2">
				<AddressLine2>
					<xsl:value-of select="acord:Address/acord:Line2" />
				</AddressLine2>
			</xsl:if>
			<xsl:if test="acord:Address/acord:Line3">
				<AddressCity>
					<xsl:value-of select="acord:Address/acord:Line3" />
				</AddressCity>
			</xsl:if>
			<xsl:if test="acord:Address/acord:AddressState">
				<AddressState>
					<xsl:value-of select="acord:Address/acord:AddressState" />
				</AddressState>
			</xsl:if>
			<xsl:if test="acord:Address/acord:Zip">
				<AddressPostCode>
					<xsl:value-of select="acord:Address/acord:Zip" />
				</AddressPostCode>
			</xsl:if>
			<xsl:if test="acord:Address/acord:AddressCountry">
				<AddressCountry>
					<xsl:value-of select="acord:Address/acord:AddressCountry" />
				</AddressCountry>
			</xsl:if>
			<!-- Phone -->
			<xsl:for-each select="acord:Phone">
				<xsl:choose>
					<xsl:when test="acord:PhoneTypeCode/@tc = '1'">
						<HomePhone>
							<xsl:value-of select="acord:DialNumber" />
						</HomePhone>
					</xsl:when>
					<xsl:when test="acord:PhoneTypeCode/@tc = '12'">
						<Mobile>
							<xsl:value-of select="acord:DialNumber" />
						</Mobile>
					</xsl:when>
					<xsl:otherwise />
				</xsl:choose>
			</xsl:for-each>
			<!-- Employment -->
			<xsl:for-each select="acord:Employment">
				<xsl:if test="@id ='CurrentOccupation'">
					<xsl:if test="acord:Occupation">
						<Occupation>
							<xsl:value-of select="acord:Occupation" />
						</Occupation>
					</xsl:if>
					<xsl:if test="acord:ReturnDate !=''">
						<ReturnDate>
							<xsl:value-of select="date:formatdate(acord:ReturnDate,$cvDateFormatyyyy-MM-dd)" />
						</ReturnDate>
					</xsl:if>
				</xsl:if>
			</xsl:for-each>
			<!--Risk/Medication Condition -->
			<xsl:for-each select="acord:Risk/acord:MedicalCondition/acord:KeyedValue">
				<xsl:if test="@id ='Cause'">
					<Cause>
						<xsl:value-of select="acord:KeyValue" />
					</Cause>
					<CauseCode>
						<xsl:value-of select="acord:KeyName"/>
					</CauseCode>
				</xsl:if>
				<xsl:if test="@id ='ResultofInjury'">
						<ResultofInjury>
							<xsl:value-of select="acord:KeyValue" />
						</ResultofInjury>
				</xsl:if>
				<xsl:if test="@id = 'AccidentDescription'">
					<AccidentDescription>
						<xsl:value-of select="acord:KeyValue"/>
					</AccidentDescription>
				</xsl:if>
				<xsl:if test="@id ='PartOfBody'">
					<PartOfBodyInjured>
						<xsl:value-of select="acord:KeyValue"/>
					</PartOfBodyInjured>
					<PartOfBodyInjuredCode>
						<xsl:value-of select="acord:KeyName" />
					</PartOfBodyInjuredCode>
				</xsl:if>
			</xsl:for-each>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
