<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:date="http://exslt.org/dates-and-times"
	exclude-result-prefixes="date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:template match="/Figtree_Systems">
		<xsl:apply-templates select="response/claims" />
	</xsl:template>
	<xsl:template match="response/claims">
		<OLifE xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
			xmlns="http://ACORD.org/Standards/Life/2">
			<Holding>
				<Policy>
					<xsl:if test="claim/policy">
						<PolNumber>
							<xsl:value-of select="claim/policy" />
						</PolNumber>
					</xsl:if>
					<xsl:if test="translate(claim/permanent_or_casual_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'INC' ">
						<ProductType tc="19" />
					</xsl:if>
					<xsl:if test="claim/spare_date_1 !=''">
						<EffDate>
							<xsl:value-of select="date:formatdate(claim/spare_date_1,$cvDateFormatyyyy-MM-dd)" />
						</EffDate>
					</xsl:if>
					<xsl:if test="claim/prevent_recommendn_code != ''">
						<KeyedValue id="COB">
							<KeyName>
								<xsl:value-of select="claim/prevent_recommendn_code" />
							</KeyName>
							<KeyValue>
								<xsl:value-of select="claim/prevent_recommendn_code/@description" />
							</KeyValue>
						</KeyedValue>
					</xsl:if>
					<xsl:if test="claim/spare_num_1">
						<KeyedValue id="NumPriorClaims">
							<KeyName>
								<xsl:value-of select="'NumPriorClaims'" />
							</KeyName>
							<KeyValue>
								<xsl:value-of select="claim/spare_num_1" />
							</KeyValue>
						</KeyedValue>
					</xsl:if>
					<!-- Disability Health -->
					<DisabilityHealth id="DH">
						<xsl:if test="claim/address_number_02 !=''">
							<Participant PartyID="Witness1">
								<ParticipantRoleCode tc="17" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/address_number_09 !=''">
							<Participant PartyID="Witness2">
								<ParticipantRoleCode tc="17" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/address_number_06 !=''">
							<Participant PartyID="Witness3">
								<ParticipantRoleCode tc="17" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/reh_creditor_number !=''">
							<Participant PartyID="ContactPerson1">
								<ParticipantRoleCode tc="5" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/address_number_07 !=''">
							<Participant PartyID="ContactPerson2">
								<ParticipantRoleCode tc="5" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/address_number_03 !=''">
							<Participant PartyID="ContactPerson3">
								<ParticipantRoleCode tc="5" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/spare_28_code !=''">
							<Participant PartyID="CoOrd">
								<ParticipantRoleCode tc="25" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/spare_15_code !=''">
							<Participant PartyID="Assessor">
								<ParticipantRoleCode tc="30" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/address_number_07 !=''">
							<Participant PartyID="Advisor">
								<ParticipantRoleCode tc="15" />
							</Participant>
						</xsl:if>
						<xsl:if test="claim/spare_char_1">
							<Rider>
								<Description>
									<xsl:value-of select="claim/spare_char_1" />
								</Description>
							</Rider>
						</xsl:if>
						<xsl:if test="claim/employee/spare_decimal_2">
							<IncomeAmtCov>
								<xsl:value-of select="claim/employee/spare_decimal_2" />
							</IncomeAmtCov>
						</xsl:if>
						<xsl:if test="claim/employee/spare_4_code != ''">
							<xsl:choose>
								<xsl:when test="translate(claim/employee/spare_4_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'Y'">
									<BenefitMode tc="1" />
								</xsl:when>
								<xsl:when test="translate(claim/employee/spare_4_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'M'">
									<BenefitMode tc="4" />
								</xsl:when>
								<xsl:when test="translate(claim/employee/spare_4_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'W'">
									<BenefitMode tc="6" />
								</xsl:when>
								<xsl:otherwise />
							</xsl:choose>
						</xsl:if>
					</DisabilityHealth>
					<!-- Claim -->
					<xsl:if test="claim">
						<Claim id="Claim" PartyID="PolicyOwner" ActualObjectID="DH" ParticipantObjectID="Claimant"
							ProviderPartyID="InsProvider">
							<HOClaimReferenceID>
								<xsl:value-of select="claim/@incident_number" />
							</HOClaimReferenceID>
							<xsl:if test="claim/accident_date != ''">
								<ClaimDateOfLoss>
									<xsl:value-of select="date:formatdate(claim/accident_date,$cvDateFormatyyyy-MM-dd)" />
								</ClaimDateOfLoss>
							</xsl:if>
							<xsl:if test="claim/spare_char_10">
								<ClaimKey>
									<xsl:value-of select="claim/spare_char_10" />
								</ClaimKey>
							</xsl:if>
							<xsl:if test="claim/registration_date != ''">
								<LossReportDate>
									<xsl:value-of select="date:formatdate(claim/registration_date,$cvDateFormatyyyy-MM-dd)" />
								</LossReportDate>
							</xsl:if>
							<xsl:if test="claim/status">
								<ClaimStatus>
									<xsl:value-of select="claim/status" />
								</ClaimStatus>
							</xsl:if>
							<!-- CSCClaimExtension -->
							<CSCClaimExtension>
								<xsl:if test="claim/finalisation_date != ''">
									<FinalisedDate>
										<xsl:value-of select="date:formatdate(claim/finalisation_date,$cvDateFormatyyyy-MM-dd)" />
									</FinalisedDate>
								</xsl:if>
								<xsl:if test="claim/reh_last_med_cert_date != ''">
									<LastMedicalCertDate>
										<xsl:value-of select="date:formatdate(claim/reh_last_med_cert_date,$cvDateFormatyyyy-MM-dd)" />
									</LastMedicalCertDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_15 != ''">
									<CCFSent>
										<xsl:value-of select="date:formatdate(claim/spare_date_15,$cvDateFormatyyyy-MM-dd)" />
									</CCFSent>
								</xsl:if>
								<xsl:if test="claim/estimated_resume_date != ''">
									<KeyedValue id="EstimateReturnToWork">
										<KeyName>
											<xsl:value-of select="'EstimateReturnToWork'" />
										</KeyName>
										<KeyValue>
											<xsl:value-of select="date:formatdate(claim/estimated_resume_date,$cvDateFormatyyyy-MM-dd)" />
										</KeyValue>
									</KeyedValue>
								</xsl:if>
								<xsl:if test="claim/worksafe_location_code != ''">
									<KeyedValue id="ClaimsTeam">
										<KeyName>
											<xsl:value-of select="claim/worksafe_location_code" />
										</KeyName>
										<KeyValue>
											<xsl:value-of select="claim/worksafe_location_code/@description" />
										</KeyValue>
									</KeyedValue>
								</xsl:if>
								<!-- StatusDetails -->
								<StatusDetails>
									<xsl:if test="claim/reopened_reason_flag">
										<ReopenReason>
											<xsl:value-of select="claim/reopened_reason_flag" />
										</ReopenReason>
									</xsl:if>
									<xsl:if test="claim/reopened_date != ''">
										<ReopenDate>
											<xsl:value-of select="date:formatdate(claim/reopened_date,$cvDateFormatyyyy-MM-dd)" />
										</ReopenDate>
									</xsl:if>
									<xsl:if test="claim/spare_06_code != ''">
										<Decision>
											<xsl:value-of select="claim/spare_06_code" />
										</Decision>
									</xsl:if>
									<xsl:if test="claim/spare_date_10 != ''">
										<DecisionDate>
											<xsl:value-of select="date:formatdate(claim/spare_date_10,$cvDateFormatyyyy-MM-dd)" />
										</DecisionDate>
									</xsl:if>
									<xsl:if test="claim/spare_31_code != ''">
										<FinalDecision>
											<xsl:value-of select="claim/spare_31_code" />
										</FinalDecision>
									</xsl:if>
									<xsl:if test="claim/spare_date_19 != ''">
										<FinalDecisionDate>
											<xsl:value-of select="date:formatdate(claim/spare_date_19,$cvDateFormatyyyy-MM-dd)" />
										</FinalDecisionDate>
									</xsl:if>
								</StatusDetails>
								<!-- Estimate -->
								<xsl:if test="claim/spare_decimal_16">
									<Estimate id="EstCostClaim">
										<EstimateAmt>
											<xsl:value-of select="claim/spare_decimal_16" />
										</EstimateAmt>
									</Estimate>
								</xsl:if>
								<!-- Financial Details -->
								<FinancialDetails>
									<xsl:if test="claim/resumed_work_date != ''">
										<LastPaymentDate>
											<xsl:value-of select="date:formatdate(claim/resumed_work_date,$cvDateFormatyyyy-MM-dd)" />
										</LastPaymentDate>
									</xsl:if>
									<xsl:if test="claim/paid_ltd">
										<PaidToDateAmt>
											<xsl:value-of select="claim/paid_ltd" />
										</PaidToDateAmt>
									</xsl:if>
								</FinancialDetails>
								<CSCPolicyExtension>
									<xsl:if test="claim/spare_decimal_14">
										<BenefitPeriod>
											<xsl:value-of select="claim/spare_decimal_14" />
										</BenefitPeriod>
									</xsl:if>
									<xsl:if test="claim/employee/award_salary">
										<QualifyingPeriod>
											<xsl:value-of select="claim/employee/award_salary" />
										</QualifyingPeriod>
									</xsl:if>
									<xsl:if test="claim/employee/spare_5_code != ''">
										<xsl:choose>
											<xsl:when test="translate(claim/employee/spare_5_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'Y'">
												<QualifyingPeriodUnits>1</QualifyingPeriodUnits>
											</xsl:when>
											<xsl:when test="translate(claim/employee/spare_5_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'M'">
												<QualifyingPeriodUnits>4</QualifyingPeriodUnits>
											</xsl:when>
											<xsl:when test="translate(claim/employee/spare_5_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'W'">
												<QualifyingPeriodUnits>6</QualifyingPeriodUnits>
											</xsl:when>
											<xsl:otherwise />
										</xsl:choose>
									</xsl:if>
								</CSCPolicyExtension>
							</CSCClaimExtension>
						</Claim>
					</xsl:if>
				</Policy>
			</Holding>
			<!-- Claimant Party -->
			<Party id="Claimant">
				<xsl:if test="claim/employee_number">
					<PartySysKey>
						<xsl:value-of select="claim/employee_number" />
					</PartySysKey>
				</xsl:if>
				<xsl:if test="claim/employee/address/email">
					<EMailAddress>
						<xsl:value-of select="claim/employee/address/email" />
					</EMailAddress>
				</xsl:if>
				<xsl:if test="claim/special_1_code != ''">
					<KeyedValue id="TrafficLight">
						<KeyName>
							<xsl:value-of select="claim/special_1_code" />
						</KeyName>
						<KeyValue>
							<xsl:value-of select="claim/special_1_code/@description" />
						</KeyValue>
					</KeyedValue>
				</xsl:if>
				<xsl:if test="claim/editor_04">
					<KeyedValue id="EmploymentSituation">
						<KeyName>
							<xsl:value-of select="'EmploymentSituation'" />
						</KeyName>
						<KeyValue>
							<xsl:value-of select="claim/editor_04" />
						</KeyValue>
					</KeyedValue>
				</xsl:if>
				<!-- Person -->
				<Person>
					<xsl:if test="claim/employee/address">
						<xsl:if test="claim/employee/address/surname">
							<LastName>
								<xsl:value-of select="claim/employee/address/surname" />
							</LastName>
						</xsl:if>
						<!-- If there are two names in the given name, split to firstname and middlename -->
						<xsl:if test="claim/employee/address/given_name">
							<xsl:variable name="vGivenName">
								<xsl:value-of select="normalize-space(claim/employee/address/given_name)" />
							</xsl:variable>
							<xsl:choose>
								<xsl:when test="contains($vGivenName,' ')">
									<FirstName>
										<xsl:value-of select="substring-before($vGivenName,' ')" />
									</FirstName>
									<MiddleName>
										<xsl:value-of select="substring-after($vGivenName,' ')" />
									</MiddleName>
								</xsl:when>
								<xsl:otherwise>
									<FirstName>
										<xsl:value-of select="$vGivenName" />
									</FirstName>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
						<xsl:if test="claim/employee/address/title">
							<Title>
								<xsl:value-of select="claim/employee/address/title" />
							</Title>
						</xsl:if>
					</xsl:if>
					<xsl:if test="claim/employee/birth_date !='' ">
						<BirthDate>
							<xsl:value-of select="date:formatdate(claim/employee/birth_date,$cvDateFormatyyyy-MM-dd)" />
						</BirthDate>
					</xsl:if>
					<xsl:if test="claim/employee/sex">
						<xsl:variable name="vSex">
							<xsl:choose>
								<xsl:when test="translate(claim/employee/sex,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'M' ">
									<xsl:value-of select="'1'" />
								</xsl:when>
								<xsl:when test="translate(claim/employee/sex,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'F' ">
									<xsl:value-of select="'2'" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="'0'" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<Gender>
							<xsl:value-of select="$vSex" />
						</Gender>
					</xsl:if>
				</Person>
				<!-- Address -->
				<xsl:if test="claim/employee/address">
					<Address>
						<xsl:if test="claim/employee/address/address_1">
							<Line1>
								<xsl:value-of select="claim/employee/address/address_1" />
							</Line1>
						</xsl:if>
						<xsl:if test="claim/employee/address/address_2">
							<Line2>
								<xsl:value-of select="claim/employee/address/address_2" />
							</Line2>
						</xsl:if>
						<xsl:if test="claim/employee/address/city_1">
							<City>
								<xsl:value-of select="claim/employee/address/city_1" />
							</City>
						</xsl:if>
						<xsl:if test="claim/employee/address/city_2">
							<AddressState>
								<xsl:value-of select="claim/employee/address/city_2" />
							</AddressState>
						</xsl:if>
						<xsl:if test="claim/employee/address/post">
							<Zip>
								<xsl:value-of select="claim/employee/address/post" />
							</Zip>
						</xsl:if>
						<!-- Blank code is default to Australia -->
						<xsl:if test="claim/employee/address/country_code">
							<AddressCountry>
								<xsl:value-of select="claim/employee/address/country_code/@description" />
							</AddressCountry>
						</xsl:if>
						<xsl:if test="claim/employee/address/date_commenced !='' ">
							<StartDate>
								<xsl:value-of select="date:formatdate(claim/employee/address/date_commenced,$cvDateFormatyyyy-MM-dd)" />
							</StartDate>
						</xsl:if>
						<xsl:if test="claim/employee/address/date_terminated !=''">
							<EndDate>
								<xsl:value-of select="date:formatdate(claim/employee/address/date_terminated,$cvDateFormatyyyy-MM-dd)" />
							</EndDate>
						</xsl:if>
					</Address>
				</xsl:if>
				<!--Phones -->
				<xsl:if test="claim/employee/address/phone != ''">
					<!-- Assumes that the phone field stores the home/main contact number -->
					<Phone>
						<PhoneTypeCode tc="1" />
						<DialNumber>
							<xsl:value-of select="concat(claim/employee/address/phone_prefix,claim/employee/address/phone)" />
						</DialNumber>
					</Phone>
				</xsl:if>
				<xsl:if test="claim/employee/address/fax != ''">
					<!-- Assumes that the fax field stores the mobile number -->
					<Phone>
						<PhoneTypeCode tc="12" />
						<DialNumber>
							<xsl:value-of select="claim/employee/address/fax" />
						</DialNumber>
					</Phone>
				</xsl:if>
				<!-- Employment -->
				<Employment id="CurrentOccupation">
					<xsl:if test="claim/org_code_1 != ''">
						<EmployerName>
							<xsl:value-of select="claim/org_code_1" />
						</EmployerName>
					</xsl:if>
					<xsl:if test="claim/spare_date_22 != ''">
						<HireDate>
							<xsl:value-of select="date:formatdate(claim/spare_date_22,$cvDateFormatyyyy-MM-dd)" />
						</HireDate>
					</xsl:if>
					<xsl:if test="claim/ceased_work_date != ''">
						<RetirementDate>
							<xsl:value-of select="date:formatdate(claim/ceased_work_date,$cvDateFormatyyyy-MM-dd)" />
						</RetirementDate>
					</xsl:if>
					<xsl:if test="claim/spare_12_code != ''">
						<xsl:choose>
							<xsl:when test="translate(claim/spare_12_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'Y'">
								<SalaryMode tc="1" />
							</xsl:when>
							<xsl:when test="translate(claim/spare_12_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'M'">
								<SalaryMode tc="4" />
							</xsl:when>
							<xsl:when test="translate(claim/spare_12_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'W'">
								<SalaryMode tc="6" />
							</xsl:when>
							<xsl:otherwise />
						</xsl:choose>
					</xsl:if>
					<xsl:if test="claim/spare_decimal_4">
						<AnnualSalary>
							<xsl:value-of select="claim/spare_decimal_4" />
						</AnnualSalary>
					</xsl:if>
					<xsl:if test="claim/spare_date_24 != ''">
						<TerminationDate>
							<xsl:value-of select="date:formatdate(claim/spare_date_24,$cvDateFormatyyyy-MM-dd)" />
						</TerminationDate>
					</xsl:if>
					<xsl:if test="claim/spare_21_code != ''">
						<EmploymentKey>
							<xsl:value-of select="claim/spare_21_code" />
						</EmploymentKey>
						<Occupation>
							<xsl:value-of select="claim/spare_21_code/@description" />
						</Occupation>
					</xsl:if>
					<xsl:if test="claim/spare_date_20 != ''">
						<ReturnDate>
							<xsl:value-of select="date:formatdate(claim/spare_date_20,$cvDateFormatyyyy-MM-dd)" />
						</ReturnDate>
					</xsl:if>
				</Employment>
				<xsl:if test="claim/occupation_code !='' or claim/spare_date_25 !='' or claim/spare_date_26 != ''">
					<Employment id="PreviousOccupation1">
						<xsl:if test="claim/occupation_code != ''">
							<EmploymentKey>
								<xsl:value-of select="claim/occupation_code" />
							</EmploymentKey>
							<Occupation>
								<xsl:value-of select="claim/occupation_code/@description" />
							</Occupation>
						</xsl:if>
						<xsl:if test="claim/spare_date_25 != ''">
							<HireDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_25,$cvDateFormatyyyy-MM-dd)" />
							</HireDate>
						</xsl:if>
						<xsl:if test="claim/spare_date_26 != ''">
							<TerminationDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_26,$cvDateFormatyyyy-MM-dd)" />
							</TerminationDate>
						</xsl:if>
					</Employment>
				</xsl:if>
				<xsl:if test="claim/spare_20_code !='' or claim/spare_date_27 !='' or claim/spare_date_28 !=''">
					<Employment id="PreviousOccupation2">
						<xsl:if test="claim/spare_20_code != ''">
							<EmploymentKey>
								<xsl:value-of select="claim/spare_20_code" />
							</EmploymentKey>
							<Occupation>
								<xsl:value-of select="claim/spare_20_code/@description" />
							</Occupation>
						</xsl:if>
						<xsl:if test="claim/spare_date_27 !=''">
							<HireDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_27,$cvDateFormatyyyy-MM-dd)" />
							</HireDate>
						</xsl:if>
						<xsl:if test="claim/spare_date_28 !=''">
							<TerminationDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_28,$cvDateFormatyyyy-MM-dd)" />
							</TerminationDate>
						</xsl:if>
					</Employment>
				</xsl:if>
				<!-- Attactment -->
				<xsl:if test="claim/image_id">
					<Attachment>
						<AttachmentSource>
							<xsl:value-of select="claim/image_id" />
						</AttachmentSource>
						<Description>Disability Claim Form</Description>
						<ImageType tc="2" />
						<AttachmentType tc="5" />
					</Attachment>
				</xsl:if>
				<!-- Risk -->
				<Risk id="Risk1">
					<!-- Medical Condition -->
					<MedicalCondition id="MedC1" PrimaryPhysicianID="Doctor1">
						<xsl:if test="claim/cause_code !=''">
							<KeyedValue id="Cause">
								<KeyName>
									<xsl:value-of select="claim/cause_code" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/cause_code/@description" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/editor_02">
							<KeyedValue id="ResultofInjury">
								<KeyName>
									<xsl:value-of select="'ResultofInjury'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/editor_02" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/accident_description">
							<KeyedValue id="AccidentDescription">
								<KeyName>
									<xsl:value-of select="'AccidentDescription'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/accident_description" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/location_code != ''">
							<KeyedValue id="PartOfBody">
								<KeyName>
									<xsl:value-of select="claim/location_code" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/location_code/@description" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/editor_06">
							<KeyedValue id="Location">
								<KeyName>
									<xsl:value-of select="'Location'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/editor_06" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/spare_char_13">
							<KeyedValue id="TreatmentType">
								<KeyName>
									<xsl:value-of select="'TreatmentType'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/spare_char_13" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/spare_char_21">
							<KeyedValue id="TreatmentResult">
								<KeyName>
									<xsl:value-of select="'TreatmentResult'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/spare_char_21" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/spare_char_23">
							<KeyedValue id="TreatmentComments">
								<KeyName>
									<xsl:value-of select="'TreatmentComments'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/spare_char_23" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<!--Medical Treatment -->
						<xsl:if test="claim/address_number_10 !=''">
							<MedicalTreatment id="MedT1" TreatmentPhysicianPartyID="Doctor1">
								<xsl:if test="claim/spare_date_29 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_29,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/address_number_05 !=''">
							<MedicalTreatment id="MedT2" TreatmentFacilityPartyID="IME"></MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/spare_char_15 !='' or claim/address_number_04 !=''">
							<MedicalTreatment>
								<xsl:attribute name="id">
									<xsl:value-of select="'MedT3'" />
								</xsl:attribute>
								<xsl:if test="claim/spare_char_15 !=''">
									<xsl:attribute name="TreatmentPhysicianPartyID">
										<xsl:value-of select="'Physician1'" />
									</xsl:attribute>
								</xsl:if>
								<xsl:if test="claim/address_number_04 !=''">
									<xsl:attribute name="TreatmentFacilityPartyID">
										<xsl:value-of select="'Hospital1'" />
									</xsl:attribute>
								</xsl:if>
								<xsl:if test="claim/spare_date_29 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_29,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_21 !=''">
									<DateLastSeen>
										<xsl:value-of select="date:formatdate(claim/spare_date_21,$cvDateFormatyyyy-MM-dd)" />
									</DateLastSeen>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/spare_char_22 !='' or claim/address_number_01 !=''">
							<MedicalTreatment>
								<xsl:attribute name="id">
									<xsl:value-of select="'MedT4'" />
								</xsl:attribute>
								<xsl:if test="claim/spare_char_22 !=''">
									<xsl:attribute name="TreatmentPhysicianPartyID">
										<xsl:value-of select="'Physician2'" />
									</xsl:attribute>
								</xsl:if>
								<xsl:if test="claim/address_number_01 !=''">
									<xsl:attribute name="TreatmentFacilityPartyID">
										<xsl:value-of select="'Hospital2'" />
									</xsl:attribute>
								</xsl:if>
								<xsl:if test="claim/spare_date_30 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_30,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_31 !=''">
									<DateLastSeen>
										<xsl:value-of select="date:formatdate(claim/spare_date_31,$cvDateFormatyyyy-MM-dd)" />
									</DateLastSeen>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/spare_char_24 !='' or claim/address_number_08 !=''">
							<MedicalTreatment>
								<xsl:attribute name="id">
									<xsl:value-of select="'MedT5'" />
								</xsl:attribute>
								<xsl:if test="claim/spare_char_24 !=''">
									<xsl:attribute name="TreatmentPhysicianPartyID">
										<xsl:value-of select="'Physician3'" />
									</xsl:attribute>
								</xsl:if>
								<xsl:if test="claim/address_number_08 !=''">
									<xsl:attribute name="TreatmentFacilityPartyID">
										<xsl:value-of select="'Hospital3'" />
									</xsl:attribute>
								</xsl:if>
								<xsl:if test="claim/spare_date_32 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_32,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_33 !=''">
									<DateLastSeen>
										<xsl:value-of select="date:formatdate(claim/spare_date_33,$cvDateFormatyyyy-MM-dd)" />
									</DateLastSeen>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
					</MedicalCondition>
				</Risk>
			</Party>
			<!-- Witness Party -->
			<xsl:if test="claim/address_number_02 !=''">
				<Party id="Witness1">
					<FullName>
						<xsl:value-of select="claim/address_number_02/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_02" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<xsl:if test="claim/address_number_09 !=''">
				<Party id="Witness2">
					<FullName>
						<xsl:value-of select="claim/address_number_09/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_09" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<xsl:if test="claim/address_number_06 !=''">
				<Party id="Witness3">
					<FullName>
						<xsl:value-of select="claim/address_number_06/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_06" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<!-- Contact Party -->
			<xsl:if test="claim/reh_creditor_number !=''">
				<Party id="ContactPerson1">
					<FullName>
						<xsl:value-of select="claim/reh_creditor_number/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/reh_creditor_number" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<xsl:if test="claim/address_number_07 !=''">
				<Party id="ContactPerson2">
					<FullName>
						<xsl:value-of select="claim/address_number_07/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_07" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<xsl:if test="claim/address_number_03 !=''">
				<Party id="ContactPerson3">
					<FullName>
						<xsl:value-of select="claim/address_number_03/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_03" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<!-- Treating Doctor Party -->
			<xsl:if test="claim/address_number_10 !=''">
				<Party id="Doctor1">
					<FullName>
						<xsl:value-of select="claim/address_number_10/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_10" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<!-- Hospital Party -->
			<xsl:if test="claim/address_number_04 !=''">
				<Party id="Hospital1">
					<PartyTypeCode tc="2" />
					<FullName>
						<xsl:value-of select="claim/address_number_04/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_04" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<xsl:if test="claim/address_number_01 !=''">
				<Party id="Hospital2">
					<PartyTypeCode tc="2" />
					<FullName>
						<xsl:value-of select="claim/address_number_01/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_01" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<xsl:if test="claim/address_number_08 !=''">
				<Party id="Hospital3">
					<PartyTypeCode tc="2" />
					<FullName>
						<xsl:value-of select="claim/address_number_08/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_08" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<!-- Physician Party -->
			<xsl:if test="claim/spare_char_15 !=''">
				<Party id="Physcian1">
					<FullName>
						<xsl:value-of select="claim/spare_char_15" />
					</FullName>
					<PartyTypeCode tc="1" />
				</Party>
			</xsl:if>
			<xsl:if test="claim/spare_char_22 !=''">
				<Party id="Physcian2">
					<FullName>
						<xsl:value-of select="claim/spare_char_22" />
					</FullName>
					<PartyTypeCode tc="1" />
				</Party>
			</xsl:if>
			<xsl:if test="claim/spare_char_24 !=''">
				<Party id="Physcian3">
					<FullName>
						<xsl:value-of select="claim/spare_char_24" />
					</FullName>
					<PartyTypeCode tc="1" />
				</Party>
			</xsl:if>
			<!-- Ind. Med. Exam.-->
			<xsl:if test="claim/address_number_05 !=''">
				<Party id="IME">
					<FullName>
						<xsl:value-of select="claim/address_number_13/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_13" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<!-- Advisor Party -->
			<xsl:if test="claim/address_number_07 !=''">
				<Party id="Advisor">
					<FullName>
						<xsl:value-of select="claim/address_number_07/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/address_number_07" />
					</PartySysKey>
				</Party>
			</xsl:if>
			<!-- Coordinator Party -->
			<xsl:if test="claim/spare_28_code !=''">
				<Party id="CoOrd">
					<FullName>
						<xsl:value-of select="claim/spare_28_code/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/spare_28_code" />
					</PartySysKey>
					<xsl:if test="claim/reh_spare_1_code != ''">
						<Phone>
							<DialNumber>
								<xsl:value-of select="claim/reh_spare_1_code/@description" />
							</DialNumber>
						</Phone>
					</xsl:if>
				</Party>
			</xsl:if>
			<!-- Assessor -->
			<xsl:if test="claim/spare_15_code !=''">
				<Party id="Assessor">
					<FullName>
						<xsl:value-of select="claim/spare_15_code/@description" />
					</FullName>
					<PartySysKey>
						<xsl:value-of select="claim/spare_15_code" />
					</PartySysKey>
					<xsl:if test="claim/reh_spare_2_code != ''">
						<Phone>
							<DialNumber>
								<xsl:value-of select="claim/reh_spare_2_code/@description" />
							</DialNumber>
						</Phone>
					</xsl:if>
				</Party>
			</xsl:if>
			<!-- Insurance Provider -->
			<xsl:if test="claim/site_code !=''">
				<Party id="InsProvider">
					<FullName>
						<xsl:value-of select="claim/site_code/@description" />
					</FullName>
					<Organisation>
						<OrgCode>
							<xsl:value-of select="claim/site_code" />
						</OrgCode>
					</Organisation>
				</Party>
			</xsl:if>
		</OLifE>
	</xsl:template>
</xsl:stylesheet>
