<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:date="http://exslt.org/dates-and-times"
	exclude-result-prefixes="date">
	<xsl:import href="CommonVariables.xsl" />
	<xsl:template match="/Figtree_Systems">
		<xsl:apply-templates select="response/claims" />
	</xsl:template>
	<xsl:template match="response/claims">
		<OLifE xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
			xmlns="http://ACORD.org/Standards/Life/2">
			<Holding>
				<Policy>
					<xsl:if test="claim/policy">
						<PolNumber>
							<xsl:value-of select="claim/policy" />
						</PolNumber>
					</xsl:if>
					<xsl:if test="translate(claim/permanent_or_casual_code,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'INC' ">
						<ProductType tc="19" />
					</xsl:if>
					<!-- Claim -->
					<xsl:if test="claim">
						<Claim id="Claim" PartyID="PolicyOwner" ActualObjectID="DH" ParticipantObjectID="Claimant"
							ProviderPartyID="InsProvider">
							<HOClaimReferenceID>
								<xsl:value-of select="claim/@incident_number" />
							</HOClaimReferenceID>
							<xsl:if test="claim/accident_date != ''">
								<ClaimDateOfLoss>
									<xsl:value-of select="date:formatdate(claim/accident_date,$cvDateFormatyyyy-MM-dd)" />
								</ClaimDateOfLoss>
							</xsl:if>
							<xsl:if test="claim/spare_char_10">
								<ClaimKey>
									<xsl:value-of select="claim/spare_char_10" />
								</ClaimKey>
							</xsl:if>
						</Claim>
					</xsl:if>
				</Policy>
			</Holding>
			<!-- Claimant Party -->
			<Party id="Claimant">
				<xsl:if test="claim/employee/address/given_name or claim/employee/address/surname">
					<FullName>
						<xsl:value-of select="concat(claim/employee/address/given_name,' ',claim/employee/address/surname)" />
					</FullName>
				</xsl:if>
				<xsl:if test="claim/employee_number">
					<PartySysKey>
						<xsl:value-of select="claim/employee_number" />
					</PartySysKey>
				</xsl:if>
				<xsl:if test="claim/editor_04">
					<KeyedValue id="EmpSit">
						<KeyName>
							<xsl:value-of select="'EmploymentSituation'" />
						</KeyName>
						<KeyValue>
							<xsl:value-of select="claim/editor_04" />
						</KeyValue>
					</KeyedValue>
				</xsl:if>
				<!-- Person -->
				<Person>
					<xsl:if test="claim/employee/address">
						<xsl:if test="claim/employee/address/surname">
							<LastName>
								<xsl:value-of select="claim/employee/address/surname" />
							</LastName>
						</xsl:if>
						<!-- If there are two names in the given name, split to firstname and middlename -->
						<xsl:if test="claim/employee/address/given_name">
							<xsl:variable name="vGivenName">
								<xsl:value-of select="normalize-space(claim/employee/address/given_name)" />
							</xsl:variable>
							<xsl:choose>
								<xsl:when test="contains($vGivenName,' ')">
									<FirstName>
										<xsl:value-of select="substring-before($vGivenName,' ')" />
									</FirstName>
									<MiddleName>
										<xsl:value-of select="substring-after($vGivenName,' ')" />
									</MiddleName>
								</xsl:when>
								<xsl:otherwise>
									<FirstName>
										<xsl:value-of select="$vGivenName" />
									</FirstName>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:if>
						<xsl:if test="claim/employee/address/title">
							<Title>
								<xsl:value-of select="claim/employee/address/title" />
							</Title>
						</xsl:if>
					</xsl:if>
					<xsl:if test="claim/employee/birth_date !='' ">
						<BirthDate>
							<xsl:value-of select="date:formatdate(claim/employee/birth_date,$cvDateFormatyyyy-MM-dd)" />
						</BirthDate>
					</xsl:if>
					<xsl:if test="claim/employee/sex">
						<xsl:variable name="vSex">
							<xsl:choose>
								<xsl:when test="translate(claim/employee/sex,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'M' ">
									<xsl:value-of select="'1'" />
								</xsl:when>
								<xsl:when test="translate(claim/employee/sex,$cvLowerCaseLetters,$cvUpperCaseLetters) = 'F' ">
									<xsl:value-of select="'2'" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="'0'" />
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<Gender>
							<xsl:value-of select="$vSex" />
						</Gender>
					</xsl:if>
				</Person>
				<!-- Address -->
				<xsl:if test="claim/employee/address">
					<Address>
						<xsl:if test="claim/employee/address/address_1">
							<Line1>
								<xsl:value-of select="claim/employee/address/address_1" />
							</Line1>
						</xsl:if>
						<xsl:if test="claim/employee/address/address_2">
							<Line2>
								<xsl:value-of select="claim/employee/address/address_2" />
							</Line2>
						</xsl:if>
						<xsl:if test="claim/employee/address/city_1">
							<City>
								<xsl:value-of select="claim/employee/address/city_1" />
							</City>
						</xsl:if>
						<xsl:if test="claim/employee/address/city_2">
							<AddressState>
								<xsl:value-of select="claim/employee/address/city_2" />
							</AddressState>
						</xsl:if>
						<xsl:if test="claim/employee/address/post">
							<Zip>
								<xsl:value-of select="claim/employee/address/post" />
							</Zip>
						</xsl:if>
						<xsl:if test="claim/employee/address/country_code">
							<AddressCountry>
								<xsl:value-of select="claim/employee/address/country_code/@description" />
							</AddressCountry>
						</xsl:if>
					</Address>
				</xsl:if>
				<!--Phones -->
				<xsl:if test="claim/employee/address/phone != ''">
					<!-- Assumes that the phone field stores the home/main contact number -->
					<Phone>
						<PhoneTypeCode tc="1" />
						<DialNumber>
							<xsl:value-of select="concat(claim/employee/address/phone_prefix,claim/employee/address/phone)" />
						</DialNumber>
					</Phone>
				</xsl:if>
				<xsl:if test="claim/employee/address/fax != ''">
					<!-- Assumes that the fax field stores the mobile number -->
					<Phone>
						<PhoneTypeCode tc="12" />
						<DialNumber>
							<xsl:value-of select="claim/employee/address/fax" />
						</DialNumber>
					</Phone>
				</xsl:if>
				<!-- Employment -->
				<Employment id="CurrentOccupation">
					<xsl:if test="claim/spare_date_22 != ''">
						<HireDate>
							<xsl:value-of select="date:formatdate(claim/spare_date_22,$cvDateFormatyyyy-MM-dd)" />
						</HireDate>
					</xsl:if>
					<xsl:if test="claim/ceased_work_date != ''">
						<RetirementDate>
							<xsl:value-of select="date:formatdate(claim/ceased_work_date,$cvDateFormatyyyy-MM-dd)" />
						</RetirementDate>
					</xsl:if>
					<xsl:if test="claim/spare_21_code !=''">
						<EmploymentKey>
							<xsl:value-of select="claim/spare_21_code" />
						</EmploymentKey>
						<Occupation>
							<xsl:value-of select="claim/spare_21_code/@description" />
						</Occupation>
					</xsl:if>
					<xsl:if test="claim/spare_date_20 != ''">
						<ReturnDate>
							<xsl:value-of select="date:formatdate(claim/spare_date_20,$cvDateFormatyyyy-MM-dd)" />
						</ReturnDate>
					</xsl:if>
				</Employment>
				<xsl:if test="claim/occupation_code !='' or claim/spare_date_25 !='' or claim/spare_date_26 != ''">
					<Employment id="PreviousOccupation1">
						<xsl:if test="claim/occupation_code !=''">
							<EmploymentKey>
								<xsl:value-of select="claim/occupation_code" />
							</EmploymentKey>
							<Occupation>
								<xsl:value-of select="claim/occupation_code/@description" />
							</Occupation>
						</xsl:if>
						<xsl:if test="claim/spare_date_25 != ''">
							<HireDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_25,$cvDateFormatyyyy-MM-dd)" />
							</HireDate>
						</xsl:if>
						<xsl:if test="claim/spare_date_26 != ''">
							<TerminationDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_26,$cvDateFormatyyyy-MM-dd)" />
							</TerminationDate>
						</xsl:if>
					</Employment>
				</xsl:if>
				<xsl:if test="claim/spar_20_code !='' or claim/spare_date_27 !='' or claim/spare_date_28 !=''">
					<Employment id="PreviousOccupation2">
						<xsl:if test="claim/spare_20_code !=''">
							<EmploymentKey>
								<xsl:value-of select="claim/spare_20_code" />
							</EmploymentKey>
							<Occupation>
								<xsl:value-of select="claim/spare_20_code/@description" />
							</Occupation>
						</xsl:if>
						<xsl:if test="claim/spare_date_27 !=''">
							<HireDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_27,$cvDateFormatyyyy-MM-dd)" />
							</HireDate>
						</xsl:if>
						<xsl:if test="claim/spare_date_28 !=''">
							<TerminationDate>
								<xsl:value-of select="date:formatdate(claim/spare_date_28,$cvDateFormatyyyy-MM-dd)" />
							</TerminationDate>
						</xsl:if>
					</Employment>
				</xsl:if>
				<!-- Risk -->
				<Risk id="Risk1">
					<!-- Medical Condition -->
					<MedicalCondition id="MedC1" PrimaryPhysicianID="Doctor1">
						<xsl:if test="claim/accident_date !=''">
							<ConditionOnsetDate>
								<xsl:value-of select="date:formatdate(claim/accident_date,$cvDateFormatyyyy-MM-dd)" />
							</ConditionOnsetDate>
						</xsl:if>
						<xsl:if test="claim/editor_01">
							<ExamineeDesc>
								<xsl:value-of select="claim/editor_01" />
							</ExamineeDesc>
						</xsl:if>
						<xsl:if test="claim/cause_code !=''">
							<KeyedValue id="Cause">
								<KeyName>
									<xsl:value-of select="claim/cause_code" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/cause_code/@description" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/editor_02">
							<KeyedValue id="ResultInj">
								<KeyName>
									<xsl:value-of select="'ResultofInjury'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/editor_02" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/accident_description">
							<KeyedValue id="AccidentDescription">
								<KeyName>
									<xsl:value-of select="'AccidentDescription'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/accident_description" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/location_code !=''">
							<KeyedValue id="PartOfBody">
								<KeyName>
									<xsl:value-of select="claim/location_code" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/location_code/@description" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<xsl:if test="claim/editor_06">
							<KeyedValue id="Location">
								<KeyName>
									<xsl:value-of select="'Location'" />
								</KeyName>
								<KeyValue>
									<xsl:value-of select="claim/editor_06" />
								</KeyValue>
							</KeyedValue>
						</xsl:if>
						<!--Medical Treatment -->
						<xsl:if test="claim/address_number_10 !=''">
							<MedicalTreatment id="MedT1">
								<xsl:if test="claim/spare_date_29 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_29,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/address_number_04 !=''">
							<MedicalTreatment id="MedT2">
								<xsl:if test="claim/spare_date_3 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_3,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_21 !=''">
									<DateLastSeen>
										<xsl:value-of select="date:formatdate(claim/spare_date_21,$cvDateFormatyyyy-MM-dd)" />
									</DateLastSeen>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/address_number_01 !=''">
							<MedicalTreatment id="MedT3">
								<xsl:if test="claim/spare_date_30 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_30,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_31 !=''">
									<DateLastSeen>
										<xsl:value-of select="date:formatdate(claim/spare_date_31,$cvDateFormatyyyy-MM-dd)" />
									</DateLastSeen>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
						<xsl:if test="claim/address_number_08 !=''">
							<MedicalTreatment id="MedT4">
								<xsl:if test="claim/spare_date_32 !=''">
									<TreatmentStartDate>
										<xsl:value-of select="date:formatdate(claim/spare_date_32,$cvDateFormatyyyy-MM-dd)" />
									</TreatmentStartDate>
								</xsl:if>
								<xsl:if test="claim/spare_date_33 !=''">
									<DateLastSeen>
										<xsl:value-of select="date:formatdate(claim/spare_date_33,$cvDateFormatyyyy-MM-dd)" />
									</DateLastSeen>
								</xsl:if>
							</MedicalTreatment>
						</xsl:if>
					</MedicalCondition>
				</Risk>
			</Party>
		</OLifE>
	</xsl:template>
</xsl:stylesheet>
