'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
Imports System.Xml
Imports System.IO

Public Enum eEnvelopeStatus
    eSuccess = 0
    eFailed = 1
    eErrors = 2
    eWarnings = 4
    eInconsistentEnvelope = 8
End Enum

Public Interface IEnvelope
    Inherits IEnvelopeProvider

    'Implemented in EnvelopeBase
    Sub SwapToAndFromAddresses()
    Sub SetInternalEnvelope(byval oEnv As IEnvelopeProvider)
    Function GetInternalEnvelope() As IEnvelopeProvider
    Property DefaultEnvelopeType() As String
    Property Doc(Optional ByVal tDocumentName As Type = Nothing, Optional ByVal lIndex As Long = -1) As String

    'Implemented in Envelope
    Overloads Function GetDocument(ByVal tDocumentName As Type, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
    Overloads Function GetDocument(ByVal tDocumentName As Type, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
    Overloads Function GetDocument(ByVal tDocumentName As Type, ByVal lIndex As Long, ByRef lReturnedDocIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
    Overloads Function GetDocumentAsObject(ByVal tDocumentName As Type, Optional ByVal bCopyDocument As Boolean = False) As Object
    Overloads Function GetDocumentAsObject(ByVal tDocumentName As Type, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As Object
    Overloads Function PrettyFormatXML(ByVal oStream As MemoryStream, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String
    Overloads Function PrettyFormatXML(ByVal sXML As String, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String
    ReadOnly Property MessageType() As String
    ReadOnly Property ResolveName() As String

    Property XML() As String


End Interface

Public Interface IInlineDocument
    Function GetAsXMLDocument(Optional ByVal bCopyDocument As Boolean = False) As XmlDocument
    Function GetAsXMLElement(Optional ByVal bCopyDocument As Boolean = False) As XmlElement
    Function GetAsXMLNode(Optional ByVal bCopyDocument As Boolean = False) As XmlNode
    Function GetAsObject(ByVal tType As Type, Optional ByVal bCopyDocument As Boolean = False) As Object
    Function GetAsObject(Optional ByVal bCopyDocument As Boolean = False) As Object
    Function GetAsString(Optional ByVal bCopyDocument As Boolean = False) As String
    Function GetAsStream(Optional ByVal bCopyDocument As Boolean = False) As System.IO.Stream
    Function GetAsDataSet(Optional ByVal bCopyDocument As Boolean = False) As System.Data.DataSet
    Function SetDocument(ByVal oXMLDocument As XmlDocument, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function SetDocument(ByVal oXMLElement As XmlElement, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function SetDocument(ByVal oXMLNode As XmlNode, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function SetDocument(ByVal obj As Object, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function SetDocument(ByVal sXMLDoc As String, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function SetDocument(ByVal oStream As System.IO.Stream, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function SetDocument(ByVal oDataSet As System.Data.DataSet, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function ToString() As String
End Interface

Public Interface IEnvelopeProvider : Inherits ICloneable
    Function CreateResponse() As IEnvelope
  
    '++Routing

    'Declare these properties with the XmlIgnoreAttribute in the implementing envelope classes...
    '++ These properties serve as the primary key for the PipelineManager to do matches on
    'Strictly speaking, they should probably be in another interface, but in order to keep
    'things a bit simple, they've been left like this...
    ReadOnly Property XMLSchemaName() As String
    ReadOnly Property Context() As String 'Pipeline/Derived Total Request Context
    Property MessageContext() As String 'Envelope Context
    '-- These properties serve as the primary key for the PipelineManager to do matches on
    ReadOnly Property XMLSchemaNamespace() As String

    Property ConversationID() As String
    Property MessageID() As String
    Property InternalMessageID() As String

    Property ToAddress() As String
    Property ToAddressType() As String
    Property FromAddress() As String
    Property FromAddressType() As String
    Property Environment() As String
    Property SystemName() As String
    Property SystemType() As String
    Property SystemVersion() As String
    Property ServiceName() As String
    Property ServiceNamespace() As String
    Property ServiceVersion() As String
    Property CallSynchronizationType() As String
    Property FromName() As String
    Property ToName() As String
    Property Channel() As String
    

    '--Routing


    '++General Envelope Manipulation
    '++Manifest
    Function AddDocument(byval oEnvDoc As IInlineDocument, Optional ByVal bCopyDocument As Boolean = False) As Long
    Function AddDocument(byval oEnvDoc As IInlineDocument, ByVal lDocumentIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As Long

    Function GetDocument(byval sDocumentName As String, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
    Function GetDocument(ByVal sDocumentName As String, ByVal lIndex As Long, ByRef lReturnedDocIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
    Function GetDocument(byval sDocumentName As String, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
    Function GetDocument(ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument


    Function DocumentCount() As Long

    Function AddAttachment(Optional ByVal sAttachmentName As String = "", Optional ByVal sAttachmentPath As String = "", Optional ByVal lIndex As Long = 0, Optional ByVal sType As String = "", Optional ByVal sUrl As String = "") As Long
    Function GetAttachment(byval lIndex As Long) As IAttachment
    Function GetAttachments() As System.Collections.Generic.IList(Of IAttachment)
    ReadOnly Property AttachmentsExist() As Boolean

    Function GetDocuments() As System.Collections.Generic.IList(Of IInlineDocument)
    Function GetCustomParams() As IList
    Sub ClearCustomParams()

    '++CopyDocumentSectionAcrossNamespaces
    Function CopyDocumentSection(byval FromDoc1 As Object, byval tFromDoc1Type As Type, byval ToDoc2 As Object, byval tTODoc2Type As Type) As Boolean
    '--CopyDocumentSectionAcrossNamespaces

    '--Manifest

    Sub SetCustomParam(ByVal sParamName As String, ByVal sParamValue As String)
    Sub SetCustomParam(ByVal sParamNodeName As String, ByVal sParamName As String, ByVal sParamValue As String)
    Function GetCustomParam(ByVal sParamName As String) As String
    Function GetCustomParam(ByVal sParamNodeName As String, ByVal sParamName As String) As String

    '++Message and Execution status
    Function IsError() As Boolean
    Function IsError(ByVal sXML As String) As Boolean
    Function IsWarning() As Boolean
    Function IsWarning(ByVal sXML As String) As Boolean
    Function GetMsgStatus() As eEnvelopeStatus
    Function GetMsgStatus(ByVal sXML As String) As eEnvelopeStatus
    '--Message and Execution status


    '--General Envelope Manipulation

    '++Serialization/Conversion
    Function ToXML() As String
    Function ToString() As String 'same as ToXML and XML
    Function ToXMLDocument() As XmlDocument
    Function ToStream() As System.IO.Stream

    Function ToESBEnvelope() As IEnvelope 'This is identical to the Clone interface/method...Included for completeness...

    Sub FromXML(ByVal sXML As String)
    Sub FromXMLDocument(ByVal oXML As XmlDocument)
    Sub FromStream(ByVal oStream As System.IO.Stream)

    Function Serialize(ByVal oObjectToSerialize As System.Object) As String
    Function DeSerialize(ByVal sXMLStringToDeSerialize As String, ByVal tType As Type) As Object

    '++XML Manipulation

    Function GetXMLAsString(ByVal oDoc As XmlDocument) As String
    Function PrettyFormatXML(ByVal oStream As System.IO.MemoryStream, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String
    Function PrettyFormatXML(ByVal sXML As String, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String
    '--XML Manipulation

    '--Serialization/Conversion




    '++Message Details

    '--Message Details

    '++Envelope Detection - new interface??? Not for now...
    'Function GetEnvelopeType(Optional byval sXML As String = "", Optional byval oEnvOut As Envelope = Nothing) As eEnvelopeType
    Function GetEnvelopeType(Optional ByVal sXML As String = "", Optional ByRef oEnvOut As IEnvelopeProvider = Nothing) As String
    Function IsCorrectEnvelopeType(ByVal sXML As String) As Boolean
    Function GetEnvelopeNodeName(Optional ByVal sXML As String = "") As Boolean
    Function GetEnvelopeNameSpace(Optional ByVal sXML As String = "") As Boolean
    Function GetEnvelopeVersion(Optional ByVal sXML As String = "") As Boolean
    '--Envelope Detection - new interface??? Not for now...

   
End Interface


Public Interface IAttachment
    '<remarks/>

    Property index() As String
    '<remarks/>
    Property filename() As String
    '<remarks/>
    Property description() As String
    '<remarks/>
    Property type() As String
    '<remarks/>
    Property url() As String
End Interface