using System;
using System.Xml.XPath;

namespace Exslt
{
	/// <summary>
	/// An iterator over the ExsltNodeList
	/// </summary>
	public class ExsltNodeIterator : XPathNodeIterator 
	{

		#region Private Fields and Properties

		/// <summary>
		/// The list this object iterates over. 
		/// </summary>
		private ExsltNodeList myList; 

		#endregion 
		
		#region Public Fields and Properties

		private int _currentPosition; 

		/// <summary>
		/// Returns the number of items in the list being iterated over. 
		/// </summary>
		public override int Count {
			get { return this.myList.Count;}
		}

		/// <summary>
		/// The 1-based index of the current position of the iterator in the list. 
		/// </summary>
		public override int CurrentPosition {
			get{ return _currentPosition; } 
		}

		public override XPathNavigator Current {
			get { return this.myList[_currentPosition-1]; } 
		}


		#endregion

		#region Constructors

		/// <summary>
		/// We always want one of these initialized with an ExsltNodeList so prevent 
		/// instantiation without one. 
		/// </summary>
		private ExsltNodeIterator(){;}
			

		/// <summary>
		/// Constructor initializes the class. 
		/// </summary>
		/// <param name="list">The list to iterate over.</param>
		internal ExsltNodeIterator(ExsltNodeList list){
		this.myList = list; 
		this._currentPosition = 0; 
		}

		#endregion


		#region Public Methods

		/// <summary>
		/// Moves the XPathNavigator to the next node in the selected set
		/// </summary>
		/// <returns></returns>
		public override bool MoveNext(){
			
			if( (_currentPosition + 1) > this.myList.Count){
				return false; 
			}else{
				this._currentPosition++; 
				return true; 
			}
		
		}

		/// <summary>
		/// Creates a new ExsltNodeIterator
		/// </summary>
		/// <returns>A new ExsltNodeIterator</returns>
		public override XPathNodeIterator Clone(){
			return new ExsltNodeIterator(this.myList); 
		}

		#endregion
	}
}
