'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
Option Strict Off

Imports ESB.Core.XMLSchemaClasses.ESBEnvelope
Imports ESB.Core.Interfaces.Envelope
Imports ESB.Core.Interfaces.ProcessMsg
Imports ESB.Core.Adaptors.Transport.Send


Public Class Sender
    Implements ESB.Core.Interfaces.ProcessMsg.IProcessMsg
    Implements ESB.Core.Interfaces.ProcessMsg.IProcessMsgIfByRef

    Protected m_sUrl As String
    Protected m_sAddressType As String
    Protected m_sClientEndPointName As String

    Public Sub New()

    End Sub

    Public Sub New(ByVal sUrl As String, Optional ByVal sAddressType As String = "", Optional ByVal sClientEndPointName As String = "")
        m_sUrl = sUrl
        m_sAddressType = sAddressType
        m_sClientEndPointName = sClientEndPointName
    End Sub

    '''-----------------------------------------------------------------------------
    ''' <summary>
    ''' URL Precedence is as follows:
    ''' 1)Custom Params have the ultimate say in where the request goes...
    ''' 2)If Custom Params are not set, then Constructor settings are used
    ''' 3)If Constructor is not used, then the Envelope "ToAddress" is used.
    ''' 4)If none of the above are set, then config values are used.
    ''' </summary>
    ''' <param name="oEnvRq"></param>
    ''' <remarks></remarks>
    ''' <history>
    ''' 	[MCasiou] 	5/05/2004	Created
    ''' </history>
    '''-----------------------------------------------------------------------------
    Protected Sub PrepareRequestMessage(ByRef oEnvRq As IEnvelope)

        Dim sExecuteAsync As String = oEnvRq.GetCustomParam(C_sCustomParamESBCore, C_sRunAsyncOnSync)
        If sExecuteAsync = C_sRunAsyncOnSyncTrue Then
            oEnvRq.CallSynchronizationType = C_sRunAsyncOnSyncTrue
        ElseIf sExecuteAsync = C_sRunAsyncOnSyncFalse Then
            oEnvRq.CallSynchronizationType = C_sRunAsyncOnSyncFalse
        End If

        '***Set the WcfClientEndPointName from the value passed into the Constructor of this class..***
        'The Send TransportAdaptor looks for this CustomParam to override any WcfClientEndPointName settings
        'Send TransportAdaptor: 
        'First gets from CustomParams
        'if not there, gets from Config
        'if not there, uses hard-coded default of 'ESBSoapXmlDocTransportSoap'
        If m_sClientEndPointName <> "" Then
            oEnvRq.SetCustomParam(C_sInprocClientEndPointName, m_sClientEndPointName)
        End If


        '***Override the default envelope and constructor URL settings with any
        'specified in the custom params...***
        Dim sCustomParamAddressType As String = oEnvRq.GetCustomParam(C_sCustomParamESBCore, C_sCustomParamOverrideTransportType)
        Dim sCustomParamURL As String = oEnvRq.GetCustomParam(C_sCustomParamESBCore, C_sCustomParamOverrideDestinationServerURL)


        If sCustomParamAddressType <> "" Or sCustomParamURL <> "" Then
            'When forwarding requests off...
            'we potentially need to clear some values...
            ClearRelevantCustomParams(oEnvRq)

            If sCustomParamAddressType <> "" Then
                oEnvRq.ToAddressType = sCustomParamAddressType
            End If

            If sCustomParamURL <> "" Then
                oEnvRq.ToAddress = sCustomParamURL
            End If

            Exit Sub
        End If

        'If the settings have not been set above, then:
        'Override the default envelope URL settings with those
        'specified in the constructor
        If m_sUrl <> "" Or m_sAddressType <> "" Then
            If m_sUrl <> "" Then
                oEnvRq.ToAddress = m_sUrl
            End If

            If m_sAddressType <> "" Then
                oEnvRq.ToAddressType = m_sAddressType
            End If

            Exit Sub
        End If

        If oEnvRq.ToAddress <> "" Then 'Or oEnvRq.ToAddressType <> "" Then
            ' do nothing...only if an address has been set...
            Exit Sub
        End If

        'By this time, if we haven't had an address set, then
        'we'll try & use the config default value.
        'This will happen Regardless of whether ad AddressType has been set.

        'If the settings have not been set above, and they have not already been
        ' set in the envelope then:
        'Read from Configuration

        'Make sure we read in the AddressType before assigning an address, 
        'to the envelope as once we do this, the default address out of the 
        'URL will be returned!
        If oEnvRq.ToAddressType = "" Then
            oEnvRq.ToAddressType = System.Configuration.ConfigurationManager.AppSettings(C_sConfiguredClientTransport)
        End If

        If oEnvRq.ToAddress = "" Then
            oEnvRq.ToAddress = System.Configuration.ConfigurationManager.AppSettings(C_sDestinationServerURL)
        End If



    End Sub
    Protected Sub ClearRelevantCustomParams(ByRef oEnvRq As IEnvelope)

        'Clear all custom params...
        'making sure we preserve the InternalMessageID 
        '(some envelopes store this as a custom param, so we must
        'do this crap stuff...)
        Dim sInternalMessageID As String = oEnvRq.InternalMessageID
        oEnvRq.ClearCustomParams()
        oEnvRq.InternalMessageID = sInternalMessageID

    End Sub
    Protected Sub PrepareResponseMessage(ByRef oEnvRs As IEnvelope)

    End Sub

    Public Sub ProcessMsgAsync(ByVal oEnv As Interfaces.Envelope.IEnvelope) Implements Interfaces.ProcessMsg.IProcessMsg.ProcessMsgAsync
        PrepareRequestMessage(oEnv)
        Dim oTxAdaptor As IProcessMsg = GetSendTransportAdaptor(oEnv.ToAddressType())

        If ShouldLog(LogDefs.eLogLevel.eLOG_INFO) Then : LogMsg(LogDefs.eLogLevel.eLOG_INFO, "Sending request to URL=:" & oEnv.ToAddress & ": using the :" & oEnv.ToAddressType & ": transport.") : End If
        oTxAdaptor.ProcessMsgAsync(oEnv)
        PrepareResponseMessage(oEnv)
    End Sub

    Public Function ProcessMsgSync(ByVal oEnv As Interfaces.Envelope.IEnvelope) As Interfaces.Envelope.IEnvelope Implements Interfaces.ProcessMsg.IProcessMsg.ProcessMsgSync
        'look at the envelope & get the proper transporttype...
        'then get the appropriate transport adaptor client 
        PrepareRequestMessage(oEnv)
        Dim oTxAdaptor As IProcessMsg = GetSendTransportAdaptor(oEnv.ToAddressType())
        If ShouldLog(LogDefs.eLogLevel.eLOG_INFO) Then : LogMsg(LogDefs.eLogLevel.eLOG_INFO, "Sending request to URL=:" & oEnv.ToAddress & ": using the :" & oEnv.ToAddressType & ": transport.") : End If
        If ShouldLog(LogDefs.eLogLevel.eLOG_INFO) Then : LogMsg(LogDefs.eLogLevel.eLOG_DATAXFER, "Sending:" & oEnv.XML) : End If
        Dim oEnvRs As IEnvelope = oTxAdaptor.ProcessMsgSync(oEnv)
        If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(LogDefs.eLogLevel.eLOG_DATAXFER, "Received:" & oEnvRs.XML) : End If
        PrepareResponseMessage(oEnvRs)
        Return oEnvRs
    End Function

    'just accept a string, however we will end up deserializing it & then serializing 
    'it again by the time it gets to the server, so don't bother deserialising it just 
    'to call into this function...
    Public Sub ProcessMsgASync(ByRef sXMLrq As String) Implements Interfaces.ProcessMsg.IProcessMsgIfByRef.ProcessMsgASync
        Dim oEnvRq As New Envelope(sXMLrq)
        ProcessMsgAsync(DirectCast(oEnvRq, IEnvelope))
    End Sub

    Public Sub ProcessMsgSync(ByRef sXMLrq As String, ByRef sXMLrs As String) Implements Interfaces.ProcessMsg.IProcessMsgIfByRef.ProcessMsgSync
        Dim oEnvRq As New Envelope(sXMLrq)
        Dim oEnvRs As Envelope
        oEnvRs = DirectCast(ProcessMsgSync(DirectCast(oEnvRq, IEnvelope)), Envelope)
        sXMLrs = oEnvRs.XML
    End Sub


    Protected Function GetSendTransportAdaptor(ByRef sAddressType As String) As IProcessMsg
        Dim oTxAdaptor As IProcessMsg

        Select Case sAddressType.ToUpper

            Case "SOAP"
                oTxAdaptor = New Wcf.ESBSoapXmlDocTransportWcfSender

            Case "SOAP-WSE3"
                oTxAdaptor = New ESBSoapXmlDocTransportWse3Sender

            Case "HTTP"
                oTxAdaptor = New HTTPSender

            Case "MSMQ"
                oTxAdaptor = New MSMQSender

            Case "MSSQL"
                oTxAdaptor = New MSSQLSender

            Case "INPROC"
                oTxAdaptor = New ESB.Core.Adaptors.Transport.Send.Inproc.ESBSoapXmlDocTransportInProcSender

            Case Else
                oTxAdaptor = Nothing
                LogError(eLogLevel.eLOG_ERROR, "No send transport adaptor available for " & sAddressType)
                Throw New Exception("No send transport adaptor available for " & sAddressType)
        End Select

        Return oTxAdaptor

    End Function

End Class
