'''-----------------------------------------------------------------------------
''' <summary>
''' Copyright Keystroke IT 2003
''' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
''' </summary>
'''-----------------------------------------------------------------------------
Public Class MSMQSender
    Inherits BaseSend


    Protected m_bUseOverrideQPathName As Boolean = False
    Protected m_sOverrideQPathName As String
    Protected m_sMsgLabel As String = ""

    Public Overrides Sub ProcessMsgAsync(ByVal oEnv As IEnvelope)
        ProcessMsgSync(oEnv)
    End Sub

    Public Property UseOverrideQPathName() As Boolean
        Get
            Return m_bUseOverrideQPathName
        End Get
        Set(ByVal Value As Boolean)
            m_bUseOverrideQPathName = Value
        End Set
    End Property

    Public Property OverrideQPathName() As String
        Get
            Return m_sOverrideQPathName
        End Get
        Set(ByVal Value As String)
            m_bUseOverrideQPathName = True
            m_sOverrideQPathName = Value
        End Set
    End Property

    Public Property MsgLabel() As String
        Get
            Return m_sMsgLabel
        End Get
        Set(ByVal Value As String)
            m_sMsgLabel = Value
        End Set
    End Property

    Public Overrides Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope
        'Response envelope???None???Confirmation???Where does this happen again???
        StartMethod()
        MyBase.ProcessMsgSync(oEnv)
        Dim oEnvRs As Envelope
        Try
            Dim oTimer As New ESB.Core.Util.Timer
            oTimer.StartTimer()

            'If m_sMsgLabel = "" Then
            '    m_sMsgLabel = ""
            'End If

            Dim oQueue As New ESB.Core.Services.Messaging
            If m_bUseOverrideQPathName = True Then
                oQueue.AddMsgToQueue(GetMSMQAddressFromEnvelopeAddressUrl(m_sOverrideQPathName), oEnv.XML, m_sMsgLabel)
            Else
                If oEnv.ToAddressType <> "MSMQ" Then
                    Throw New Exception("AddressType not appropriate for this Transport Adaptor")
                End If
                oQueue.AddMsgToQueue(GetMSMQAddressFromEnvelopeAddressUrl(oEnv.ToAddress), oEnv.XML, m_sMsgLabel)
            End If

            oTimer.StopTimer()
            Dim oCtx As ESB.Core.Services.BaseContext = GetBaseContext()
            oCtx.ExecutionTime = oTimer.Duration()

            'Need to return an instance of a Handshake document...
            Dim oHs As New ESB.Core.XMLSchemaClasses.Handshake.Handshake
            oHs.Ack = New Handshake.Ack
            With oHs.Ack.Details
                .Description = "Request has been successfully queued for Async processing."
                '.Timestamp = ESB.Core.Util.Timer.NowAsString
                '.Value = "1"
                .Value = Handshake.Value.Succeeded
            End With
            oEnvRs = oEnv.CreateResponse()
            oEnvRs.AddDocument(New InlineDocument(oHs))
            Return oEnvRs
        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(eLogLevel.eLOG_ERROR, ex.ToString)

            'Need to add an error document so that is shows up as an error...
            Dim oErrorDoc As New ErrorXML.ErrorXML
            oErrorDoc.ErrorEntry = New ErrorXML.ErrorEntryCol
            Dim oError As New ErrorXML.ErrorEntry()
            oError.Code = 1
            oError.Description = ex.ToString
            oError.Source = My.Application.Info.AssemblyName & "." & System.Reflection.MethodInfo.GetCurrentMethod.Name
            oErrorDoc.ErrorEntry.Add(oError)



            Dim oHsDoc As New ESB.Core.XMLSchemaClasses.Handshake.Handshake
            oHsDoc.Nack = New Handshake.Nack
            With oHsDoc.Nack.Details
                .Description = "Failed to queue Request for Async processing. " & ex.ToString
                '.Timestamp = Now
                '.Value = "0"
                .Value = Handshake.Value.Failed

            End With
            oEnvRs = oEnv.CreateResponse()

            oEnvRs.AddDocument(New InlineDocument(oErrorDoc))
            oEnvRs.AddDocument(New InlineDocument(oHsDoc))
            Return oEnvRs
        Finally
            EndMethod()
        End Try

    End Function

    ''' <summary>
    ''' Removes the leading MSMQ: from the Queue name so that it works with the MSMQ name format!
    ''' </summary>
    ''' <param name="sURL"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetMSMQAddressFromEnvelopeAddressUrl(ByRef sURL As String) As String
        'MSMQ:./private$/ResponseQ ==> ./private$/ResponseQ
        Return sURL.Substring(sURL.IndexOf(":") + 1)
    End Function
End Class
