Imports System.Text
Imports System.IO
Imports System.Net
Imports System.Xml
Imports System
Namespace ESB.Core.HTTPClient

	Public Enum eReadyState
	Inherits int
		 Private UNINITIALIZED As  = 0
		 Private LOADING As  = 1
		 Private LOADED As  = 2
		 Private INTERACTIVE As  = 3
		 Private COMPLETED As  = 4
	End Enum 

	Public Class XMLHttpRequest
		Public lgRequest As HttpWebRequest = Nothing
		Private lgResponse As HttpWebResponse = Nothing
		Private lgbIsAsync As Boolean = False
		Private Const Const SOAP_CONTENT_TYPE As String = "text/xml"
		Private lgMsg As String = ""
		Private lgReadyState As eReadyState = eReadyState.UNINITIALIZED

		Public Sub New()
		End Sub

		Public Sub Dispose()
			lgRequest = Nothing
			lgResponse = Nothing
		End Sub

		Event OnReadyStatusChange As EventHandler

		Public Sub Open(ByVal Method As String, ByVal Url As String, ByVal Asynch As Boolean, ByVal User As String, ByVal Password As String)
			If Not (lgRequest Is Nothing) Then
				Throw New InvalidOperationException ("Connection Already open")
			End If
			If Url = "" OrElse Url Is Nothing Then
				Throw New ArgumentNullException ("URL must be specified")
			End If
			Dim uriObj As System.Uri = New System.Uri (Url)
			If Not ((uriObj.Scheme = System.Uri.UriSchemeHttp) OrElse (uriObj.Scheme = System.Uri.UriSchemeHttps)) Then
				Throw New ArgumentOutOfRangeException ("URL Scheme is not http or https")
			End If
			If Method Is Nothing OrElse (Not (Method.ToUpper = "POST") AndAlso Not (Method.ToUpper = "GET") AndAlso Not (Method.ToUpper = "PUT") AndAlso Not (Method.ToUpper = "PROPFIND")) Then
				Throw New ArgumentOutOfRangeException ("Method argument type not defined")
			End If
			lgbIsAsync = Asynch
			lgRequest = CType(WebRequest.CreateDefault(uriObj), HttpWebRequest)
			lgRequest.Method = Method
			If Not (Method = "GET") Then
				lgRequest.ContentType = "text/xml"
			End If
			lgRequest.Credentials = New System.Net.NetworkCredential (User, Password)
			uriObj = Nothing
			lgReadyState = eReadyState.LOADING
		End Sub

		Public Sub SetRequestHeader(ByVal headerName As String, ByVal headerValue As String)
			Try
				If Not (lgReadyState = eReadyState.LOADING) Then
					Throw New InvalidOperationException ("Setting request headers is not allowed at this ReadyState")
				End If
				Select headerName
				Case "Accept"
					lgRequest.Accept = headerValue
					' break
				Case "Connection"
					lgRequest.Connection = headerValue
					' break
				Case "Content-Length"
					lgRequest.ContentLength = Convert.ToInt32(headerValue)
					' break
				Case "Content-Type"
					lgRequest.ContentType = headerValue
					' break
				Case "Expect"
					lgRequest.Expect = headerValue
					' break
				Case "Date"
					Throw New Exception ("These headers are set by the system")
				Case "Host"
					Throw New Exception ("These headers are set by the system")
				Case "Range"
					Throw New Exception ("This header is set with AddRange")
				Case "Referer"
					lgRequest.Referer = headerValue
					' break
				Case "Transfer-Encoding"
					lgRequest.TransferEncoding = headerValue
					' break
				Case "User-Agent"
					lgRequest.UserAgent = headerValue
					' break
				Case Else
					lgRequest.Headers.Add(headerName, headerValue)
					' break
				End Select 
			Catch e As Exception
				Throw New Exception ("Error occurred while setting request headers", e)
			End Try
		End Sub

		Public Function GetResponseHeader(ByVal Header As String) As String
			If lgReadyState = eReadyState.LOADED OrElse lgReadyState = eReadyState.INTERACTIVE OrElse lgReadyState = eReadyState.COMPLETED Then
				Return lgResponse.GetResponseHeader(Header)
			Else
				Throw New InvalidOperationException ("Getting Response Headers forbidden at current ReadyState")
			End If
		End Function

		Public Function GetAllResponseHeaders() As String()
			If lgReadyState = eReadyState.LOADED OrElse lgReadyState = eReadyState.INTERACTIVE OrElse lgReadyState = eReadyState.COMPLETED Then
				Return lgResponse.Headers.AllKeys
			Else
				Throw New InvalidOperationException ("Getting Response Headers forbidden at current ReadyState")
			End If
		End Function

		Public Sub Send(ByVal body As String)
			If Not (lgReadyState = eReadyState.LOADING) Then
				Throw New InvalidOperationException ("Sending a message is not allowed at this ReadyState")
			End If
			If Not (body Is Nothing) Then
				If lgbIsAsync Then
					lgMsg = body
					Dim res As IAsyncResult = lgRequest.BeginGetRequestStream(AddressOf ReqCallback, lgRequest)
					lgReadyState = eReadyState.LOADED
				Else
					Dim stream As StreamWriter = New StreamWriter (lgRequest.GetRequestStream, Encoding.ASCII)
					stream.Write(body)
					stream.Close
					lgResponse = CType(lgRequest.GetResponse, HttpWebResponse)
					lgReadyState = eReadyState.COMPLETED
				End If
			End If
		End Sub

		Public Sub Send(ByVal body As Byte(), ByVal len As Integer)
			If Not (lgReadyState = eReadyState.LOADING) Then
				Throw New InvalidOperationException ("Sending a message is not allowed at this ReadyState")
			End If
			If Not (body Is Nothing) Then
				lgResponse = CType(lgRequest.GetResponse, HttpWebResponse)
				lgReadyState = eReadyState.COMPLETED
			End If
		End Sub

		Public Sub Abort()
			lgRequest = Nothing
			lgReadyState = eReadyState.UNINITIALIZED
		End Sub

		Public Function GetStatus() As Integer
			If lgReadyState = eReadyState.COMPLETED Then
				Return CType(lgResponse.StatusCode, Integer)
			Else
				Throw New InvalidOperationException ("Getting response status is forbidden at current ReadyState")
			End If
		End Function

		Public Function GetStatusText() As String
			If lgReadyState = eReadyState.COMPLETED Then
				Return lgResponse.StatusDescription
			Else
				Throw New InvalidOperationException ("Getting response status is forbidden at current ReadyState")
			End If
		End Function

		Public Function GetResponseXML() As XmlDocument
			Try
				If lgReadyState = eReadyState.COMPLETED Then
					Dim stream As Stream = GetResponseStream
					Dim reader As XmlTextReader = New XmlTextReader (stream)
					Dim document As XmlDocument = New XmlDocument 
					document.Load(reader)
					reader.Close
					stream.Close
					Return document
				Else
					Throw New InvalidOperationException ("Getting response XML is forbidden at current ReadyState")
				End If
			Catch e As Exception
				Throw New Exception ("Error occurred while retrieving XML response", e)
			End Try
		End Function

		Public Function GetResponseText() As String
			If lgReadyState = eReadyState.COMPLETED Then
				Dim reader As StreamReader = New StreamReader (GetResponseStream)
				Return reader.ReadToEnd
			Else
				Throw New InvalidOperationException ("Getting response text is forbidden at current ReadyState")
			End If
		End Function

		Public Function GetResponseBody() As Byte()
			If lgReadyState = eReadyState.COMPLETED Then
				Dim stream As Stream = GetResponseStream
				Dim reader As BinaryReader = New BinaryReader (stream)
				Dim count As Long = stream.Length
				Return reader.ReadBytes(CType(count, Integer))
			Else
				Throw New InvalidOperationException ("Getting response body is forbidden at current ReadyState")
			End If
		End Function

		Public Function GetResponseStream() As Stream
			If lgReadyState = eReadyState.COMPLETED Then
				Return lgResponse.GetResponseStream
			Else
				Throw New InvalidOperationException ("Getting response stream is forbidden at current ReadyState")
			End If
		End Function

		Public Function GetReadyState() As eReadyState
			Return lgReadyState
		End Function

		Private Sub ReqCallback(ByVal ar As IAsyncResult)
			Dim req As HttpWebRequest = CType(ar.AsyncState, HttpWebRequest)
			Dim stream As Stream = req.EndGetRequestStream(ar)
			Dim streamW As StreamWriter = New StreamWriter (stream)
			streamW.Write(lgMsg)
			streamW.Close
			stream.Close
			Dim resp As IAsyncResult = req.BeginGetResponse(AddressOf RespCallback, req)
		End Sub

		Private Sub RespCallback(ByVal ar As IAsyncResult)
			Dim req As HttpWebRequest = CType(ar.AsyncState, HttpWebRequest)
			lgResponse = CType(req.EndGetResponse(ar), HttpWebResponse)
			lgReadyState = eReadyState.COMPLETED
			Dim e As EventArgs = New EventArgs 
			OnReadyStatusChange(Me, e)
		End Sub
	End Class 
End Namespace
