'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------

Option Strict On

Imports System.ServiceModel
Imports System.IO


Namespace ESB.Core.Adaptors.Transport.Internet.Receive.Wcf

    Public Class AttachmentManagerBase

        Protected Overridable Function SaveAttachment(ByRef oSoapMTOMAttachment As Byte(), ByRef sPath As String, ByRef sFileName As String, ByRef sFullPath As String) As Boolean
            '++Ensure the directory exists...
            Dim d As New DirectoryInfo(sPath)
            d.Create()

            Dim oBinaryFile As New FileStream(sFullPath, FileMode.Create)
            Dim bw As New BinaryWriter(oBinaryFile)

            bw.Write(oSoapMTOMAttachment)

            bw.Close()
            Return True
        End Function

        ''' <summary>
        ''' For each attachment listed in the request envelope, add it to the MTOM attachment bytes...
        ''' </summary>
        ''' <param name="Request"></param>
        ''' <param name="oESBEnvelope"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Overridable Function SendAttachments(ByVal Request As ESBMessage, ByRef oESBEnvelope As Envelope) As Boolean
            'only do this if there's any attachments referenced in the ESB Envelope...
            If oESBEnvelope.AttachmentsExist = False Then
                Return True
            End If

            'For each attachment referenced in the response ESB Envelope, add it to the 
            'SOAP MTOM attachments collection.
            Dim oEnvAtt As IAttachment

            For Each oEnvAtt In oESBEnvelope.GetAttachments
                Request.Attachment.Add(System.IO.File.ReadAllBytes(Utils.ExpandRelativePath(oEnvAtt.filename.Trim)))
            Next
            Return True
        End Function
        Public Overridable Function ReceiveAttachments(ByVal Request As ESBMessage, ByRef oEnv As Envelope, ByRef sMessageID As String) As Boolean
            'For each attachment referenced in the request ESB Envelope, add it to the 
            'SOAP MTOM attachments collection.

            'only do this if there's any attachments referenced in the ESB Envelope...
            If oEnv.AttachmentsExist = False Then
                Return True
            End If

            If Request Is Nothing Then
                Throw New Exception("Request is Nothing")
            End If

            Dim oEnvAtt As IAttachment
            Dim iAttachmentIndex As Integer = 0
            Dim sFullPath As String
            Dim sPath As String
            Dim sFileName As String

            'check to see that the number of attachments in the ESBEnvelope
            'matches the actual number of DIME attachments in the SoapEnvelope.

            If oEnv.GetAttachments.Count <> Request.Attachment.Count Then
                Throw New Exception("Attachments Error: The number of attachments in the ESBEnvelope must match the actual number of DIME attachments in the SoapEnvelope")
                'todo - dump of attachments collections...
            End If


            For Each oEnvAtt In oEnv.GetAttachments()
                GetFileSavePathForAttachment(sMessageID, oEnvAtt, sPath, sFileName, sFullPath)

                SaveAttachment(Request.Attachment(iAttachmentIndex), sPath, sFileName, sFullPath)

                'Change the attachment path in the envelope 
                'to reflect the new path the attachment will be saved to
                oEnvAtt.filename = sFullPath

                iAttachmentIndex += 1

                sFullPath = ""
                sPath = ""
                sFileName = ""
            Next
            Return True
        End Function
        Protected Overridable Function GetFileSavePathForAttachment(ByVal sMessageID As String, ByRef oEnvAtt As IAttachment, ByRef sPath As String, ByRef sFileName As String, ByRef sFullPath As String) As Boolean
            'Get the base path for attachments
            Dim sAttachmentSavePath As String = Utils.ExpandRelativePath(System.Configuration.ConfigurationManager.AppSettings("AttachmentsReceivePath").Trim)

            'Append the MessageID to this path
            sAttachmentSavePath = Path.Combine(sAttachmentSavePath, sMessageID.Trim)

            'Now attempt to save the attachment with a name that describes as much as
            'possible (given the attachment information) the original attachment.

            Dim sAttachmentFileName As String
            With oEnvAtt
                sAttachmentSavePath = Path.Combine(sAttachmentSavePath, .index().Trim)
                sPath = sAttachmentSavePath

                'Use the description as the name of the file if specified
                'otherwise, use the attachment index as the filename
                If .description.Trim.Length > 0 Then
                    sAttachmentFileName += System.Web.HttpUtility.UrlEncode(.description.Trim)
                Else
                    sAttachmentFileName += .index().Trim
                End If
            End With

            sFileName = sAttachmentFileName

            sFullPath = sAttachmentSavePath + "\" & sAttachmentFileName
            Return True

        End Function
    End Class

End Namespace