'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------

Option Explicit On 
Option Strict Off

Imports ESB.Core.Services.Logger


Public Class PerfMonManager
    ' Const C_sESBTransportAdaptorCategory As String = "ESBTransportAdaptors"
    Const C_sESBTransportAdaptorCounters As String = "ESB Transport Adaptor Counters"
    Public m_bInitialised As Boolean = False
    Dim m_sTransportAdaptorName As String
    Public m_bEnablePerformanceCounters As Boolean = True
    Dim m_sCounterCategory As String
    Dim m_sCounterPrefix As String

    'Use when resetting...
    Public Sub New()

    End Sub

    'Use when setting values...
    Public Overloads Sub Init(ByVal sTransportAdaptorName As String, ByVal oServerContext As ESB.Core.Interfaces.ServerContext.IServerContext)
        m_sTransportAdaptorName = sTransportAdaptorName
        m_bEnablePerformanceCounters = (oServerContext.threadLogDetail.ePerformanceCounterlevel And ePerformanceCounterLevel.eLOG_Transports_Wcf = ePerformanceCounterLevel.eLOG_Transports_Wcf)

        If m_bEnablePerformanceCounters = True Then
            'm_sCounterCategory = "ESB.NET." & oServerContext.FullyQualifiedInstanceName & ".Transport." & m_sTransportAdaptorName
            m_sCounterCategory = "ESB.NET." & oServerContext.FullyQualifiedInstanceName & "Transport." & m_sTransportAdaptorName
            m_sCounterPrefix = ""
        End If

        If m_bInitialised = False Then
            Try
                If m_bEnablePerformanceCounters = True Then
                    CreatePerfMonCounters(oServerContext)
                End If

                m_bInitialised = True
            Catch ex As Exception
                ExceptionManager.Publish(ex)
                LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString)
                'If there's something wrong with any of the ServiceHandler Level counters, just bail...
                m_bEnablePerformanceCounters = False
            Finally

            End Try

        End If
    End Sub
    Private Sub CreatePerfMonCounters(ByVal oServerContext As ESB.Core.Interfaces.ServerContext.IServerContext)

        ' Create a collection of type CounterCreationDataCollection.
        Dim CounterDatas As New CounterCreationDataCollection
        ' Create the counters and set their properties.
        Dim cdCounter As New CounterCreationData

        'Service Level Counters
        'Service Handler Level Counters
        'Silly API - can't create category without a counter...
        'and checking whether a counter exists in a non-existent category throws an exception...
        'So...dodgy code below...
        If System.Diagnostics.PerformanceCounterCategory.Exists(m_sCounterCategory) Then

            If Not System.Diagnostics.PerformanceCounterCategory.CounterExists(m_sCounterPrefix & "ServiceExecuteTime", m_sCounterCategory) Then

                cdCounter.CounterName = m_sCounterPrefix & "ServiceExecuteTime"
                cdCounter.CounterHelp = "The time taken for the Transport Adaptor to Execute. To reset this value, use the ESB Management Portal."
                cdCounter.CounterType = PerformanceCounterType.NumberOfItems64

                ' Add counters to the collection.
                CounterDatas.Add(cdCounter)
            End If

        Else
            cdCounter.CounterName = m_sCounterPrefix & "ServiceExecuteTime"
            cdCounter.CounterHelp = "The time taken for the Transport Adaptor to Execute. To reset this value, use the ESB Management Portal."
            cdCounter.CounterType = PerformanceCounterType.NumberOfItems64
            ' Add counters to the collection.
            CounterDatas.Add(cdCounter)
        End If
        ' Create the category and pass the collection to it.
        If CounterDatas.Count > 0 Then
            System.Diagnostics.PerformanceCounterCategory.Create( _
                m_sCounterCategory, _
                C_sESBTransportAdaptorCounters, _
                 PerformanceCounterCategoryType.MultiInstance, _
                CounterDatas)
        End If

    End Sub
    Public Sub SetCounterValues(ByVal bTxnState As Boolean, Optional ByRef lExecutionTime As Double = 0)
        If m_bEnablePerformanceCounters = True Then

            '' Assumes the category and counter have already been created.


            '' State that you will now be working with a different instance.
            ''myCounter.InstanceName = "instance2"
            ' '' Setting the value actually creates instance2.
            ''myCounter.RawValue = 200

            If lExecutionTime <> 0 Then
                Dim ExecutionTimeCounter As New System.Diagnostics.PerformanceCounter( _
               m_sCounterCategory, m_sCounterPrefix & "ServiceExecuteTime", m_sTransportAdaptorName, False)
                ' Set the raw value to automatically create instance1.
                ExecutionTimeCounter.RawValue = lExecutionTime
            End If
        End If
    End Sub

    Public Sub ResetCounters()
        Dim ExecutionTimeCounter As New System.Diagnostics.PerformanceCounter( _
            m_sCounterCategory, m_sCounterPrefix & "ServiceExecuteTime" & m_sTransportAdaptorName, m_sTransportAdaptorName, False)
        ExecutionTimeCounter.RawValue = 0
    End Sub
End Class
