Imports ESB.Core.Services

''-----------------------------------------------------------------------------
'' <summary>
'' Copyright Keystroke IT 2003
'' </summary>
'' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
''  </summary>
''-----------------------------------------------------------------------------
Imports System.Xml
Imports System.Text
Imports System.IO

Namespace ESBEnvelope
    <Serializable()> _
    Public Class Envelope
        Inherits EnvelopeBase
        Implements IEnvelope
        Implements Serialization.IXmlSerializable




#Region "Constructors"
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Constructs a new envelope object.
        ''' </summary>
        ''' <remarks>Creates a new envelope object and creates a new internal envelope object.
        ''' The internal envelope object is that of the default envelope, as returned by GetRegisteredEnvelopeTypes</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Sub New()
            Init()
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Creates a new envelope object and sets the internal envelope type to that specified in sEnvType.
        ''' </summary>
        ''' <param name="sEnvType">The full class name(including the namespace) of the desired envelope implementation.</param>
        ''' <param name="lDummyParam">A dummy parameter to distinguish between constructors. Can be set to an empty string ("").</param>
        ''' <remarks>Call GetRegisteredEnvelopeTypes to get the list of registered envelopes.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Sub New(ByVal sEnvType As String, ByVal lDummyParam As Long)
            'Set the internal envelope type to be created...
            m_sDefaultEnvelopeType = sEnvType
            Init()
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Creates a new envelope object representing the envelope specified by sEnvXML.
        ''' </summary>
        ''' <param name="sEnvXML"></param>
        ''' <remarks>Creates an envelope object and consults the list of registered envelopes
        ''' in an attempt to find an implementation that can recognize the envelope specified by sEnvXML.
        ''' Once the appropriate envelope is found, the sEnvXML parameter is deserialises to the internal envelope.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Sub New(ByVal sEnvXML As String)
            m_Envelope = GetEnvelopeImplementation(sEnvXML)     'Currently parses into XMLDOM!
            m_Envelope.FromXML(sEnvXML)                         'Also re-parses into XMLDOM!!!

            'TODO: Step1 - pass XMLDOM from first line to second!!! and avoid re-parsing!!!
            'Step2 - change this to not use XMLDOM at all!

        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Creates a new envelope object representing the envelope specified by oEnvXMLDoc.
        ''' </summary>
        ''' <param name="oEnvXMLDoc"></param>
        ''' <remarks>Creates an envelope object and consults the list of registered envelopes
        ''' in an attempt to find an implementation that can recognize the envelope specified by oEnvXMLDoc.
        ''' Once the appropriate envelope is found, the sEnvXML parameter is deserialises to the internal envelope.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Sub New(ByVal oEnvXMLDoc As XmlDocument)
            Init(oEnvXMLDoc.OuterXml)
        End Sub

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Creates a new envelope, with the internal envelope pointing to the original internal envelope object.
        ''' </summary>
        ''' <param name="iEnv"></param>
        ''' <remarks>
        ''' This constructor does NOT make a copy of the original internal envelope. To do that, call the 
        ''' Clone method of the envelope. 
        ''' This method is seldom used.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Sub New(ByVal iEnv As IEnvelope)
            'try & avoid the overhead of creating a new envelope
            'using serialization etc.
            'if we already have an envelope, we should be able to 
            'use its internal envelope etc...somehow...preferably it...
            Me.SetInternalEnvelope(iEnv.GetInternalEnvelope())
            Me.DefaultEnvelopeType = iEnv.DefaultEnvelopeType

        End Sub

#End Region
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Initialise an envelope with a specific XML string representing an envelope.
        ''' </summary>
        ''' <param name="sEnvXML"></param>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Protected Overrides Sub Init(Optional ByVal sEnvXML As String = "")
            If m_Envelope Is Nothing Or sEnvXML <> "" Then
                MyBase.init()
                If sEnvXML = "" Then
                    m_Envelope = ESBClassFactory.CreateObjectByStringName(m_sDefaultEnvelopeType, m_sDefaultEnvelopeAssembly)
                Else
                    m_Envelope = GetEnvelopeImplementation(sEnvXML)
                End If
            End If
        End Sub

#Region "IEnvelope"
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the first document in the envelope of the specified type
        ''' </summary>
        ''' <param name="tDocumentName"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks>The document is returned as an InlineDocument. You then need to get 
        ''' call the appropriate method on the InlineDocument to get the Document in the
        ''' desired format 
        ''' e.g. Deserialised Object, XMLDocument, String, XMLNode, XMLElement etc.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Function GetDocument(ByVal tDocumentName As Type, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelope.GetDocument
            Return m_Envelope.GetDocument(tDocumentName.Name, bCopyDocument)
        End Function

        Public Property Doc(Optional ByVal tDocumentName As Type = Nothing, Optional ByVal lIndex As Long = -1) As String Implements IEnvelope.Doc
            Get
                If Not tDocumentName Is Nothing Then

                    If lIndex = -1 Then
                        Return GetDocument(tDocumentName).GetAsString
                    Else
                        Return GetDocument(tDocumentName, lIndex).GetAsString
                    End If
                Else
                    If lIndex <> -1 Then
                        Return GetDocument(lIndex).GetAsString
                    End If
                End If
                Return ""
            End Get
            Set(ByVal value As String)
                Dim oInlineDoc As New InlineDocument(value)
                If lIndex = -1 Then
                    AddDocument(oInlineDoc)
                Else
                    AddDocument(oInlineDoc, lIndex)
                End If
            End Set
        End Property


        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the document lIndex(th) instance of the specified document type in the envelope.
        ''' </summary>
        ''' <param name="tDocumentName"></param>
        ''' <param name="lIndex"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks>The document is returned as an InlineDocument. You then need to get 
        ''' call the appropriate method on the InlineDocument to get the Document in the
        ''' desired format 
        ''' e.g. Deserialised Object, XMLDocument, String, XMLNode, XMLElement etc.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Function GetDocument(ByVal tDocumentName As Type, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelope.GetDocument
            Return m_Envelope.GetDocument(tDocumentName.Name, lIndex, bCopyDocument)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the first document in the envelope of the specified type, and deserialises
        ''' the Document into the specified class(hierarchy) type.
        ''' </summary>
        ''' <param name="tDocumentName"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Function GetDocumentAsObject(ByVal tDocumentName As Type, Optional ByVal bCopyDocument As Boolean = False) As Object Implements IEnvelope.GetDocumentAsObject
            Dim oInlineDoc As InlineDocument
            oInlineDoc = m_Envelope.GetDocument(tDocumentName.Name, bCopyDocument)
            If oInlineDoc Is Nothing Then
                Throw New Exception("There is no document in the envelope available with the following details:" & tDocumentName.FullName)
            Else
                Return oInlineDoc.GetAsObject(tDocumentName)
            End If
            'The line below does not give adequate information when an error indicating that there is no document available...
            'Return m_Envelope.GetDocument(tDocumentName.Name, bCopyDocument).GetAsObject(tDocumentName)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the lIndex(th) instance of the document in the envelope of the specified type, and deserialises
        ''' the Document into the specified class(hierarchy) type.</summary>
        ''' <param name="tDocumentName"></param>
        ''' <param name="lIndex"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Function GetDocumentAsObject(ByVal tDocumentName As Type, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As Object Implements IEnvelope.GetDocumentAsObject
            Return m_Envelope.GetDocument(tDocumentName.Name, lIndex, bCopyDocument).GetAsObject(tDocumentName)
        End Function

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the lIndex(th) instance of the document in the envelope of the specified type, and deserialises
        ''' the Document into the specified class(hierarchy) type. Also returns the position index in the envelope of the returned document.</summary>
        ''' <param name="tDocumentName"></param>
        ''' <param name="lIndex"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns>position index of the document and the document itself.</returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------

        Public Overloads Function GetDocument(ByVal tDocumentName As Type, ByVal lIndex As Long, ByRef lReturnedDocIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelope.GetDocument
            Return m_Envelope.GetDocument(tDocumentName.Name, lIndex, lReturnedDocIndex, bCopyDocument)
        End Function


        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Formats an XML Stream into a nested XML layout for easy reading.
        ''' </summary>
        ''' <param name="oStream"></param>
        ''' <param name="bXMLDOMParseFirst"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Function PrettyFormatXML(ByVal oStream As MemoryStream, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String Implements IEnvelope.PrettyFormatXML
            Return m_Envelope.PrettyFormatXML(oStream, bXMLDOMParseFirst)
        End Function
        'bXMLDOMParseFirst - to correct problems such as the space in the following xml string <xxx >sss<xxx>.
        'Not good for large XML (100's of KBytes,MBytes etc.) as it does an XMLDOM parse etc...

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Formats an XML Stream into a nested XML layout for easy reading.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <param name="bXMLDOMParseFirst"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Function PrettyFormatXML(ByVal sXML As String, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String Implements IEnvelope.PrettyFormatXML
            Return m_Envelope.PrettyFormatXML(sXML)
        End Function
        'Properties...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the name of the first (default) document in the envelope
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public ReadOnly Property MessageType() As String Implements IEnvelope.MessageType
            Get
                Return m_Envelope.XMLSchemaName
            End Get
        End Property
        'Properties...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the name and namespace of the first (default) document in the envelope separated by 2 colons i.e. ::.
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public ReadOnly Property ResolveName() As String Implements IEnvelope.ResolveName
            Get
                Return m_Envelope.XMLSchemaName & "::" & m_Envelope.XMLSchemaNamespace
            End Get
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets or sets the XML encapsulated by an envelope object.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This property calls the FromXML or the ToXML methods depending upon whether the 
        ''' XML is being assigned or extracted.
        ''' The assignment results in the deserialization of the XML into classes for
        ''' the appropriate envelope. The envelope list in consulted and the registered envelopes
        ''' asked in turn whether this XML string is of a type they can handle.
        ''' The extraction results in the appropriate envelope implemention serializing itself
        ''' to an XML string. </remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Property XML() As String Implements IEnvelope.XML
            Get
                Return ToXML()
            End Get
            Set(ByVal Value As String)
                FromXML(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns whether any attachments exist in the envelope.
        ''' This is reflected in the attachments collection of the envelope manifest.
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overrides ReadOnly Property AttachmentsExist() As Boolean
            Get
                Return m_Envelope.AttachmentsExist
            End Get
        End Property
#Region "IEnvelope methods from base class"
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Swaps the To and From addresses in the envelope header.
        ''' This is usually used internally by the envelope when creating a response.
        ''' </summary>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Sub SwapToAndFromAddresses() Implements IEnvelope.SwapToAndFromAddresses
            MyBase.SwapToAndFromAddresses()
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Sets the internal envelope interface pointer.
        ''' </summary>
        ''' <param name="oEnv"></param>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Sub SetInternalEnvelope(ByVal oEnv As IEnvelopeProvider) Implements IEnvelope.SetInternalEnvelope
            MyBase.SetInternalEnvelope(oEnv)
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the internal envelope interface pointer.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetInternalEnvelope() As IEnvelopeProvider Implements IEnvelope.GetInternalEnvelope
            Return MyBase.GetInternalEnvelope
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the default envelope type. This is usually the first envelope 
        ''' in the list of registered envelopes.
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Property DefaultEnvelopeType() As String Implements IEnvelope.DefaultEnvelopeType
            Get
                Return MyBase.DefaultEnvelopeType
            End Get
            Set(ByVal Value As String)
                MyBase.DefaultEnvelopeType = Value
            End Set
        End Property
#End Region
#End Region

#Region "IEnvelopeProvider"
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Clones an envelope
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>This method uses serialization/deserialization to clone
        ''' the current envelope.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function Clone() As Object
            'Best way to Clone is probably to Serialise & Deserialize as is done below...
            Dim oEnv As New Envelope
            Dim sEnvXML As String = Me.ToXML()
            oEnv.FromXML(sEnvXML)
            Return oEnv
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This method copies sections of a document from one document to another.
        ''' It changes the namespace to that of the destination document.
        ''' </summary>
        ''' <param name="FromDoc1"></param>
        ''' <param name="tFromDoc1Type"></param>
        ''' <param name="ToDoc2"></param>
        ''' <param name="tTODoc2Type"></param>
        ''' <returns></returns>
        ''' <remarks>
        ''' This is a helper method to help you copy sections of a document from 
        ''' one document to another. It saves you having to copy individual properties
        ''' from one document to the other. 
        ''' Useful when different schemas have the same sub-schema structure, and
        ''' you want to copy that structure across from say, the request document 
        ''' to the response document.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function CopyDocumentSection(ByVal FromDoc1 As Object, ByVal tFromDoc1Type As System.Type, ByVal ToDoc2 As Object, ByVal tTODoc2Type As System.Type) As Boolean
            Return m_Envelope.CopyDocumentSection(FromDoc1, tFromDoc1Type, ToDoc2, tTODoc2Type)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Serializes an arbitrary Object hierarchy (with the appropriate serialization 
        ''' attributes set) to an XML string. 
        ''' </summary>
        ''' <param name="oObjectToSerialize"></param>
        ''' <returns></returns>
        ''' <remarks>Usually used on classes generated against
        ''' an XSD Schema, using XSD.exe or some other code generation tool.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function Serialize(ByVal oObjectToSerialize As Object) As String
            Return m_Envelope.Serialize(oObjectToSerialize)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Deserializes an XML String into an object hierarchy as defined by the 
        ''' root object type in 'tType'.
        ''' </summary>
        ''' <param name="sXMLStringToDeSerialize"></param>
        ''' <param name="tType"></param>
        ''' <returns></returns>
        ''' <remarks>
        ''' The type in 'tType' must be serialization aware.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function DeSerialize(ByVal sXMLStringToDeSerialize As String, ByVal tType As System.Type) As Object
            Return m_Envelope.DeSerialize(sXMLStringToDeSerialize, tType)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the number of documents in the envelope.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function DocumentCount() As Long
            Return m_Envelope.DocumentCount
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Deserialises an envelope in String format into the appropriate
        ''' envelope objects, and encapsulates it in the general envelope object.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Sub FromXML(ByVal sXML As String)
            Init(sXML)
            m_Envelope.FromXML(sXML)
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets a custom parameter from the envelope.
        ''' </summary>
        ''' <param name="sParamNodeName">The name of a logical grouping of Custom Parameters by which the custom parameters are organized.</param>
        ''' <param name="sParamName">The name of the custom parameter being queried.</param>
        ''' <returns></returns>
        ''' <remarks>
        ''' Custom Parameters are a mechanism by which arbitrary data - in the form of name-value pairs- 
        ''' can be added to the envelope. Usually used by the ESB core internally to 
        ''' help process messages. The information is, however made available to the client.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	25/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetCustomParam(ByVal sParamNodeName As String, ByVal sParamName As String) As String
            Return m_Envelope.GetCustomParam(sParamNodeName, sParamName)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the value of the specified custom parameter. No group is specified for the parameter.
        ''' </summary>
        ''' <param name="sParamName"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetCustomParam(ByVal sParamName As String) As String
            Return m_Envelope.GetCustomParam(sParamName)
        End Function
        'By default, work with references to the documents - this is best for performance...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Adds a document to the envelope.
        ''' </summary>
        ''' <param name="oEnvDoc">The document to be added to the envelope.</param>
        ''' <param name="bCopyDocument">Specifies whether to copy the document or to just add a reference
        ''' to the existing document. References are preferred for performance.</param>
        ''' <returns>Returns the index of the document in the envelope.</returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function AddDocument(ByVal oEnvDoc As IInlineDocument, Optional ByVal bCopyDocument As Boolean = False) As Long
            Init()
            Return m_Envelope.AddDocument(oEnvDoc, bCopyDocument)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Adds the specified document to the envelope as the lDocumentIndex(th) document.
        ''' </summary>
        ''' <param name="oEnvDoc"></param>
        ''' <param name="lDocumentIndex"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks>Rarely used.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function AddDocument(ByVal oEnvDoc As IInlineDocument, ByVal lDocumentIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As Long
            Init()
            Return m_Envelope.AddDocument(oEnvDoc, lDocumentIndex, bCopyDocument)
        End Function
        'By default, work with references to the documents - this is best for performance...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the document at the index in the envelope specified by the lIndex parameter.
        ''' </summary>
        ''' <param name="lIndex">The index of the document in the envelope to retrieve.</param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetDocument(ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
            Return m_Envelope.GetDocument(lIndex, bCopyDocument)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the first document of name "sDocumentName" in the envelope.
        ''' </summary>
        ''' <param name="sDocumentName"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetDocument(ByVal sDocumentName As String, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
            Return m_Envelope.GetDocument(sDocumentName, bCopyDocument)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the lIndex(th) instance of the document named "sDocumentName" in the envelope.
        ''' </summary>
        ''' <param name="sDocumentName"></param>
        ''' <param name="lIndex"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetDocument(ByVal sDocumentName As String, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
            Return m_Envelope.GetDocument(sDocumentName, lIndex, bCopyDocument)
        End Function

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the lIndex(th) instance of the document named "sDocumentName" in the envelope, and returns the index of the returned document.
        ''' </summary>
        ''' <param name="sDocumentName"></param>
        ''' <param name="lIndex"></param>
        ''' <param name="bCopyDocument"></param>
        ''' <returns>Selected document and its index.</returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetDocument(ByVal sDocumentName As String, ByVal lIndex As Long, ByRef lReturnedDocIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument
            Return m_Envelope.GetDocument(sDocumentName, lIndex, lReturnedDocIndex, bCopyDocument)
        End Function

        'Need to have a property etc. to get collection of attachments, documents etc.
        'Also need to be able to change the attributes of the attachment when moving them around...
        ':., need access to the colections...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Adds a reference to an attachment in the envelope at the specified index.
        ''' </summary>
        ''' <param name="sAttachmentName">Logical name of the attachment</param>
        ''' <param name="sAttachmentPath">File Path or URL to the attachment.</param>
        ''' <param name="lIndex">Index of the attachment in the manifest.</param>
        ''' <param name="sType">The MIME type of the attachment, i.e. Image, Text, Zip, etc...</param>
        ''' <returns></returns>
        ''' <remarks>Adds a reference to an attachment. Does not add the attachment inline in the envelope.
        ''' When the envelope is sent out, the Attachment Manager includes the attachment in the SOAP envelope
        ''' and sends it.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function AddAttachment(Optional ByVal sAttachmentName As String = "", Optional ByVal sAttachmentPath As String = "", Optional ByVal lIndex As Long = 0, Optional ByVal sType As String = "", Optional ByVal sUrl As String = "") As Long
            Return m_Envelope.AddAttachment(sAttachmentName, sAttachmentPath, lIndex, sType, sUrl)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the attachment at the specified index.
        ''' </summary>
        ''' <param name="lIndex">Index of attachment to retrieve.</param>
        ''' <returns>IAttachment interface</returns>
        ''' <remarks>This method returns the information about the attachment, not the attachment itself.
        ''' The attachment itself is at the file location specified in the IAttachment interface properties.
        ''' When an envelope referencing an attachment is received, the attachment is extracted out of the DIME message and 
        ''' saved to the attachments location as specified by the ESB configuration.
        ''' The file location property of the IAttachments interface specifies the 
        ''' path to the locally saved attachment.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetAttachment(ByVal lIndex As Long) As IAttachment
            Return m_Envelope.GetAttachment(lIndex)
        End Function

        Public Overrides Function GetCustomParams() As IList
            Return m_Envelope.GetCustomParams
        End Function
        Public Overrides Sub ClearCustomParams()
            m_Envelope.ClearCustomParams()
        End Sub

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the Attachments collection as an untyped collection.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetAttachments() As System.Collections.Generic.IList(Of IAttachment)
            Return m_Envelope.GetAttachments()
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the Documents collection as an untyped collection.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetDocuments() As System.Collections.Generic.IList(Of IInlineDocument)
            Return m_Envelope.GetDocuments()
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the status of the specified envelope.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <returns>An enum of the type eEnvelopeStatus indicating the consistency and status of the envelope.</returns>
        ''' <remarks>
        ''' Usually called after a response has been received from the server to ascertain the response code of the call.
        ''' Also checks the consistency of the envelope.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetMsgStatus(ByVal sXML As String) As eEnvelopeStatus
            Return m_Envelope.GetMsgStatus(sXML)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines whether the specified envelope has any errors.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <returns></returns>
        ''' <remarks>An error is represented by an ErrorXML document in the envelope 
        ''' containing an errorEntry node.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function IsError(ByVal sXML As String) As Boolean
            Return m_Envelope.IsError(sXML)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Specifies whether the current envelope has any errors.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>An error is represented by an ErrorXML document in the envelope 
        ''' containing an errorEntry node.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function IsError() As Boolean
            Return m_Envelope.IsError
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        '''   Specifies whether the current envelope has any errors.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>A warning is represented by an ErrorXML document in the envelope 
        ''' containing an warningEntry node.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function IsWarning() As Boolean
            Return m_Envelope.IsWarning
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the status of the specified envelope.
        ''' </summary>
        ''' <returns>An enum of the type eEnvelopeStatus indicating the consistency and status of the envelope.</returns>
        ''' <remarks>
        ''' Usually called after a response has been received from the server to ascertain the response code of the call.
        ''' Also checks the consistency of the envelope.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function GetMsgStatus() As eEnvelopeStatus
            Return m_Envelope.GetMsgStatus
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        '''   Specifies whether the specified envelope has any errors.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>A warning is represented by an ErrorXML document in the envelope 
        ''' containing an warningEntry node.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function IsWarning(ByVal sXML As String) As Boolean
            Return m_Envelope.IsWarning(sXML)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Sets a custom parameter value, logically grouped by "sParamNodeName".
        ''' </summary>
        ''' <param name="sParamNodeName">Custom Parameter group name</param>
        ''' <param name="sParamName">Custom Parameter name</param>
        ''' <param name="sParamValue">Custom Parameter value</param>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Sub SetCustomParam(ByVal sParamNodeName As String, ByVal sParamName As String, ByVal sParamValue As String)
            m_Envelope.SetCustomParam(sParamNodeName, sParamName, sParamValue)
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Sets a custom parameter value.
        ''' </summary>
        ''' <param name="sParamName">Custom Parameter name.</param>
        ''' <param name="sParamValue">Custom Parameter value.</param>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Sub SetCustomParam(ByVal sParamName As String, ByVal sParamValue As String)
            m_Envelope.SetCustomParam(sParamName, sParamValue)
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Serializes the current envelope to a String.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function ToString() As String
            Return m_Envelope.ToString()
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Serializes the current envelope to a string.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>This is the same as the ToString method, and the XML property.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function ToXML() As String
            Return m_Envelope.ToXML()
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function ToXMLDocument() As XmlDocument
            Return m_Envelope.ToXMLDocument()
        End Function
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="oXML"></param>
        ''' <remarks></remarks>
        Public Overrides Sub FromXMLDocument(ByVal oXML As XmlDocument)
            m_Envelope.FromXMLDocument(oXML)
        End Sub

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>Not implemented</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function ToESBEnvelope() As IEnvelope 'This is identical to the Clone interface/method...Included for completeness...
            Return Clone()
        End Function

        'Envelope Detection - new interface??? Not for now...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This method determines whether a particular envelope implementation 
        ''' can handle a specified string of XML as its envelope.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function IsCorrectEnvelopeType(ByVal sXML As String) As Boolean
            Return m_Envelope.IsCorrectEnvelopeType(sXML)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the friendly name of an envelope.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <returns></returns>
        ''' <remarks>Gets the friendly name of the envelope specified by sXML, or of the current envelope
        ''' if the "sXML" parameter is omitted or is an empty String.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetEnvelopeNodeName(Optional ByVal sXML As String = "") As Boolean
            Return m_Envelope.GetEnvelopeNodeName(sXML)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the namespace of an envelope.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <returns></returns>
        ''' <remarks>Returns the namespace of the envelope specified in sXML or of the current
        ''' envelope if the "sXML" parameter is omitted or is an empty String.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetEnvelopeNameSpace(Optional ByVal sXML As String = "") As Boolean
            Return m_Envelope.GetEnvelopeNameSpace(sXML)
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets the version of an envelope.
        ''' </summary>
        ''' <param name="sXML"></param>
        ''' <returns></returns>
        ''' <remarks>Returns the version numner of the envelope specified in sXML or of the current
        ''' envelope if the "sXML" parameter is omitted or is an empty String.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetEnvelopeVersion(Optional ByVal sXML As String = "") As Boolean
            Return m_Envelope.GetEnvelopeVersion(sXML)
        End Function

        'Utility methods
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Gets an XML document as a string. 
        ''' </summary>
        ''' <param name="oDoc"></param>
        ''' <returns></returns>
        ''' <remarks>I can't remember what this gives us over the .OuterXML method on the XMLDocument Object!!!</remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overrides Function GetXMLAsString(ByVal oDoc As XmlDocument) As String
            Dim oXMLUtil As New XMLUtil
            Return oXMLUtil.GetXMLAsString(oDoc)
        End Function

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Creates a response envelope based upon the request envelope.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>
        ''' This method returns an envelope with most of the header information
        ''' preserved from the original envelope.
        ''' The Manifest nodes for the documents and referenced attachments are removed.
        ''' The body nodes present in the original envelope are also removed from the new
        ''' envelope.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	26/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overloads Overrides Function CreateResponse() As IEnvelope
            Dim oEnvRs As Envelope = m_Envelope.CreateResponse()

            'MINA: DO NOT DO HERE - It stuffs up routing!
            'Leave this to the PipelineManager/ResponseManager...
            'oEnvRs.SwapToAndFromAddresses()
            Return oEnvRs
        End Function

#End Region

        Public Function GetSchema() As System.Xml.Schema.XmlSchema Implements System.Xml.Serialization.IXmlSerializable.GetSchema
            Return Nothing
        End Function

        Public Sub ReadXml(ByVal reader As System.Xml.XmlReader) Implements System.Xml.Serialization.IXmlSerializable.ReadXml
            'DeSerialize
            Me.FromXML(reader.ReadOuterXml())
        End Sub

        Public Sub WriteXml(ByVal writer As System.Xml.XmlWriter) Implements System.Xml.Serialization.IXmlSerializable.WriteXml
            'Serialize
            writer.WriteRaw(Me.ToXML)
        End Sub
    End Class
End Namespace
