'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
Imports System
Imports System.Xml
Imports System.Text
Imports System.IO
Imports System.Runtime.Remoting
Imports Microsoft.ApplicationBlocks.ConfigurationManagement
Imports ESB.Core.Services

Namespace ESBEnvelope
    'By default, work with references to the documents - this is best for performance...
    <Serializable()> _
   Public MustInherit Class EnvelopeBase
        Inherits System.Object
        Implements IEnvelopeProvider

#Region "Protected member variables"
        'These properties MUST NOT BE declared as PUBLIC for Serialization(of derived classes...) to work...
        'otherwise, they must be given the <System.Xml.Serialization.XmlIgnore()>  attribute...
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The internal envelope. This represents the actual envelope being used. It's type is encapsulated
        ''' by the outer envelope object.
        ''' </summary>
        '''-----------------------------------------------------------------------------
        Protected m_Envelope As IEnvelopeProvider

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The type of the envelope object above.
        ''' </summary>
        '''-----------------------------------------------------------------------------
        Protected m_sEnvelopeType As String

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The list of registered envelopes. Currently hard coded in a list. Needs to be stored in a 
        ''' configuration file. The default envelope is set as the first envelope in this list. 
        ''' </summary>
        '''-----------------------------------------------------------------------------
        Protected m_bRegisteredEnvelopeTypesInit As Boolean = False         'Protected m_oRegisteredEnvelopes As New RegisteredEnvelopes

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The default envelope. When a new outer envelope object is constructed without any parameters
        ''' set to specify an internal envelope type, this is the type of internal envelope created.
        ''' </summary>
        '''-----------------------------------------------------------------------------
        Protected m_sDefaultEnvelopeType As String = "ESB.Core.XMLSchemaClasses.ESBEnvelope.ESBEnvelopeProvider"
        Protected m_sDefaultEnvelopeAssembly As String = "ESB.Core.XMLSchemaClasses"
#End Region
#Region "Constructors"
        'Overridden in derived class - Envelope

        Public Sub New()

        End Sub
        Public Sub New(ByVal sEnvXML As String)

        End Sub
        Public Sub New(ByVal oEnvXMLDoc As XmlDocument)

        End Sub
#End Region
#Region "Non-interface member functions"
        Protected Overridable Sub init(Optional ByVal sEnvXML As String = "")
            If m_Envelope Is Nothing Then
                GetRegisteredEnvelopeTypes()
            End If
        End Sub
        'Protected Overridable Function CreateObjectByStringName(ByVal sObjectName As String, Optional ByVal sAssemblyFile As String = "") As Object
        '    StartMethod()
        '    Dim x As Object
        '    Try
        '        If sAssemblyFile = "" Then
        '            x = System.Activator.CreateInstance(System.Type.GetType(sObjectName))
        '            Return x
        '        Else
        '            x = Activator.CreateInstanceFrom(sAssemblyFile, sObjectName).Unwrap()
        '        End If
        '        If x Is Nothing Then
        '            LogError(LogDefs.eLogLevel.eLOG_ERROR, "Could not create object!")
        '            Return Nothing
        '        Else
        '            Return x
        '        End If
        '    Catch ex As Exception
        '        ExceptionManager.Publish(ex)
        '        LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString)
        '        Return Nothing
        '    Finally
        '        EndMethod()
        '    End Try
        'End Function
#End Region
#Region "IEnvelope"
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This method swaps the To and From addresses in the envelope. 
        ''' </summary>
        ''' <remarks>Usually only used by infrastructure code.
        ''' Used for envelope routing. Usually used by the CreateResponse method.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overridable Sub SwapToAndFromAddresses()
            Dim sTmpToAddress As String = m_Envelope.ToAddress
            Dim sTmpToAddressType As String = m_Envelope.ToAddressType

            m_Envelope.ToAddress = m_Envelope.FromAddress
            m_Envelope.ToAddressType = m_Envelope.FromAddressType
            m_Envelope.FromAddress = sTmpToAddress
            m_Envelope.FromAddressType = sTmpToAddressType
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary>  Sets the internal envelope type the the specified implementation.
        ''' 
        ''' </summary>
        ''' <param name="oEnv"></param>
        ''' <remarks>Usually only used by infrastructure code.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overridable Sub SetInternalEnvelope(ByVal oEnv As IEnvelopeProvider)
            m_Envelope = oEnv
        End Sub
        '''-----------------------------------------------------------------------------
        ''' <summary> Returns the internal envelope of the current OuterEnvelope (ESBEnvelope) object.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>Usually only used by infrastructure code.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overridable Function GetInternalEnvelope() As IEnvelopeProvider
            Return m_Envelope
        End Function
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Returns the default envelope.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>Usually only used by infrastructure code.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property DefaultEnvelopeType() As String
            Get
                Return m_sDefaultEnvelopeType
            End Get
            Set(ByVal Value As String)
                m_sDefaultEnvelopeType = Value
            End Set
        End Property

#End Region
#Region "IEnvelopeProvider"
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Clones the current envelope.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks>Uses serialization to clone the current envelope. It is a deep copy.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        Public Overridable Function Clone() As Object Implements ICloneable.Clone
            'Best way to Clone is probably to Serialise & Deserialize as is done below...
            Dim oEnv As New Envelope
            Dim sEnvXML As String = oEnv.ToXML()
            oEnv.FromXML(sEnvXML)
            Return oEnv
        End Function
        Public MustOverride Overloads Function CreateResponse() As IEnvelope Implements IEnvelopeProvider.CreateResponse
        'Public MustOverride Overloads Function CreateResponse(byval oEnv As IEnvelope) As IEnvelope Implements IEnvelopeProvider.CreateResponse
        'Public MustOverride Overloads Function CreateResponse(byval sEnvXMLStr As String) As IEnvelope Implements IEnvelopeProvider.CreateResponse
        'Public MustOverride Overloads Function CreateResponse(byval oEnvXMLDoc As XmlDocument) As IEnvelope Implements IEnvelopeProvider.CreateResponse

        '-----------------------------------------------------------------------------
        ' <summary>
        ' The XMLSchema name of the default document - this is one of the parameters used on the 
        ' ESB server to construct the pipeline for the request
        ' </summary>
        ' <value>XSD Schema name of the first document in the envelope.</value>
        ' <remarks>This is the name of the root XML node of the first document in the envelope.</remarks>
        ' <history>
        ' 	[MCasiou] 	29/03/2004	Created
        ' </history>
        '-----------------------------------------------------------------------------
        'Declare these properties with the XmlIgnoreAttribute in the implementing envelope classes...
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable ReadOnly Property XMLSchemaName() As String Implements IEnvelopeProvider.XMLSchemaName
            Get
                Return m_Envelope.XMLSchemaName
            End Get
        End Property

        '-----------------------------------------------------------------------------
        ' <summary>
        ' The XMLSchemaNamespace name of the default document - this can be one of the parameters used on the 
        ' ESB server to construct the pipeline for the request
        ' </summary>
        ' <value>XSD SchemaNamespace name of the first document in the envelope.</value>
        ' <remarks>This is the name of the XML Schema Namespace of the first document in the envelope.</remarks>
        ' <history>
        ' 	[MCasiou] 	12/06/2007	Created
        ' </history>
        '-----------------------------------------------------------------------------
        'Declare these properties with the XmlIgnoreAttribute in the implementing envelope classes...
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable ReadOnly Property XMLSchemaNamespace() As String Implements IEnvelopeProvider.XMLSchemaNamespace
            Get
                Return m_Envelope.XMLSchemaNamespace
            End Get
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is the Service or business transaction name that is being executed.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This is one of the parameters used on the 
        ''' ESB server to construct the pipeline for the request
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property ServiceName() As String Implements IEnvelopeProvider.ServiceName
            Get
                Return Trim(m_Envelope.ServiceName)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ServiceName = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is the namespace of the Service/business transaction name that is being executed.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	18/04/2009	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property ServiceNamespace() As String Implements IEnvelopeProvider.ServiceNamespace
            Get
                Return Trim(m_Envelope.ServiceNamespace)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ServiceNamespace = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is the Version of the Service or business transaction name that is being executed.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This is one of the parameters used on the 
        ''' ESB server to construct the pipeline for the request
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	07/04/2008	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property ServiceVersion() As String Implements IEnvelopeProvider.ServiceVersion
            Get
                Return Trim(m_Envelope.ServiceVersion)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ServiceVersion = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is an optional value that allows the server to behave differently for the
        ''' same request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks> This is the Total Pipeline Request Context the ESB server uses to determine the pipeline for the request.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' 	[MCasiou] 	07/04/2008	Modified 
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable ReadOnly Property Context() As String Implements IEnvelopeProvider.Context
            Get
                Return Trim(m_Envelope.Context)
            End Get

        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is an optional value that allows the server to behave differently for the
        ''' same request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks> This is one of the parameters used on the 
        ''' ESB server to construct the pipeline for the request.
        ''' This is an optional value that allows the server to behave differently for the
        ''' same request. For example, the ESB core may be configured differently based upon
        ''' where the request comes from. For example, an internet facing request may be processed
        ''' differently to an internal Client-Server based application.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	07/04/2008	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property MessageContext() As String Implements IEnvelopeProvider.MessageContext
            Get
                Return Trim(m_Envelope.MessageContext)
            End Get
            Set(ByVal Value As String)
                m_Envelope.MessageContext = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The address the envelope will be routed to if set.
        ''' </summary>
        ''' <value>A valid SOAP or HTTP URL.</value>
        ''' <remarks>On the server side, this node is only used for asynchronous messages.
        ''' On the client(sending) side, this may be used to determine the address to send a message to
        ''' however it is entirely up to the client to determine what to do with this node.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property ToAddress() As String Implements IEnvelopeProvider.ToAddress
            Get
                Return Trim(m_Envelope.ToAddress)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ToAddress = Trim(Value)

            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines the transport type to be used for routing a message.
        ''' </summary>
        ''' <value>SOAP or HTTP</value>
        ''' <remarks>This overrides the transport type inferred by the URL in the ToAddress node.
        ''' The AddressType is the part of the string in the URL before the colon. 
        ''' eg. in the following URL, http://www.csc.com/, the TransportType is "http".
        ''' You can override the transport being used to SOAP by setting this property.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property ToAddressType() As String Implements IEnvelopeProvider.ToAddressType
            Get
                Return Trim(m_Envelope.ToAddressType)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ToAddressType = Trim(Value)
            End Set
        End Property



        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' A name of a message recipient.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This field can be used for request remapping. Included for completeness, although 
        ''' there may often be many recipients of a message. As such, this field may not be very useful for many scenarios.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/06/2008	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property ToName() As String Implements IEnvelopeProvider.ToName
            Get
                Return Trim(m_Envelope.ToName)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ToName = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The address to send responses to. Usually the sender's address.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This field is mandatory for asynchronous messages.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property FromAddress() As String Implements IEnvelopeProvider.FromAddress
            Get
                Return Trim(m_Envelope.FromAddress)
            End Get
            Set(ByVal Value As String)
                m_Envelope.FromAddress = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The name of the message sender.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This field can be used for request remapping.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/06/2008	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property FromName() As String Implements IEnvelopeProvider.FromName
            Get
                Return Trim(m_Envelope.FromName)
            End Get
            Set(ByVal Value As String)
                m_Envelope.FromName = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' The name of the channel through which the message is travelling.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This field can be used for request remapping.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/06/2008	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property Channel() As String Implements IEnvelopeProvider.Channel
            Get
                Return Trim(m_Envelope.Channel)
            End Get
            Set(ByVal Value As String)
                m_Envelope.Channel = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines the transport type to be used for routing a message.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This overrides the transport type inferred by the URL in the ToAddress node.
        ''' The AddressType is the part of the string in the URL before the colon. 
        ''' eg. in the following URL, http://www.csc.com/, the TransportType is "http".
        ''' You can override the transport being used to SOAP by setting this property.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property FromAddressType() As String Implements IEnvelopeProvider.FromAddressType
            Get
                Return Trim(m_Envelope.FromAddressType)
            End Get
            Set(ByVal Value As String)
                m_Envelope.FromAddressType = Trim(Value)
            End Set
        End Property

        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is the ConversationID or Business Process ID.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This value is meant to be set by the client in order to be able to group requests and 
        ''' responses as part of a particular Business Process or Conversation. If not set, the ESB will NOT generate a unique GUID
        ''' for this property.
        '''</remarks>
        ''' <history>
        ''' 	[MCasiou] 	12/06/2007	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property ConversationID() As String Implements IEnvelopeProvider.ConversationID
            Get
                Return Trim(m_Envelope.ConversationID)
            End Get
            Set(ByVal Value As String)
                m_Envelope.ConversationID = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is the external MessageID (correlationID).
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This value is meant to be set by the client in order to be able to correlate requests and 
        ''' responses, especially in an asynchronous environment. If not set, ESB will generate a unique GUID
        ''' for this node and return it to the client.
        ''' Note that the ESB core uses an internal MessageID which is guaranteed to be unique as
        ''' it does not have any client side involvement.</remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property MessageID() As String Implements IEnvelopeProvider.MessageID
            Get
                Return Trim(m_Envelope.MessageID)
            End Get
            Set(ByVal Value As String)
                m_Envelope.MessageID = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is the Internal MessageID (correlationID).
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This value is only set by the Server in order to generate a unique GUID for a given request.
        '''</remarks>
        ''' <history>
        ''' 	[MCasiou] 	13/05/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
        Public Overridable Property InternalMessageID() As String Implements IEnvelopeProvider.InternalMessageID
            Get
                Return Trim(m_Envelope.InternalMessageID)
            End Get
            Set(ByVal Value As String)
                m_Envelope.InternalMessageID = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This is an alias to the environment of a particular back end system that an adaptor is
        ''' aimed at integrating to. This value is not used within the ESB core. It is passed 
        ''' on to the adaptor by virtue of being in the envelope. The adaptor makes the appropriate 
        ''' decisions based upon the value of this parameter.
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property Environment() As String Implements IEnvelopeProvider.Environment
            Get
                Return Trim(m_Envelope.Environment)
            End Get
            Set(ByVal Value As String)
                m_Envelope.Environment = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This field specifies a friendly name for the system being targeted.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>This field is optional and is not used by the ESB core to decide
        ''' on how to route/process the message.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property SystemName() As String Implements IEnvelopeProvider.SystemName
            Get
                Return Trim(m_Envelope.SystemName)
            End Get
            Set(ByVal Value As String)
                m_Envelope.SystemName = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This field specifies the type of system being targetted by the request.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This is one of the parameters used on the 
        ''' ESB server to construct the pipeline for the request
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property SystemType() As String Implements IEnvelopeProvider.SystemType
            Get
                Return Trim(m_Envelope.SystemType)
            End Get
            Set(ByVal Value As String)
                m_Envelope.SystemType = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' This field specifies the version of the system being targetted by the request.
        ''' It is used in conjunction with the SystemType, ServiceName and Context fields
        ''' to determine how a message is processed within ESB.
        ''' </summary>
        ''' <value></value>
        ''' <remarks>
        ''' This is one of the parameters used on the 
        ''' ESB server to construct the pipeline for the request
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property SystemVersion() As String Implements IEnvelopeProvider.SystemVersion
            Get
                Return Trim(m_Envelope.SystemVersion)
            End Get
            Set(ByVal Value As String)
                m_Envelope.SystemVersion = Trim(Value)
            End Set
        End Property
        '''-----------------------------------------------------------------------------
        ''' <summary>
        ''' Determines whether a request is executed synchronously or asynchronously by the ESB
        ''' server when called on the ProcessMsgSync SOAP method or by the HTTP POST method of the 
        ''' transport adaptors.
        ''' </summary>
        ''' <value>
        '''     Blank, 0, 2 = Synchronous
        '''     1 = Asynchronous
        ''' </value>
        ''' <remarks>
        ''' When set to Asynchronous, the ESB core queues the message and sends back an envelope
        ''' containing a Handshake document indicating whether the message was received and queued 
        ''' successfully. This Handshake is returned on the calling thread. At a later point in time, the
        ''' response is sent to the FromAddress (and FromAddressType) fields as specified in the Request 
        ''' Envelope Header.
        ''' </remarks>
        ''' <history>
        ''' 	[MCasiou] 	29/03/2004	Created
        ''' </history>
        '''-----------------------------------------------------------------------------
        <System.Xml.Serialization.XmlIgnore()> _
                Public Overridable Property CallSynchronizationType() As String Implements IEnvelopeProvider.CallSynchronizationType
            Get
                Return m_Envelope.CallSynchronizationType
            End Get
            Set(ByVal Value As String)
                m_Envelope.CallSynchronizationType = Value
            End Set
        End Property
        '--Routing

        Public Overridable Function CopyDocumentSection(ByVal FromDoc1 As Object, ByVal tFromDoc1Type As System.Type, ByVal ToDoc2 As Object, ByVal tTODoc2Type As System.Type) As Boolean Implements IEnvelopeProvider.CopyDocumentSection
            Return m_Envelope.CopyDocumentSection(FromDoc1, tFromDoc1Type, ToDoc2, tTODoc2Type)
        End Function
        Public Overridable Function Serialize(ByVal oObjectToSerialize As Object) As String Implements IEnvelopeProvider.Serialize
            Return m_Envelope.Serialize(oObjectToSerialize)
        End Function
        Public Overridable Function DeSerialize(ByVal sXMLStringToDeSerialize As String, ByVal tType As System.Type) As Object Implements IEnvelopeProvider.DeSerialize
            Return m_Envelope.DeSerialize(sXMLStringToDeSerialize, tType)
        End Function
        Public Overridable Function DocumentCount() As Long Implements IEnvelopeProvider.DocumentCount
            Return m_Envelope.DocumentCount
        End Function
        Public Overridable Sub FromXML(ByVal sXML As String) Implements IEnvelopeProvider.FromXML
            init()
            m_Envelope.FromXML(sXML)
        End Sub
        Public Overridable Function GetCustomParam(ByVal sParamNodeName As String, ByVal sParamName As String) As String Implements IEnvelopeProvider.GetCustomParam
            Return m_Envelope.GetCustomParam(sParamNodeName, sParamName)
        End Function
        Public Overridable Function GetCustomParam(ByVal sParamName As String) As String Implements IEnvelopeProvider.GetCustomParam
            Return m_Envelope.GetCustomParam(sParamName)
        End Function
        Public MustOverride Overloads Function AddDocument(ByVal oEnvDoc As IInlineDocument, Optional ByVal bCopyDocument As Boolean = False) As Long Implements IEnvelopeProvider.AddDocument
        Public MustOverride Overloads Function AddDocument(ByVal oEnvDoc As IInlineDocument, ByVal lDocumentIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As Long Implements IEnvelopeProvider.AddDocument
        Public MustOverride Overloads Function GetDocument(ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelopeProvider.GetDocument
        Public MustOverride Overloads Function GetDocument(ByVal sDocumentName As String, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelopeProvider.GetDocument
        Public MustOverride Overloads Function GetDocument(ByVal sDocumentName As String, ByVal lIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelopeProvider.GetDocument
        Public MustOverride Overloads Function GetDocument(ByVal sDocumentName As String, ByVal lIndex As Long, ByRef lReturnedDocIndex As Long, Optional ByVal bCopyDocument As Boolean = False) As IInlineDocument Implements IEnvelopeProvider.GetDocument


        Public MustOverride Function AddAttachment(Optional ByVal sAttachmentName As String = "", Optional ByVal sAttachmentPath As String = "", Optional ByVal lIndex As Long = 0, Optional ByVal sType As String = "", Optional ByVal sUrl As String = "") As Long Implements IEnvelopeProvider.AddAttachment
        Public MustOverride Function GetAttachment(ByVal lIndex As Long) As IAttachment Implements IEnvelopeProvider.GetAttachment
        Public MustOverride Function GetAttachments() As System.Collections.Generic.IList(Of IAttachment) Implements IEnvelopeProvider.GetAttachments
        Public MustOverride Function GetDocuments() As System.Collections.Generic.IList(Of IInlineDocument) Implements IEnvelopeProvider.GetDocuments
        Public MustOverride Function GetCustomParams() As IList Implements IEnvelopeProvider.GetCustomParams
        Public MustOverride Sub ClearCustomParams() Implements IEnvelopeProvider.ClearCustomParams

        Public MustOverride Overloads Function IsError(ByVal sXML As String) As Boolean Implements IEnvelopeProvider.IsError
        Public MustOverride Overloads Function IsWarning(ByVal sXML As String) As Boolean Implements IEnvelopeProvider.IsWarning
        Public MustOverride Overloads Function GetMsgStatus(ByVal sXML As String) As eEnvelopeStatus Implements IEnvelopeProvider.GetMsgStatus
        Public Overridable Overloads Sub SetCustomParam(ByVal sParamNodeName As String, ByVal sParamName As String, ByVal sParamValue As String) Implements IEnvelopeProvider.SetCustomParam
            m_Envelope.SetCustomParam(sParamNodeName, sParamName, sParamValue)
        End Sub
        Public Overridable Overloads Sub SetCustomParam(ByVal sParamName As String, ByVal sParamValue As String) Implements IEnvelopeProvider.SetCustomParam
            m_Envelope.SetCustomParam(sParamName, sParamValue)
        End Sub
        Public Overrides Function ToString() As String Implements IEnvelopeProvider.ToString
            Return m_Envelope.ToString()
        End Function
        Public Overridable Function ToXML() As String Implements IEnvelopeProvider.ToXML
            Return m_Envelope.ToXML()
        End Function
        Public Overridable Function ToXMLDocument() As XmlDocument Implements IEnvelopeProvider.ToXMLDocument
            Return m_Envelope.ToXMLDocument()
        End Function
        Public Overridable Function ToStream() As System.IO.Stream Implements IEnvelopeProvider.ToStream
            Dim oStream As New MemoryStream
            'Dim inputByteArray() As Byte = Encoding.Unicode.GetBytes(ToXML())
            Dim inputByteArray() As Byte = Encoding.UTF8.GetBytes(ToXML())

            oStream.Write(inputByteArray, 0, inputByteArray.Length)
            oStream.Position = 0

            Return oStream
        End Function
        Public Overridable Function ToESBEnvelope() As IEnvelope Implements IEnvelopeProvider.ToESBEnvelope   'This is identical to the Clone interface/method...Included for completeness...
            Return Me.Clone()
        End Function

        ''-----------------------------------------------------------------------------
        '' <summary>
        '' This method returns the list of registered envelopes and sets the default envelope type to 
        '' the first envelope in the list.
        '' </summary>
        '' <returns></returns>
        '' <remarks>This method should be changed to read the envelope types from a configuration file.</remarks>
        '' <history>
        '' 	[MCasiou] 	29/03/2004	Created
        '' </history>
        ''-----------------------------------------------------------------------------
        'From Hard-coded
        'Public Overridable Sub GetRegisteredEnvelopeTypes() 'As System.Collections.Specialized.StringCollection
        '    'Envelope Detection - new interface??? Not for now...
        '    If m_bRegisteredEnvelopeTypesInit = False Then
        '        'Dim sc As New System.Collections.Specialized.StringCollection
        '        'need the dll name as well...for external envelopes...


        '        '++Load from Config
        '        m_oRegisteredEnvelopes.m_scFullClassNamespace.Add("ESB.Core.XMLSchemaClasses.ESBEnvelope.ESBEnvelopeProvider")
        '        m_oRegisteredEnvelopes.m_scAssemblyName.Add("")

        '        m_oRegisteredEnvelopes.m_scFullClassNamespace.Add("ESB.Core.XMLSchemaClasses.JSKEnvelope.JSKEnvelope")
        '        m_oRegisteredEnvelopes.m_scAssemblyName.Add("")
        '        '--Load from Config
        '        'GetConfigData()



        '        m_bRegisteredEnvelopeTypesInit = True

        '        'If it has been set to a particlar type explicitely
        '        'by the client, it would have been set already.
        '        If m_sDefaultEnvelopeType = "" Then
        '            m_sDefaultEnvelopeType = m_oRegisteredEnvelopes.m_scFullClassNamespace(0) 'set the default namespace
        '        End If
        '    End If
        'End Sub
        'From Config files...
        Public Overridable Sub GetRegisteredEnvelopeTypes() 'As System.Collections.Specialized.StringCollection
            'Envelope Detection - new interface??? Not for now...
            If m_bRegisteredEnvelopeTypesInit = False Then
                'Dim sc As New System.Collections.Specialized.StringCollection
                'need the dll name as well...for external envelopes...


                ''++Load from Config
                'm_oRegisteredEnvelopes.m_scFullClassNamespace.Add("ESB.Core.XMLSchemaClasses.ESBEnvelope.ESBEnvelopeProvider")
                'm_oRegisteredEnvelopes.m_scAssemblyName.Add("")

                'm_oRegisteredEnvelopes.m_scFullClassNamespace.Add("ESB.Core.XMLSchemaClasses.JSKEnvelope.JSKEnvelope")
                'm_oRegisteredEnvelopes.m_scAssemblyName.Add("")
                ''--Load from Config
                GetConfigData()



                m_bRegisteredEnvelopeTypesInit = True

                'If it has been set to a particlar type explicitely
                'by the client, it would have been set already.
                If m_sDefaultEnvelopeType = "" Then
                    'm_sDefaultEnvelopeType = m_oCfgData.Envelopes(0).ObjectName
                    'm_sDefaultEnvelopeAssembly = m_oCfgData.Envelopes(0).AssemblyPath
                    m_sDefaultEnvelopeType = Utils.ExpandRelativePath(m_oCfgData.Envelopes(0).ObjectName)
                    m_sDefaultEnvelopeAssembly = Utils.ExpandRelativePath(m_oCfgData.Envelopes(0).AssemblyPath)
                End If
            End If
        End Sub
        'From Hard coded list
        'Public Overridable Function GetEnvelopeType(Optional ByVal sXML As String = "", Optional ByRef oEnvOut As IEnvelopeProvider = Nothing) As String Implements IEnvelopeProvider.GetEnvelopeType
        '    'iterate through all registered envelopes and ask them for their type etc...???

        '    Dim envType As String

        '    If sXML = "" Then
        '        If m_Envelope Is Nothing Then
        '            LogError(LogDefs.eLogLevel.eLOG_ERROR, "m_Envelope has not been set.!!!")
        '            Throw New Exception("m_Envelope has not been set.!!!")
        '        End If
        '        Return m_Envelope.GetEnvelopeType()
        '    End If

        '    GetRegisteredEnvelopeTypes()

        '    Dim oEnv As IEnvelopeProvider
        '    Dim sEnvObjName As String
        '    Dim sEnvObjAssembly As String
        '    Dim i As Long = 0
        '    For Each sEnvObjName In m_oRegisteredEnvelopes.m_scFullClassNamespace
        '        sEnvObjAssembly = m_oRegisteredEnvelopes.m_scAssemblyName(i)
        '        oEnv = CreateObjectByStringName(sEnvObjName, sEnvObjAssembly)

        '        'Don't use method below as it requires the internal envelopes to 
        '        'be specified using the assembly name as well- yuk....
        '        'oEnv = ESBClassFactory.CreateObjectByStringName(sEnvObjName, sEnvObjAssembly)

        '        If oEnv.IsCorrectEnvelopeType(sXML) Then
        '            'this is the correct envelope
        '            envType = oEnv.GetEnvelopeType()
        '            oEnvOut = oEnv
        '            Exit For
        '        End If
        '        i = i + 1
        '    Next

        '    If oEnvOut Is Nothing Then
        '        Throw New Exception("None of the registered Envelopes Handlers can read this envelope. Check envelope format for Encoding, XML and Envelope errors. XML=: ==>" & sXML & "<==")
        '    End If
        '    Return envType
        'End Function
        'From Config files...
        Public Overridable Function GetEnvelopeType(Optional ByVal sXML As String = "", Optional ByRef oEnvOut As IEnvelopeProvider = Nothing) As String Implements IEnvelopeProvider.GetEnvelopeType
            'iterate through all registered envelopes and ask them for their type etc...???

            Dim envType As String

            If sXML = "" Then
                If m_Envelope Is Nothing Then
                    LogError(LogDefs.eLogLevel.eLOG_ERROR, "m_Envelope has not been set.!!!")
                    Throw New Exception("m_Envelope has not been set.!!!")
                End If
                Return m_Envelope.GetEnvelopeType()
            End If

            GetRegisteredEnvelopeTypes()

            Dim oEnv As IEnvelopeProvider
            'Dim sEnvObjName As CustomConfigurationData.EnvelopeCfg
            'Dim sEnvObjAssembly As String
            Dim i As Long = 0
            For Each oEnvCfg As CustomConfigurationData.EnvelopeCfg In m_oCfgData.Envelopes
                oEnv = ESBClassFactory.CreateObjectByStringName(oEnvCfg.ObjectName, oEnvCfg.AssemblyPath)

                If oEnv.IsCorrectEnvelopeType(sXML) Then
                    'this is the correct envelope
                    envType = oEnv.GetEnvelopeType()
                    oEnvOut = oEnv
                    Exit For
                End If
                i = i + 1
            Next

            If oEnvOut Is Nothing Then
                Throw New Exception("None of the registered Envelopes Handlers can read this envelope. Check envelope format for Encoding, XML and Envelope errors. XML=: ==>" & sXML & "<==")
            End If
            Return envType
        End Function
        Public Overridable Function GetEnvelopeImplementation(ByVal sXML As String) As IEnvelopeProvider
            Dim oEnv As Object

            m_sEnvelopeType = GetEnvelopeType(sXML, oEnv)
            m_Envelope = CType(oEnv, IEnvelopeProvider)
            Return m_Envelope

        End Function
        Public Overridable Function IsCorrectEnvelopeType(ByVal sXML As String) As Boolean Implements IEnvelopeProvider.IsCorrectEnvelopeType
            Return m_Envelope.IsCorrectEnvelopeType(sXML)
        End Function
        Public Overridable Function GetEnvelopeNodeName(Optional ByVal sXML As String = "") As Boolean Implements IEnvelopeProvider.GetEnvelopeNodeName
            Return m_Envelope.GetEnvelopeNodeName(sXML)
        End Function
        Public Overridable Function GetEnvelopeNameSpace(Optional ByVal sXML As String = "") As Boolean Implements IEnvelopeProvider.GetEnvelopeNameSpace
            Return m_Envelope.GetEnvelopeNameSpace(sXML)
        End Function
        Public Overridable Function GetEnvelopeVersion(Optional ByVal sXML As String = "") As Boolean Implements IEnvelopeProvider.GetEnvelopeVersion
            Return m_Envelope.GetEnvelopeVersion(sXML)
        End Function

        'Utility methods
        Public Overridable Function GetXMLAsString(ByVal oDoc As XmlDocument) As String Implements IEnvelopeProvider.GetXMLAsString
            Dim oXMLUtil As New XMLUtil
            Return oXMLUtil.GetXMLAsString(oDoc)
        End Function
        Public Overridable Overloads Function PrettyFormatXML(ByVal oStream As MemoryStream, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String Implements IEnvelopeProvider.PrettyFormatXML
            Dim oXMLUtil As New XMLUtil
            Return oXMLUtil.PrettyFormatXML(oStream, bXMLDOMParseFirst)
        End Function
        Public Overridable Overloads Function PrettyFormatXML(ByVal sXML As String, Optional ByVal bXMLDOMParseFirst As Boolean = True) As String Implements IEnvelopeProvider.PrettyFormatXML
            Dim oXMLUtil As New XMLUtil
            Return oXMLUtil.PrettyFormatXML(sXML, bXMLDOMParseFirst)
        End Function
        Public Overridable Sub FromStream(ByVal oStream As System.IO.Stream) Implements IEnvelopeProvider.FromStream
            Dim sMsg As String
            Dim inputByteArray(oStream.Length - 1) As Byte
            oStream.Read(inputByteArray, 0, oStream.Length)

            Dim encoding As System.Text.Encoding = System.Text.Encoding.UTF8
            sMsg = encoding.UTF8.GetString(inputByteArray)
            FromXML(sMsg)
        End Sub
        Public Overridable Sub FromXMLDocument(ByVal oXML As XmlDocument) Implements IEnvelopeProvider.FromXMLDocument
            m_Envelope.FromXMLDocument(oXML)
        End Sub

        Public MustOverride Overloads Function IsError() As Boolean Implements IEnvelopeProvider.IsError
        Public MustOverride Overloads Function IsWarning() As Boolean Implements IEnvelopeProvider.IsWarning
        Public MustOverride Overloads Function GetMsgStatus() As eEnvelopeStatus Implements IEnvelopeProvider.GetMsgStatus

        <System.Xml.Serialization.XmlIgnore()> _
        Public MustOverride ReadOnly Property AttachmentsExist() As Boolean Implements IEnvelopeProvider.AttachmentsExist

#End Region

#Region "EnvelopeConfig"

        Private m_oCfgData As CustomConfigurationData.CustomConfigurationData


        Protected Overridable Sub GetConfigData()
            Dim m_sSourceSection As String = "ESB.Core.Envelopes"
            m_oCfgData = CType(ConfigurationManager.Read(m_sSourceSection), CustomConfigurationData.CustomConfigurationData)
            If m_oCfgData Is Nothing Then
                Throw New ApplicationException("no config on provider.")
            End If
        End Sub
#End Region

    End Class


    '<Serializable()> _
    'Public Class RegisteredEnvelopes
    '    Public m_scFullClassNamespace As New System.Collections.Specialized.StringCollection
    '    Public m_scAssemblyName As New System.Collections.Specialized.StringCollection
    'End Class
End Namespace

