Option Strict Off

'''-----------------------------------------------------------------------------
''' <summary>
''' Copyright Keystroke IT 2003
''' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
''' </summary>
''' <remarks>
''' This ServiceSample class shows some basic ways your service can interact with
''' the ESB.NET runtime. This sample code is derived from some of the ESB.NET System Test Services.
''' 
''' Option Strict - notes and coding tips
''' --------------------------------------
''' Note that you may wish to turn Option Strict On. 
''' If so, it will impact readability of generated code somewhat
''' by necessitating the use of CType/DirectCast and integer to string type conversions etc.
''' As such, it has been left off by default.
''' 
''' It has no impact whatsoever and my personal recommendation is as follows:
''' 1)Start the initial coding pass with Option Strict Off  - this will help get you finished quicker 
''' and understand the code without too much clutter getting in the way, disturbing your line of thought.
''' 
''' 2)Turn Option Strict On after the first coding pass to ensure you're not caught out by silly compiler errors
''' 
''' There are a couple of bits of code in this class that will initially not work with Option Strict on.
''' The commented code below the compiler error shows you what to change in order to get your code to compile
''' with Option Strict On. 
''' 
''' i.e. just comment out the errored line of code, and uncomment the line below it...
''' 
''' This file has the option turned off here. The project-wide setting is set to Option Strict On.
''' 
''' ps. This approach ensures you take a second pass at your code and often helps pick up obvious problems.
''' ''' </remarks>
'''-----------------------------------------------------------------------------

Public Class ServiceSample
    Inherits ServiceBase

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="oEnv"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope
        StartMethod()

        Try
            If GetContext.ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then
                If ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then : LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.MessageID) : End If
            End If
            If GetContext.ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then
                If ShouldLog(LogDefs.eLogLevel.eLOG_TRACE) Then : LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.XML) : End If
            End If


            'Dim oEnvRs As ESBEnvelope.Envelope = oEnv.CreateResponse()
            Dim oEnvRs As ESBEnvelope.Envelope = DirectCast(oEnv.CreateResponse(), ESBEnvelope.Envelope)

            oEnvRs.SystemName = "Request Succeeded"
            oEnvRs.Environment = oEnvRs.Environment & "Some Env Appended"
            Dim oErr As New ErrorXML

            If oEnv.Environment = "Warning" Then
                Dim oWarningEntry As New WarningEntry
                With oWarningEntry
                    .Code = "testcode"
                    .Description = "if you get this message then the test txn worked."
                    .Description = .Description & "ESBSoapRpcUrlkey=" & System.Configuration.ConfigurationManager.AppSettings("ESBSoapRpcUrlkey")
                    .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                End With
                oErr.WarningEntry.Add(oWarningEntry)
            ElseIf oEnv.Environment = "Error" Then
                Dim oErrorEntry As New ErrorEntry
                With oErrorEntry
                    .Code = "testcode"
                    .Description = "if you get this message then the test txn worked."
                    .Description = .Description & "ESBSoapRpcUrlkey=" & System.Configuration.ConfigurationManager.AppSettings("ESBSoapRpcUrlkey")
                    .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                End With
                oErr.ErrorEntry.Add(oErrorEntry)
            ElseIf oEnv.Environment = "ErrorAndWarning" Then
                Dim oWarningEntry As New WarningEntry
                With oWarningEntry
                    .Code = "testcode"
                    .Description = "if you get this message then the test txn worked."
                    .Description = .Description & "ESBSoapRpcUrlkey=" & System.Configuration.ConfigurationManager.AppSettings("ESBSoapRpcUrlkey")
                    .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                End With
                oErr.WarningEntry.Add(oWarningEntry)
                Dim oErrorEntry As New ErrorEntry
                With oErrorEntry
                    .Code = "testcode"
                    .Description = "if you get this message then the test txn worked."
                    .Description = .Description & "ESBSoapRpcUrlkey=" & System.Configuration.ConfigurationManager.AppSettings("ESBSoapRpcUrlkey")
                    .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                End With
                oErr.ErrorEntry.Add(oErrorEntry)
            End If

            If oEnv.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "XsltTest") = "1" Then
                Try
                    'do some xslt testing...
                    Dim sResolveName As String = "TestResolveName"
                    Dim sContext As String = "TestContext"
                    Dim sXmlToTransform As String = oEnv.GetDocument(0).GetAsString()
                    Dim sTransformResult As String = ""

                    GetContext.Transform(sResolveName, sContext, sXmlToTransform, sTransformResult)

                    If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(eLogLevel.eLOG_DATAXFER, "sTransformResult=" & sTransformResult) : End If

                    oEnvRs.AddDocument(New InlineDocument(sTransformResult))

                Catch ex As Exception
                    ExceptionManager.Publish(ex)
                    LogError(eLogLevel.eLOG_ERROR, ex.ToString)
                End Try

            End If

            If oEnv.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "ConfigTest") = "1" Then
                Try
                    'do some Config Application Block testing...
                    Dim sTmp As String

                    GetConfigData()
                    sTmp = m_oCfgData.ServiceConfigData

                    Dim oWarningEntry As New WarningEntry
                    With oWarningEntry
                        .Code = "testcode"
                        .Description = "if you get this message then the test txn worked."
                        .Description = .Description & "ConfigTest Value=" & sTmp
                        .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                    End With
                    oErr.WarningEntry.Add(oWarningEntry)

                    If ShouldLog(LogDefs.eLogLevel.eLOG_DATAXFER) Then : LogMsg(eLogLevel.eLOG_DATAXFER, "sTmp=" & sTmp) : End If


                Catch ex As Exception
                    ExceptionManager.Publish(ex)
                    LogError(eLogLevel.eLOG_ERROR, ex.ToString)
                End Try

            End If


            If oEnv.Context = "TestAddToPipelineTransitionData" Then
                Dim sPTD As String = oEnv.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "TestAddToPipelineTransitionData")
                Try
                    Dim oWarningEntry As New WarningEntry
                    With oWarningEntry
                        .Code = sPTD
                        .Description = "PipelineTransitionData instance " & sPTD
                        .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                    End With
                    oErr.WarningEntry.Add(oWarningEntry)

                Catch ex As Exception
                    ExceptionManager.Publish(ex)
                    LogError(eLogLevel.eLOG_ERROR, ex.ToString)
                End Try

                If oEnv.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "TestAddToPipelineTransitionData") = "3" Then
                    Try
                        Dim sIndex As String = oEnv.GetCustomParam(ESBConstants.C_sCustomParamESBCore, "PipelineTransitionDataRSIndex")
                        Dim oWarningEntry As New WarningEntry
                        With oWarningEntry
                            .Code = sPTD
                            .Description = "output of PipelineItem " & sIndex & " (handler) was" & GetContext.PipelineTransitionData.HandlerPipeline.m_oVertical.m_oRsEnvelopesCol(sIndex).XML
                            '.Description = "output of PipelineItem " & sIndex & " (handler) was" & GetContext.PipelineTransitionData.HandlerPipeline.m_oVertical.m_oRsEnvelopesCol(Integer.Parse(sIndex)).XML
                            .Source = System.Reflection.Assembly.GetExecutingAssembly.ToString
                        End With
                        oErr.WarningEntry.Add(oWarningEntry)
                    Catch ex As Exception
                        ExceptionManager.Publish(ex)
                        LogError(eLogLevel.eLOG_ERROR, ex.ToString)
                    End Try
                End If
            End If

            oEnvRs.AddDocument(New InlineDocument(CType(oErr, Object)))
            Return oEnvRs
        Catch ex As Exception
            LogError(eLogLevel.eLOG_ERROR, ex.ToString)
            ExceptionManager.Publish(ex)

            Dim oErrEntry As New ErrorEntry
            oErrEntry.Description = ex.ToString
            oErrEntry.Source = ex.Source
            m_oErr.ErrorEntry.Add(oErrEntry)

            m_oEnvRs.AddDocument(New InlineDocument(m_oErr))

            Return m_oEnvRs
        Finally
            EndMethod()

        End Try

    End Function

End Class

'Configure this class name 
'i.e. "ESB.Adaptors.Application.ServiceSample.ServicedComponent"
'in the ESB.NET Service Pipeline if this service is to be hosted by COM+ rather than directly by the .NET Runtime
<EventTrackingEnabled(True), Transaction(TransactionOption.Supported), System.Runtime.InteropServices.ProgId("ESB.Adaptors.Application.ServiceSample.ServicedComponent")> _
Public Class ServiceSampleServicedComponent
    Inherits ESBServicedComponent

    Public Overrides Function ProcessMsgSync(ByVal oEnvRq As IEnvelope) As IEnvelope
        Dim o As New RetrieveAccountPortfolio
        Return o.ProcessMsgSync(oEnvRq)
    End Function
End Class