Imports ESB.Core.Services
Imports ESB.Core.Interfaces.ServerContext
Imports Microsoft.ApplicationBlocks.ConfigurationManagement

Public Class ServiceBase
    Inherits ESBComponent

    Protected m_oCfgData As CustomConfigurationData

    Protected m_oEnvRq As IEnvelope
    Protected m_oEnvRs As IEnvelope

    Protected m_oErr As ErrorXML
    Protected m_oErrEntry As ErrorEntry
    Protected m_oWarningEntry As WarningEntry

    Public Sub New()
        m_oErr = New ErrorXML
        m_oErrEntry = New ErrorEntry
        m_oWarningEntry = New WarningEntry
    End Sub

    'Feel free to change this base class as you require
    Public Overrides Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope

        StartMethod()
        Try
            'Load up any Adaptor (usually for the VS.NET project wide) Specific Configuration data
            'Note that you need to define and setup the configuration before enabling this
            'Unlike Custom Params set in the Pipeline Config file which apply to individual ESB.NET services 
            '(.NET classes + ESB.NET Service Pipeline Configuration combination)
            'this is static inter-service configuration that applies to the whole adaptor 
            '(i.e. available & static amongst all classes in this VS.NET Project)
            'GetConfigData()

            'Keep a class wide reference to the request message
            'so that the derived class' methods also have it available
            m_oEnvRq = oEnv

            'Create a response envelope using the request envelope as a starting point. This preserves routing information etc.
            'but strips out the original documents, attachments & relevant manifest entries.
            m_oEnvRs = m_oEnvRq.CreateResponse()

            'invoke the derived class' ProcessMsgSync method
            'Return ProcessMsgSync(m_oEnvRq)
            Return m_oEnvRq


        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString())

            'return a more structured error to the PipelineManager
            Dim oErrEntry As New ErrorEntry
            oErrEntry.Description = ex.ToString
            oErrEntry.Source = ex.Source
            m_oErr.ErrorEntry.Add(oErrEntry)
            m_oEnvRs.AddDocument(New InlineDocument(m_oErr))

            'Decide whether to continue processing the pipeline in case of error..
            'Default behaviour set by "ContinueProcessingPipelineOnAppAdaptorProcessingError" flag in web.Config for particular transport adaptor

            'if the above flag is set to true, and you're writing an Authorization Service for example, then 
            'you can uncomment the line below to ensure the pipeline processing stops after this class is invoked.
            'GetContext.continueProcessingPipeline = False

            Return m_oEnvRs

        Finally
            EndMethod()
        End Try

    End Function

    'Not usually used for Asynchronous processing 
    'ProcessMsgSync is typically also invoked for Asynchronous processing 
    'Ignore this method
    Public Overrides Sub ProcessMsgAsync(ByVal oEnv As IEnvelope)
        StartMethod()

        LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.MessageID)
        LogMsg(LogDefs.eLogLevel.eLOG_TRACE, oEnv.XML)

        EndMethod()
    End Sub
    'StartMethod() and 'EndMethod() only logged in Debug builds by default for performance reasons
    'Override this behaviour by uncommenting the code below if desired, and toggling the comment on the 
    ' <Conditional("DEBUG")> _ statement.

    ''    <Conditional("DEBUG")> _
    'Public Shared Sub StartMethod()
    '    ContextManager.getContextManager.GetExistingContext().Log(LogDefs.eLogLevel.eLOG_TRACE, "StartMethod")
    'End Sub

    ''   <Conditional("DEBUG")> _
    'Public Shared Sub EndMethod()
    '    ContextManager.getContextManager.GetExistingContext().Log(LogDefs.eLogLevel.eLOG_TRACE, "EndMethod")
    'End Sub

    Protected Overridable Sub GetConfigData()
        Dim m_sSourceSection As String = "ESB.NET.Services_Demo2"
        m_oCfgData = CType(ConfigurationManager.Read(m_sSourceSection), CustomConfigurationData)
        If m_oCfgData Is Nothing Then
            Throw New ApplicationException("no config on provider.")
        End If
    End Sub

End Class


