'-----------------------------------------------------------------------------
' <summary>
' Copyright Keystroke IT 2003
' <WrittenBy>Minas Casiou, Keystroke IT Australia</WrittenBy>
' </summary>
'-----------------------------------------------------------------------------
Option Explicit On


''' <summary>
''' Synchronously Routes a request (i.e. makes a standard web service call) based upon the first TargetSystem name in the Envelope.
''' Usage: Configure in the ServicesPipeline in the HorizontalPre section.
''' Will forward the request to another ESB Instance as per the Config file, and block waiting for the reqponse.
''' </summary>
''' <remarks>Assumes the ESBEnvelopeProvider</remarks>

<ESB.Core.Util.Attributes.Service(), ESB.Core.Util.Attributes.Documents("", "")> _
Public Class BasicLookupRoutingServiceHandler
    Inherits ServiceBase

    Dim m_oDocRq As Object 'Todo: replace 'Object' with your schema request class eg.Dim m_oDocRq As MemberEnquiryRQ.MemberDetailsEnquiryRQ
    Dim m_oDocRs As Object 'Todo: replace 'Object' with your schema response class eg. Dim m_oDocRs As MemberEnqRSUpdRQ.MemberEnqRSUpdRQ 

    'Service Entry Point 
    Public Overrides Function ProcessMsgSync(ByVal oEnv As IEnvelope) As IEnvelope
        StartMethod()

        Try
            MyBase.ProcessMsgSync(oEnv)


            DoWork()

            m_oEnvRs.AddDocument(New InlineDocument("<Router>Routed Document</Router>"))


            Return m_oEnvRs

        Catch ex As Exception
            ExceptionManager.Publish(ex)
            LogError(LogDefs.eLogLevel.eLOG_ERROR, ex.ToString())
            Throw

        Finally
            EndMethod()
        End Try
    End Function

    Protected Sub DoWork()
        Dim oMap As RequestRemapping_BasicLookup.RequestReMaps = Nothing

        Dim oEsbEnvProv As ESB.Core.XMLSchemaClasses.ESBEnvelope.ESBEnvelopeProvider
        oEsbEnvProv = m_oEnvRq.GetInternalEnvelope

        Dim sRouteToHost As String = ""
        Dim sRouteToInstance As String = ""


        Dim ex As Exception = Nothing
        If Not RequestRemapping_BasicLookup.RequestReMaps.LoadFromFile(Util.ExpandRelativePath("LobSystems\ns\au.com.keystroke.services.serviceRouting\Config\au.com.keystroke.services.serviceRouting.config"), oMap, ex) Then
            Throw ex
        End If

        For Each oMapEntry As RequestRemapping_BasicLookup.RequestReMapsReMapEntry In oMap.ReMapEntry
            If oMapEntry.LookupKeyValue.ToLower = oEsbEnvProv.header.targetSystems(0).systemID.ToLower Then
                sRouteToHost = oMapEntry.Route_ToHost
                sRouteToInstance = oMapEntry.Route_ToESBInstance

                'Recursive Routing Protection... CHECK
                If m_oEnvRq.GetCustomParam("Router_FromInstance") = m_oEnvRq.ToAddress Then
                    'Do NOT Route
                    Throw New Exception("Router_FromInstance = " & m_oEnvRq.ToAddress)
                End If

            End If
        Next

        '' AddressType
        '    Case "SOAP"
        'oTxAdaptor = New Wcf.ESBSoapXmlDocTransportWcfSender

        '    Case "SOAP-WSE3"
        'oTxAdaptor = New ESBSoapXmlDocTransportWse3Sender

        '    Case "HTTP"
        'oTxAdaptor = New HTTPSender

        '    Case "MSMQ"
        'oTxAdaptor = New MSMQSender

        '    Case "MSSQL"
        'oTxAdaptor = New MSSQLSender

        ''ClientEndpointName
        'Default
        'Reliable 

        Dim sAddressType As String = "HTTP"
        Dim sEndpointName As String = "Default" 'Only applies to WCF AddressType - WCF Endpoint Name...

        Dim oRhp As New ESB.Core.Adaptors.Transport.RequestHandlerProxy.Sender(ConstructUrl(sRouteToHost, sRouteToInstance), sAddressType, sEndpointName)

        'Recursive Routing Protection... SET
        m_oEnvRq.SetCustomParam("Router_FromInstance", m_oEnvRq.ToAddress)
        'm_oEnvRq.FromAddress = m_oEnvRq.ToAddress

        Try
            '++This will send the request to the target ESB Instance.
            m_oEnvRs = oRhp.ProcessMsgSync(m_oEnvRq)
            '--This will send the request to the target ESB Instance.

        Catch ex2 As Exception

            Throw New Exception("Error trying to Route request to: sRouteToHost=" & sRouteToHost & ": sRouteToInstance=" & sRouteToInstance & ": sAddressType=" & sAddressType & ": sEndpointName=" & sEndpointName)

        End Try
    End Sub

    Public Function ConstructUrl(ByVal sHost As String, ByVal sESBInstance As String, Optional ByVal sAddressType As String = "HTTP", Optional ByVal sEndpointName As String = "Default")
        'SOAP (WCF) - http://localhost/ESB/Source/CoreInternetTransportAdaptors/Wcf/ESBSoapXmlDocTransport.svc/Default
        'SOAP-WSE3  - http://localhost/ESB/Source/CoreInternetTransportAdaptors/Wse3/ESBSoapXmlDocTransport.asmx
        'HTTP       - http://localhost/ESB/Source/CoreInternetTransportAdaptors/Wse3/ESBHttpTransport.aspx

        Select Case sAddressType.ToUpper
            Case "HTTP"
                Return "http://" & sHost & "/ESB/" & sESBInstance & "/CoreInternetTransportAdaptors/Wse3/ESBHttpTransport.aspx"

            Case "SOAP-WSE3"
                Return "http://" & sHost & "/ESB/" & sESBInstance & "/CoreInternetTransportAdaptors/Wse3/ESBSoapXmlDocTransport.asmx"

            Case "SOAP"
                Return "http://" & sHost & "/ESB/" & sESBInstance & "/CoreInternetTransportAdaptors/Wcf/ESBSoapXmlDocTransport.svc/" & sEndpointName
        End Select

        Throw New Exception("Could not construct request for given parameters:" & sHost & ":" & sESBInstance & ":" & sAddressType & ":" & sEndpointName)

    End Function
End Class

'Dim xeAccounts As XDocument = XDocument.Parse(sXml)
''Dim xeAccounts As XElement = XElement.Parse(sXml)
'        For Each xe As XElement In xeAccounts.<root>.<AccountsToLinked>
''MsgBox(xe.<Id>.Value)
'            cmbToAccounts.Items.Add(xe.<Id>.Value & "***" & xe.<AccountNumber>.Value)
'        Next
